////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "ButtonEx.h"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__BUTTONEX__
#define __FILE__BUTTONEX__

#include <Windows.h>

#ifndef _WTL_VER

#ifdef _INC_WINDOWSX
   #undef SubclassWindow // protect template members from windowsx.h macros
#endif //_INC_WINDOWSX

class CButtonEx {
public:
   HWND m_hWnd;
protected:
   WNDPROC m_pfnSuperWindowProc;
   bool    m_fMouseOver;
   bool    m_bBoldFontWhenMouseOver;
   bool    m_bDrawingFocus;
public:
   CButtonEx(HWND hWndBttn = NULL);
  ~CButtonEx() {
      if (m_hWnd) {
         UnsubclassWindow(TRUE);
      }
   }

   BOOL SubclassWindow(HWND);
   HWND UnsubclassWindow(BOOL bForce = FALSE);
   HWND Detach() { HWND hWnd = m_hWnd; m_hWnd = NULL; return hWnd; }

   bool SetBoldFontWhenMouseOver(bool bVal) { bool b = m_bBoldFontWhenMouseOver; m_bBoldFontWhenMouseOver = bVal; return b; }
   bool SetDrawingFocus(bool bVal) { bool b = m_bDrawingFocus; m_bDrawingFocus = bVal; return b; }

protected:
   static LRESULT CALLBACK WndProcThis(HWND, UINT, WPARAM, LPARAM);
          LRESULT          WndProc    (HWND, UINT, WPARAM, LPARAM);

   virtual BOOL DrawCustom(HDC, const RECT&) { return FALSE; }
   virtual void DrawBkgrnd(HDC, const RECT&);
   void    DoPaint(HDC);
   void    OnMouseMove(int, int, UINT);
   LRESULT OnMouseLeave();
   BOOL    StartTrackMouseLeave();
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class TBase = CButton, class TWinTraits = CControlWinTraits>
class ATL_NO_VTABLE CButtonExImpl : public CWindowImpl< T, TBase, TWinTraits> {
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

protected:
   // Internal states
   bool m_fMouseOver;
   bool m_bBoldFontWhenMouseOver;
   bool m_bDrawingFocus;

public:
// Constructor/Destructor
   CButtonExImpl(HWND hWnd = NULL) : m_fMouseOver(false), m_bBoldFontWhenMouseOver(true), m_bDrawingFocus(true) {}
  ~CButtonExImpl() {}

protected:
// Overrideables
   void DoPaint(CDCHandle);

// Message map and handlers
   typedef CButtonExImpl< T, TBase, TWinTraits >   thisClass;
   BEGIN_MSG_MAP(thisClass)
      MESSAGE_HANDLER(WM_ERASEBKGND    , OnEraseBkgnd)
      MESSAGE_HANDLER(WM_PAINT         , OnPaint)
      MESSAGE_HANDLER(WM_PRINTCLIENT   , OnPaint)
      MESSAGE_HANDLER(WM_MOUSEMOVE     , OnMouseMove)
      MESSAGE_HANDLER(WM_MOUSELEAVE    , OnMouseLeave)

      MESSAGE_HANDLER(WM_SETFOCUS      , OnDrawAgain)
      MESSAGE_HANDLER(WM_KILLFOCUS     , OnDrawAgain)
      MESSAGE_HANDLER(WM_CAPTURECHANGED, OnDrawAgain)
      MESSAGE_HANDLER(WM_ENABLE        , OnDrawAgain)
      MESSAGE_HANDLER(BM_SETSTATE      , OnDrawAgain)
      MESSAGE_HANDLER(BM_SETCHECK      , OnDrawAgain)
      MESSAGE_HANDLER(WM_SETTEXT       , OnDrawAgain)
      MESSAGE_HANDLER(WM_UPDATEUISTATE , OnDrawAgain)
      if (uMsg == WM_DESTROY) {
         m_fMouseOver = false;
      }
   END_MSG_MAP()

   virtual BOOL DrawCustom(CDC&, const RECT&) { return FALSE; }
   virtual void DrawBkgrnd(CDC&, const RECT&);

   LRESULT OnEraseBkgnd(UINT, WPARAM, LPARAM, BOOL&);
   LRESULT OnPaint     (UINT, WPARAM, LPARAM, BOOL&);
   LRESULT OnDrawAgain (UINT, WPARAM, LPARAM, BOOL&);
   LRESULT OnMouseMove (UINT, WPARAM, LPARAM, BOOL&);
   LRESULT OnMouseLeave(UINT, WPARAM, LPARAM, BOOL&);

   BOOL StartTrackMouseLeave();
};

class CButtonEx : public CButtonExImpl<CButtonEx> {
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_ButtonEx_from_KSerg"), GetWndClassName())
   CButtonEx(HWND hWnd = NULL) : CButtonExImpl<CButtonEx>(hWnd) {}
};

#include "ButtonEx.hpp"

#endif // _WTL_VER

#endif // __FILE__BUTTONEX__
