////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "ButtonEx.hpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifdef _WTL_VER

#include <TmSchema.h>
#include "../DynLoadLib.h"

template <class T, class TBase, class TWinTraits>
void CButtonExImpl<T, TBase, TWinTraits>::DrawBkgrnd(CDC &hCDC, const RECT &rect) {
   HBRUSH hBrushNew = ::CreateSolidBrush(::GetSysColor(COLOR_BTNFACE));
   HBRUSH hBrushOld = hCDC.SelectBrush(hBrushNew);
   hCDC.PatBlt(rect.left,rect.top, rect.right-rect.left, rect.bottom-rect.top, PATCOPY);
   hCDC.SelectBrush(hBrushOld);
   ::DeleteObject(hBrushNew);
}

template <class T, class TBase, class TWinTraits>
void CButtonExImpl<T, TBase, TWinTraits>::DoPaint(CDCHandle dc) {
   const SIZE sizeWin = ::GetWindowSize(m_hWnd);

   CDC hCDC; hCDC.CreateCompatibleDC(dc);
   HBITMAP hBmpNew  = ::CreateBitmap(sizeWin);
   HBITMAP hBmpOld  = hCDC.SelectBitmap(hBmpNew);

   DWORD dwStyle = GetStyle();
   int iState = Button_GetState(m_hWnd); // defined in <WindowsX.h>

#ifdef _DEBUG
   if (!(((dwStyle & BS_TYPEMASK)==BS_PUSHBUTTON   ) ||
         ((dwStyle & BS_TYPEMASK)==BS_DEFPUSHBUTTON) ||
          (dwStyle & BS_PUSHLIKE))
      //||(dwStyle & BS_FLAT)
      )
   {
      _ASSERT(("  !", 0));
   }
#endif

   BOOL bChecked = !!(iState & BST_CHECKED);
   BOOL bPushed  = !!(iState & BST_PUSHED);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  State [%s];  Style=[%s] "), (LPCTSTR)CLogger::ButtonState(iState), (LPCTSTR)CLogger::WindowStyle(m_hWnd));

   RECTEX rect = ::GetClientRect(m_hWnd);
   CWinXPTheme::PF_OpenThemeData pfOpenThemeData = CWinXPTheme::OpenThemeData();
   HTHEME hTheme = pfOpenThemeData ? pfOpenThemeData(m_hWnd, L"button") : NULL;
   int iStateId = 0; // 0 or PBS_NORMAL ()

   // 1. Draw border
   if (!hTheme) {
      BOOL bDown = (bChecked || bPushed || (iState & BST_INDETERMINATE));
      hCDC.DrawFrameControl(&rect, DFC_BUTTON, bDown ? DFCS_PUSHED|DFCS_BUTTONPUSH : DFCS_BUTTONPUSH);
      /** /
      rect = RECTEX(1, 1, sizeWin.cx-1, sizeWin.cy-1);
      /**/
      if (bPushed) {
         rect.left++; rect.top++;
      } else {
         rect.right--; rect.bottom--;
      }
      hCDC.DrawFrameControl(&rect, DFC_BUTTON, bDown ? (DFCS_PUSHED|DFCS_BUTTONPUSH) : DFCS_BUTTONPUSH);
      rect = RECTEX(1+!!bPushed, 1+!!bPushed, sizeWin.cx-1-!bPushed, sizeWin.cy-1-!bPushed);
      /**/
   } else {
      if (bPushed || (bChecked && !m_fMouseOver))
         iStateId = PBS_PRESSED;
      else if (!::IsWindowEnabled(m_hWnd))
         iStateId = PBS_DISABLED;
      else if (m_fMouseOver)
         iStateId = PBS_HOT;
      else if (((dwStyle & BS_TYPEMASK) == BS_DEFPUSHBUTTON  ) ||
             //((dwStyle & BS_TYPEMASK) == BS_PUSHBUTTON     ) ||
               ((dwStyle & BS_TYPEMASK) == BS_AUTOCHECKBOX   ) ||
             //((dwStyle & BS_TYPEMASK) == BS_CHECKBOX       ) ||
             //((dwStyle & BS_TYPEMASK) == BS_AUTO3STATE     ) ||
               ((dwStyle & BS_TYPEMASK) == BS_3STATE         ) ||
             //((dwStyle & BS_TYPEMASK) == BS_RADIOBUTTON    ) ||
               ((dwStyle & BS_TYPEMASK) == BS_AUTORADIOBUTTON))
         iStateId = PBS_DEFAULTED;
      else
         iStateId = PBS_NORMAL;

      CWinXPTheme::PF_DrawThemeBackground pfDrawThemeBackground = CWinXPTheme::DrawThemeBackground();
      if(pfDrawThemeBackground) {
         DrawBkgrnd(hCDC, rect);
         pfDrawThemeBackground(hTheme, hCDC, BP_PUSHBUTTON, iStateId, &rect, 0);
      }
   }

   // 2. Draw background
   if (hTheme) {
      CWinXPTheme::PF_GetThemeBackgroundContentRect pfGetThemeBackgroundContentRect = CWinXPTheme::GetThemeBackgroundContentRect();
      if(pfGetThemeBackgroundContentRect) {
         HRESULT hRes = pfGetThemeBackgroundContentRect(hTheme, hCDC, BP_PUSHBUTTON, iStateId, &rect, &rect);
         if (SUCCEEDED(hRes)) {
            /**/
            {
               RECT rc = rect;
               if ((iStateId == PBS_HOT) || (iStateId == PBS_DEFAULTED)) {
                  rc.left++; rc.top++; rc.right--; rc.bottom--;
               }
               DrawBkgrnd(hCDC, rc);
            }
            /** /
            CWinXPTheme::PF_DrawThemeParentBackground pfDrawThemeParentBackground = CWinXPTheme::DrawThemeParentBackground();
            if (pfDrawThemeParentBackground) {
               RECT rc = rect;
               if ((iStateId == PBS_HOT) || (iStateId == PBS_DEFAULTED)) {
                  rc.left++; rc.top++; rc.right--; rc.bottom--;
               }
               pfDrawThemeParentBackground(m_hWnd, hCDC, &rc);
            }
            /**/
         }
      }
   } else {
      DrawBkgrnd(hCDC, rect);
   }

   // 3. Draw contents
   // 3.1. draw custom
   BOOL bDraw = DrawCustom(hCDC, rect);
   // 3.2. draw bitamp
   if (!bDraw && (dwStyle & BS_BITMAP)) {
      HBITMAP hBmp = (HBITMAP)::SendMessage(m_hWnd, BM_GETIMAGE, (WPARAM)IMAGE_BITMAP, 0L);
      if (hBmp) {
         RECTEX rc(rect);
         //rc = FindInnerRect((RECTEX)::SizeBitmap(hBmp), rect);

         CDC hCDC2; hCDC2.CreateCompatibleDC(hCDC);
         HBITMAP hGObj = hCDC2.SelectBitmap(hBmp);
         hCDC.SetStretchBltMode(COLORONCOLOR);
         if (!m_fMouseOver) {
            rc.left++; rc.top++; rc.right--; rc.bottom--;
         }
         SIZE sizeBMP = SizeBitmap(hBmp);
         bDraw = hCDC.StretchBlt(rc.left, rc.top, rc.height(), rc.width(), hCDC2, 0,0,sizeBMP.cx,sizeBMP.cy, SRCCOPY);
         hCDC2.SelectBitmap(hGObj);

      }
   }
   // 3.3. draw icon
   if (!bDraw && (dwStyle & BS_ICON)) {
      HICON hIcon = (HICON)::SendMessage(m_hWnd, BM_GETIMAGE, (WPARAM)IMAGE_ICON, 0L);
      if (hIcon) {
         RECTEX rc = FindInnerRect((RECTEX)::SizeIcon(hIcon), rect);
         if (!m_fMouseOver) {
            rc.left++; rc.top++; rc.right--; rc.bottom--;
         }
         bDraw = hCDC.DrawIconEx(
            rc.left, rc.top,
            hIcon,
            rc.width(), rc.height(),
            0, NULL, DI_NORMAL);
      }
   }
   // 3.4. draw text
   if (!bDraw) {
      hCDC.SetBkMode(TRANSPARENT);

      HFONT hFontBttn = (HFONT)::SendMessage(m_hWnd, WM_GETFONT, 0, 0);
      if (hFontBttn && (::GetObjectType(hFontBttn) != OBJ_FONT))
         hFontBttn = NULL;
      HFONT hFontOld = NULL;
      HFONT hFontNew = NULL;
      if (m_fMouseOver && m_bBoldFontWhenMouseOver) {
         LOGFONT logFont;
         if (sizeof(LOGFONT) == ::GetObject(hFontBttn, sizeof(LOGFONT), &logFont)) {
            logFont.lfWeight = FW_BOLD;
            hFontNew = ::CreateFontIndirect(&logFont);
            hFontOld = hCDC.SelectFont(hFontNew);
         }
      } else {
         hFontOld = hFontBttn ? hCDC.SelectFont(hFontBttn) : NULL;
      }
      CString strText = ::GetWindowText(m_hWnd);
      if (IsWindowEnabled()) {
         COLORREF clrOld = hCDC.SetTextColor(hTheme ? 0 : ::GetSysColor(COLOR_BTNTEXT));
         RECTEX rc = rect;
         if (!hTheme && bPushed)
            rc.move(1,1);
         bDraw = !!hCDC.DrawText(strText, strText.GetLength(), &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
      } else {
         if (!hTheme) {
            hCDC.SetTextColor(0xFFFFFF);
            bDraw = !!hCDC.DrawText(strText, strText.GetLength(), &RECTEX(rect).move(1,1), DT_CENTER | DT_VCENTER | DT_SINGLELINE);
         }
         hCDC.SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
         bDraw &= !!hCDC.DrawText(strText, strText.GetLength(), &rect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
      }
      if (hFontOld)
         hCDC.SelectFont(hFontOld);
      if (hFontNew)
         ::DeleteObject(hFontNew);
   }
   if (m_bDrawingFocus)
      if (iState & BST_FOCUS) { //if (m_hWnd == ::GetFocus()) { //
         if (!hTheme)
            ::InflateRect(&rect, -::GetSystemMetrics(SM_CXEDGE), -::GetSystemMetrics(SM_CYEDGE));
         hCDC.DrawFocusRect(&rect);
      }

   ::BitBlt(dc, sizeWin, hCDC, SRCCOPY);

   hCDC.SelectBitmap(hBmpOld);
   ::DeleteObject(hBmpNew);

   //::DeleteDC(hCDC); hCDC = NULL;

   CWinXPTheme::PF_CloseThemeData pfCloseThemeData = CWinXPTheme::CloseThemeData();
   if (pfCloseThemeData && hTheme) {
      pfCloseThemeData(hTheme); hTheme = NULL;
   }
}

template <class T, class TBase, class TWinTraits>
LRESULT CButtonExImpl<T, TBase, TWinTraits>::OnEraseBkgnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
   return 1;   // no background needed
}

template <class T, class TBase, class TWinTraits>
LRESULT CButtonExImpl<T, TBase, TWinTraits>::OnPaint(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
   //T* pT = static_cast<T*>(this);
   if (wParam) {
      /*pT->*/DoPaint((HDC)wParam);
   } else {
      /*pT->*/DoPaint(CPaintDC(m_hWnd).m_hDC);
   }
   return 0;
}

template <class T, class TBase, class TWinTraits>
LRESULT CButtonExImpl<T, TBase, TWinTraits>::OnDrawAgain(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
   SendMessage(WM_SETREDRAW, FALSE, 0); //  
   LRESULT lResult = DefWindowProc(uMsg, wParam, lParam);
   SendMessage(WM_SETREDRAW, TRUE, 0); //  
   if (IsWindowVisible()) {
      Invalidate();
      UpdateWindow();
   }
   bHandled = TRUE;
   return lResult;
}

template <class T, class TBase, class TWinTraits>
LRESULT CButtonExImpl<T, TBase, TWinTraits>::OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled)
{
   if (::GetCapture() == m_hWnd) {
      Invalidate();
      UpdateWindow();
   } else
      if(!m_fMouseOver)	{
		   m_fMouseOver = true;
		   Invalidate();
		   UpdateWindow();
		   StartTrackMouseLeave();
	   }
   bHandled = FALSE;
   return 1;
}

template <class T, class TBase, class TWinTraits>
LRESULT CButtonExImpl<T, TBase, TWinTraits>::OnMouseLeave(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
   if (m_fMouseOver) {
      m_fMouseOver = false;
      Invalidate();
      UpdateWindow();
   }
   return 0;
}

template <class T, class TBase, class TWinTraits>
BOOL CButtonExImpl<T, TBase, TWinTraits>::StartTrackMouseLeave()
{
	TRACKMOUSEEVENT tme;
	tme.cbSize = sizeof(tme);
	tme.dwFlags = TME_LEAVE;
	tme.hwndTrack = m_hWnd;
	return _TrackMouseEvent(&tme);
}

#endif // _WTL_VER
