////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "ButtonImage.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "ButtonImage.h"
//#pragma comment(lib, "MsImg32")

#ifndef _WTL_VER

///////////////////////////////////////////////////////////////////////////////
//                         implementation class
///////////////////////////////////////////////////////////////////////////////
CButtonImage::CButtonImage(HWND hWndBttn /*= NULL*/) :
   CButtonEx(hWndBttn)
{
   m_bDrawingFocus = FALSE;
}

CImageSerialize CButtonImage::GetImage(WORD nImage) const
{
   return (nImage<_nImageCount) ? m_Img[nImage].operator CImageSerialize() : CImageSerialize();
}

BOOL CButtonImage::SetImage(const CImageSerialize &img, WORD nImage)
{
   BOOL bRes = FALSE;
   if (nImage<_nImageCount) {
      bRes = (m_Img[nImage] = img).Valid();
      if (bRes && m_hWnd) {
         ::InvalidateRect(m_hWnd, NULL, FALSE);
         ::UpdateWindow(m_hWnd);
      }
   }
   return bRes;
}

BOOL CButtonImage::ImageSetted() const {
   BOOL bRes = FALSE;
   for (int i=0; !bRes && (i<_nImageCount); i++)
      bRes = m_Img[i].Valid();
   return bRes;
}

BOOL CButtonImage::DrawCustom(HDC hCDC, const RECT &rect) {
   BOOL bState = Button_GetState(m_hWnd); // defined in <WindowsX.h>
   BOOL bChecked = !!(bState & BST_CHECKED);
   BOOL bPushed  = !!(bState & BST_PUSHED);
 //BOOL bDown    = (bChecked || bPushed);
   BOOL bDraw = FALSE;
   // draw CImage
   if (!bDraw && ImageSetted()) {
      // -   ...
      WORD iPosImageNormal    = _nImageNormal,
           iPosImagePushed    = _nImagePushed,
           iPosImageMouseOver = _nImageMouseOver,
           iPosImageDisabled  = _nImageDisabled;
      //     ,     
      if (!m_Img[_nImageNormal].Valid()) {
         iPosImageNormal = m_Img[_nImageMouseOver].Valid() ? _nImageMouseOver :
                          (m_Img[_nImagePushed   ].Valid() ? _nImagePushed : _nImageDisabled);
      }
      if (!m_Img[_nImagePushed   ].Valid()) { iPosImagePushed    = iPosImageNormal; }
      if (!m_Img[_nImageMouseOver].Valid()) { iPosImageMouseOver = iPosImageNormal; }
      if (!m_Img[_nImageDisabled ].Valid()) { iPosImageDisabled  = iPosImageNormal; }

      WORD iPos = iPosImageNormal;
      //       
      if (!::IsWindowEnabled(m_hWnd)) {
         iPos = iPosImageDisabled;
      } else {
         if (bChecked) {
            iPos = iPosImagePushed;
         } else {
            iPos = m_fMouseOver ? iPosImageMouseOver : iPosImageNormal;
         }
      }
      RECTEX rc(rect);
      if (!m_fMouseOver) {
         rc.left++; rc.top++; rc.right--; rc.bottom--;
      }
      bDraw = m_Img[iPos].Draw(hCDC, rc);
   }
   return bDraw;
}

#endif // _WTL_VER
