////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "EditEx.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "EditEx.h"

#ifndef _WTL_VER

///////////////////////////////////////////////////////////////////////////////
//                         implementation class
///////////////////////////////////////////////////////////////////////////////
CEditEx::CEditEx(HWND hWndEdit /*= NULL*/) :
   m_hWnd(NULL),
   m_pfnSuperWindowProc(NULL)
{
   if (hWndEdit) {
      SubclassWindow(hWndEdit);
   }
}

BOOL CEditEx::SubclassWindow(HWND hWndEdit) {
   m_hWnd = hWndEdit;
   m_pfnSuperWindowProc = (WNDPROC)::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)CEditEx::WndProcThis);
   if (!m_pfnSuperWindowProc) {
      m_pfnSuperWindowProc = ::DefWindowProc;
   }
   ::SetWindowUserData(m_hWnd, (LONG_PTR)this);
   return !!m_hWnd;
}

HWND CEditEx::UnsubclassWindow(BOOL bForce /*= FALSE*/) {
   WNDCLASSEX wc = {sizeof(WNDCLASSEX), 0, NULL, 0,0, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
   ::GetClassInfoEx(::GetCurrentInstance(), ::GetClassName(m_hWnd), &wc);

   WNDPROC pActiveProc = (WNDPROC)::GetWindowLongPtr(m_hWnd, GWLP_WNDPROC);

   HWND hWnd = NULL;
   if (bForce || (wc.lpfnWndProc != pActiveProc)) {
      if (!::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)m_pfnSuperWindowProc))
         if (!::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)wc.lpfnWndProc))
            return NULL;

      m_pfnSuperWindowProc = ::DefWindowProc;
      hWnd = m_hWnd;
      m_hWnd = NULL;
   }
   return hWnd;
}

/*static*/ LRESULT CALLBACK CEditEx::WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WndProcThis: msg=%s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CEditEx *This = (CEditEx*)::GetWindowUserData(hWnd);
   if (This) {
      return This->WndProc(hWnd, uMsg, wParam, lParam);
   }
   return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CEditEx::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   switch (uMsg){
   //case WM_ERASEBKGND    : return 0;
   case WM_PAINT         : DoPaint(0); return 0;
   case WM_PRINTCLIENT   : DoPaint((HDC)wParam); return 0;
   case WM_SETFOCUS      :
   case WM_KILLFOCUS     :
   case WM_CAPTURECHANGED: 
   case WM_ENABLE        :
   case WM_SETTEXT       :
   case WM_UPDATEUISTATE :
   case WM_CHAR          :
   case WM_KEYDOWN       :
   case WM_KEYUP         :
   case WM_LBUTTONUP     :
   case WM_LBUTTONDOWN   :
   case WM_LBUTTONDBLCLK : return OnDrawAgain(uMsg, wParam, lParam);
   case WM_MOUSEMOVE     : if (wParam & MK_LBUTTON)
                              return OnDrawAgain(uMsg, wParam, lParam);
                           break;
   case WM_DESTROY:
      m_hWnd = NULL;
      break;
   }
   return ::CallWindowProc(m_pfnSuperWindowProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CEditEx::OnDrawAgain(UINT uMsg, WPARAM wParam, LPARAM lParam) {
   LRESULT lResult = ::CallWindowProc(m_pfnSuperWindowProc, m_hWnd, uMsg, wParam, lParam);
   ::InvalidateRect(m_hWnd, NULL, FALSE);
   ::UpdateWindow(m_hWnd);
   return lResult;
}

void CEditEx::DoPaint(HDC hDC) {
   PAINTSTRUCT paint = {0};
   if (!hDC)
      hDC = ::BeginPaint(m_hWnd, &paint);
   RECTEX rc = ::GetClientRect(m_hWnd);
   HDC hCDC1 = ::CreateCompatibleDC(hDC);
   HDC hCDC2 = ::CreateCompatibleDC(hDC);
   HBITMAP hBmpNew1 = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
   HBITMAP hBmpNew2 = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
   HGDIOBJ hBmpOld1 = ::SelectObject(hCDC1, hBmpNew1);
   HGDIOBJ hBmpOld2 = ::SelectObject(hCDC2, hBmpNew2);

   //  
   /**/
   //     (  , ..       :( )
   if (!DrawBkgrnd(hCDC1, rc)) {
      HBRUSH  hBrushNew = ::CreateSolidBrush(GetBkColor());
      HGDIOBJ hBrushOld = ::SelectObject(hCDC1, hBrushNew);

      ::PatBlt(hCDC1, 0,0,rc.width(),rc.height(), PATCOPY);

      ::SelectObject(hCDC1, hBrushOld);
      ::DeleteObject(hBrushNew);
   }/**/
   ::CallWindowProc(m_pfnSuperWindowProc, m_hWnd, WM_PAINT, (WPARAM)hCDC1, 0);

   //  
   if (!DrawBkgrnd(hCDC2, rc)) {
      HBRUSH  hBrushNew = ::CreateSolidBrush(GetBkColor());
      HGDIOBJ hBrushOld = ::SelectObject(hCDC2, hBrushNew);

      ::PatBlt(hCDC2, 0,0,rc.width(),rc.height(), PATCOPY);

      ::SelectObject(hCDC2, hBrushOld);
      ::DeleteObject(hBrushNew);
   }

   //  
 //::TransparentBlt(hCDC2, 0,0, rc.width(), rc.height(), hCDC1, 0,0, rc.width(), rc.height(), GetBkColorOriginal());
   ::BitBltMasked(hCDC2, RECTEX(rc.size()), hCDC1, POINTEX(), GetBkColorOriginal());
   ::BitBlt(hDC, rc, hCDC2, SRCCOPY);

   ::SelectObject(hCDC1, hBmpOld1);
   ::SelectObject(hCDC2, hBmpOld2);
   ::DeleteObject(hBmpNew1);
   ::DeleteObject(hBmpNew2);
   ::DeleteDC(hCDC1);
   ::DeleteDC(hCDC2);
   if (paint.hdc)
      ::EndPaint(m_hWnd, &paint);
}

#endif // _WTL_VER
