////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "EditEx.h"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__EDIT_EX__
#define __FILE__EDIT_EX__

#include <Windows.h>

#ifndef _WTL_VER

#ifdef _INC_WINDOWSX
   #undef SubclassWindow // protect template members from windowsx.h macros
#endif //_INC_WINDOWSX

class CEditEx {
public:
   HWND m_hWnd;
protected:
   WNDPROC m_pfnSuperWindowProc;
public:
   CEditEx(HWND hWndEdit = NULL);
  ~CEditEx() {
      if (m_hWnd) {
         UnsubclassWindow(TRUE);
      }
   }

   BOOL SubclassWindow(HWND);
   HWND UnsubclassWindow(BOOL bForce = FALSE);
   HWND Detach() { HWND hWnd = m_hWnd; m_hWnd = NULL; return hWnd; }

protected:
   static LRESULT CALLBACK WndProcThis(HWND, UINT, WPARAM, LPARAM);
          LRESULT          WndProc    (HWND, UINT, WPARAM, LPARAM);
   virtual BOOL DrawBkgrnd(HDC, const RECT&) { return FALSE; }
   void DoPaint(HDC);
   LRESULT OnDrawAgain(UINT, WPARAM, LPARAM);

   virtual COLORREF GetBkColorDisable () const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetBkColorReadOnly() const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetBkColorEnable  () const { return ::GetSysColor(COLOR_WINDOW ); }

   COLORREF GetBkColor() {
      bool bDisable = !::IsWindowEnabled(m_hWnd);
      bool bReadOnly = !!(::GetWindowStyle(m_hWnd) & ES_READONLY);
      return bDisable  ? GetBkColorDisable() :
             bReadOnly ? GetBkColorReadOnly() :
                         GetBkColorEnable();
   }
   COLORREF GetBkColorOriginal() {
      bool bDisable = !::IsWindowEnabled(m_hWnd);
      bool bReadOnly = !!(::GetWindowStyle(m_hWnd) & ES_READONLY);
      return (bDisable || bReadOnly) ? ::GetSysColor(COLOR_BTNFACE) :
                                       ::GetSysColor(COLOR_WINDOW);
   }
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class TBase = CEdit, class TWinTraits = CControlWinTraits>
class ATL_NO_VTABLE CEditExImpl : public CWindowImpl< T, TBase, TWinTraits> {
public:
   DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

protected:

// Message map and handlers
   typedef CEditExImpl< T, TBase, TWinTraits >   thisClass;
   BEGIN_MSG_MAP(thisClass)
      if (uMsg == WM_ERASEBKGND) {
         lResult = 0;
         return TRUE;
      }
      MESSAGE_HANDLER(WM_PAINT         , OnPaint)
      MESSAGE_HANDLER(WM_PRINTCLIENT   , OnPaint)
      MESSAGE_HANDLER(WM_SETFOCUS      , OnDrawAgain)
      MESSAGE_HANDLER(WM_KILLFOCUS     , OnDrawAgain)
      MESSAGE_HANDLER(WM_CAPTURECHANGED, OnDrawAgain)
      MESSAGE_HANDLER(WM_ENABLE        , OnDrawAgain)
      MESSAGE_HANDLER(WM_SETTEXT       , OnDrawAgain)
      MESSAGE_HANDLER(WM_UPDATEUISTATE , OnDrawAgain)
      MESSAGE_HANDLER(WM_CHAR          , OnDrawAgain)
      MESSAGE_HANDLER(WM_KEYDOWN       , OnDrawAgain)
      MESSAGE_HANDLER(WM_KEYUP         , OnDrawAgain)
      MESSAGE_HANDLER(WM_LBUTTONUP     , OnDrawAgain)
      MESSAGE_HANDLER(WM_LBUTTONDOWN   , OnDrawAgain)
      MESSAGE_HANDLER(WM_LBUTTONDBLCLK , OnDrawAgain)
      if ((uMsg == WM_MOUSEMOVE) && (wParam & MK_LBUTTON)) {
         lResult = OnDrawAgain(uMsg, wParam, lParam, bHandled);
         return TRUE;
      }
   END_MSG_MAP()

   void DoPaint(CDCHandle);
   virtual BOOL DrawBkgrnd(CDC&, const RECT&) { return FALSE; }

   LRESULT OnPaint    (UINT, WPARAM, LPARAM, BOOL&);
   LRESULT OnDrawAgain(UINT, WPARAM, LPARAM, BOOL&);

   virtual COLORREF GetBkColorDisable () const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetBkColorReadOnly() const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetBkColorEnable  () const { return ::GetSysColor(COLOR_WINDOW ); }

   COLORREF GetBkColor() {
      bool bDisable = !IsWindowEnabled();
      bool bReadOnly = !!(::GetWindowStyle(m_hWnd) & ES_READONLY);
      return bDisable  ? GetBkColorDisable() :
             bReadOnly ? GetBkColorReadOnly() :
                         GetBkColorEnable();
   }
   COLORREF GetBkColorOriginal() {
      bool bDisable = !IsWindowEnabled();
      bool bReadOnly = !!(::GetWindowStyle(m_hWnd) & ES_READONLY);
      return (bDisable || bReadOnly) ? ::GetSysColor(COLOR_BTNFACE) :
                                       ::GetSysColor(COLOR_WINDOW);
   }
};

class CEditEx : public CEditExImpl<CEditEx> {
public:
   DECLARE_WND_SUPERCLASS(_T("WTL_EditBox_from_KSerg"), GetWndClassName())
   CEditEx() : CEditExImpl<CEditEx>() {}
};

#include "EditEx.hpp"

#endif // _WTL_VER

#endif // __FILE__EDIT_EX__
