// Modeless dialog ala cursor
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#pragma once
#include "../Lib/CommonLib.h"

#pragma comment(lib, "MsImg32") // GradientFill

#ifndef _WTL_VER

class CGradientCursor
{
public:
   HWND m_hWnd;
private:
   HRGN m_hRgn;
   BYTE m_Color[3][3];
	enum {
		ID_TIMER_REDRAW = 123
	};

   static LPCTSTR GetWindowClass() {
      const static TCHAR szWndClass[] = _T("CGradientCursor");
      return szWndClass;
   }

   static LRESULT CALLBACK WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      CGradientCursor *This = NULL;
      if (uMsg == WM_CREATE) {
         CREATESTRUCT *pCS = (CREATESTRUCT*)lParam;
         This = pCS ? (CGradientCursor*)pCS->lpCreateParams : NULL;
         if (This) {
            This->m_hWnd = hWnd;
            ::SetWindowUserData(hWnd, (LONG_PTR)This);
         }            
      } else {
         This = (CGradientCursor*)::GetWindowUserData(hWnd);
      }
      if (This) {
         return This->WndProc(hWnd, uMsg, wParam, lParam);
      }

      return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
   }

   LRESULT WndProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      switch(uMsg){
      HANDLE_MSG(m_hWnd, WM_CREATE       , OnCreate);
      HANDLE_MSG(m_hWnd, WM_MOUSEACTIVATE, OnMouseActivate);
      HANDLE_MSG(m_hWnd, WM_DESTROY      , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_PAINT        , OnPaint);
      HANDLE_MSG(m_hWnd, WM_TIMER        , OnTimer);
      HANDLE_MSG(m_hWnd, WM_NCHITTEST    , OnNCHitTest);
      }
      return ::DefWindowProc(m_hWnd, uMsg, wParam, lParam);
   }

public:
   CGradientCursor() :
      m_hWnd(NULL),
      m_hRgn(NULL)
   {
      for (int i=0; i<3; i++)
         for (int j=0; j<3; j++)
            m_Color[i][j] = 0xFF>>rand(8);

      static WNDCLASSEX wc = {
         sizeof(WNDCLASSEX),                 // UINT        cbSize;
         0,                                  // UINT        style;
         WndProcThis,                        // WNDPROC     lpfnWndProc;
         0,                                  // int         cbClsExtra;
         0,                                  // int         cbWndExtra;
         ::GetModuleHandle(NULL),            // HINSTANCE   hInstance;
         NULL,                               // HICON       hIcon;
         ::LoadCursor(NULL, IDC_ARROW),      // HCURSOR     hCursor;
         NULL,//(HBRUSH)::GetStockObject(BLACK_BRUSH),  // HBRUSH      hbrBackground;
         NULL,                               // LPCTSTR     lpszMenuName;
         GetWindowClass(),                   // LPCTSTR     lpszClassName;
         NULL                                // HICON       hIconSm;
      };
      static ATOM atom = ::RegisterClassEx(&wc);
   }

   BOOL Create(HWND hWndParent, const RECT &rc) {
      m_hWnd = ::CreateWindowEx(
   #if(_WIN32_WINNT >= 0x0500)
         WS_EX_NOACTIVATE | WS_EX_TRANSPARENT,
   #else
         WS_EX_TRANSPARENT,
   #endif
         GetWindowClass(), NULL, WS_POPUP,
         rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,
         hWndParent, NULL, ::GetModuleHandle(NULL), this);
      return !!m_hWnd;
   }

private:
   BOOL OnCreate(HWND, LPCREATESTRUCT lpCreateStruct) {
      lpCreateStruct->cx = 18; lpCreateStruct->cy = 24; //  

      //   '- '
      POINT pArr[] = {{0,0}, {17,17}, {6,17}, {0,23}}; // {{0,0}, {17,17}, {9,17}, {14,28}, {13,29}, {11,29}, {5,18}, {0,23}};
      m_hRgn = ::CreatePolygonRgn(pArr, chDIMOF(pArr), WINDING);
      ::SetWindowRgn(m_hWnd, m_hRgn, TRUE);

      //    
      ::SetTimer(m_hWnd, ID_TIMER_REDRAW, 10, NULL);

      return TRUE;
   }

   void OnDestroy(HWND){
      //  
      ::SetWindowRgn(m_hWnd, NULL, FALSE); //  m_hRgn.   m_hRgn       -
      m_hRgn = NULL;
      m_hWnd = NULL;
   }

   void OnPaint(HWND) {
      PAINTSTRUCT ps;
      HDC hDC = ::BeginPaint(m_hWnd, &ps);

      // gradient fill
      /** /
      if (0) {
         HGDIOBJ hBrOld = ::SelectObject(hDC, ::CreateSolidBrush(RGB(m_Color[0][0], m_Color[0][1], m_Color[0][2])));
         ::PatBlt(hDC, 0, 0, 17, 23, PATCOPY);
         ::DeleteObject(hBrOld);
      } else /**/
      {
         TRIVERTEX vert[] = {
            {
               0,       // LONG    x;
               0,       // LONG    y;
               m_Color[0][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[0][1]<<8,  // COLOR16 Green;
               m_Color[0][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }, {
               17,      // LONG    x;
               17,      // LONG    y;   
               m_Color[1][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[1][1]<<8,  // COLOR16 Green;
               m_Color[1][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }, {
               0,       // LONG    x;
               23,      // LONG    y;   
               m_Color[2][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[2][1]<<8,  // COLOR16 Green;
               m_Color[2][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }
         };
         GRADIENT_TRIANGLE gTri = {0, 1, 2};
         ::GradientFill(hDC, vert, chDIMOF(vert), &gTri, 1, GRADIENT_FILL_TRIANGLE);
      }
      ::EndPaint(m_hWnd, &ps);
   }

   void OnTimer(HWND, UINT nIDEvent) {
      if (nIDEvent == ID_TIMER_REDRAW) {
         DWORD dwTC = ::GetTickCount();
         int i = (dwTC>>0)%3;//0;//
         int j = (dwTC>>2)%3;//rand(2);//
         BYTE &bC = m_Color[i][j];
         bC = ((bC & 1) ? 0x00 : 0x80) | (bC>>1);
         ::InvalidateRect(m_hWnd, &RECTEX(0,0,17,23), FALSE);
      }
   }

   int OnMouseActivate(HWND, HWND hWndTopLevel, UINT codeHitTest, UINT msg) {
      return MA_NOACTIVATE;
   }

   UINT OnNCHitTest(HWND, int x, int y) {
      return HTTRANSPARENT;
   }
};
/*
class CCursor
{
public:
   HWND m_hWnd;
private:
   HRGN m_hRgn;

   static LPCTSTR GetWindowClass() {
      const static TCHAR szWndClass[] = _T("CCursor");
      return szWndClass;
   }

   static LRESULT CALLBACK WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      CCursor *This = NULL;
      if (uMsg == WM_CREATE) {
         CREATESTRUCT *pCS = (CREATESTRUCT*)lParam;
         This = pCS ? (CCursor*)pCS->lpCreateParams : NULL;
         if (This) {
            This->m_hWnd = hWnd;
            ::SetWindowUserData(hWnd, (LONG)This);
         }            
      } else {
         This = (CCursor*)::GetWindowUserData(hWnd);
      }
      if (This) {
         return This->WndProc(hWnd, uMsg, wParam, lParam);
      }

      return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
   }

   LRESULT WndProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      switch(uMsg){
      HANDLE_MSG(m_hWnd, WM_CREATE       , OnCreate);
      HANDLE_MSG(m_hWnd, WM_MOUSEACTIVATE, OnMouseActivate);
      HANDLE_MSG(m_hWnd, WM_DESTROY      , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_NCHITTEST    , OnNCHitTest);
      }
      return ::DefWindowProc(m_hWnd, uMsg, wParam, lParam);
   }

public:
   CCursor() : m_hWnd(NULL)
   {
      static WNDCLASSEX wc = {
         sizeof(WNDCLASSEX),                 // UINT        cbSize;
         0,                                  // UINT        style;
         WndProcThis,                        // WNDPROC     lpfnWndProc;
         0,                                  // int         cbClsExtra;
         0,                                  // int         cbWndExtra;
         ::GetModuleHandle(NULL),            // HINSTANCE   hInstance;
         NULL,                               // HICON       hIcon;
         ::LoadCursor(NULL, IDC_ARROW),      // HCURSOR     hCursor;
         (HBRUSH)::GetStockObject(BLACK_BRUSH),  // HBRUSH      hbrBackground;
         //::GetSysColorBrush(COLOR_BTNFACE),  // HBRUSH      hbrBackground;
         NULL,                               // LPCTSTR     lpszMenuName;
         GetWindowClass(),                   // LPCTSTR     lpszClassName;
         NULL                                // HICON       hIconSm;
      };
      static ATOM atom = ::RegisterClassEx(&wc);
   }

   BOOL Create(HWND hWndParent, const RECT &rc) {
      m_hWnd = ::CreateWindowEx(
   #if(_WIN32_WINNT >= 0x0500)
         WS_EX_NOACTIVATE,
   #else
         0,
   #endif
         GetWindowClass(), NULL, WS_POPUP,
         rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,
         hWndParent, NULL, ::GetModuleHandle(NULL), this);
      return !!m_hWnd;
   }

private:
   BOOL OnCreate(HWND, LPCREATESTRUCT lpCreateStruct) {
      lpCreateStruct->cx = 18; lpCreateStruct->cy = 24; //  

      //   '- '
      POINT pArr[] = {{0,0}, {17,17}, {6,17}, {0,23}}; // {{0,0}, {17,17}, {9,17}, {14,28}, {13,29}, {11,29}, {5,18}, {0,23}};
      m_hRgn = ::CreatePolygonRgn(pArr, chDIMOF(pArr), WINDING);
      ::SetWindowRgn(m_hWnd, m_hRgn, TRUE);

      return TRUE;
   }

   void OnDestroy(HWND){
      //  
      ::SetWindowRgn(m_hWnd, NULL, FALSE); //  m_hRgn.   m_hRgn       -
      m_hRgn = NULL;
   }

   int OnMouseActivate(HWND, HWND hWndTopLevel, UINT codeHitTest, UINT msg) {
      return MA_NOACTIVATE;
   }

   UINT OnNCHitTest(HWND, int x, int y) {
      return HTTRANSPARENT;
   }
};
/**/
////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CGradientCursor : public CWindowImpl<CGradientCursor, CWindow, CWinTraits<WS_POPUP | WS_VISIBLE, WS_EX_NOACTIVATE /*| WS_EX_TRANSPARENT*/> >
{
private:
   HRGN m_hRgn;
   BYTE m_Color[3][3];
	enum {
		ID_TIMER_REDRAW = 123
	};

public:
   DECLARE_WND_CLASS(_T("CGradientCursor"))

private:
   BEGIN_MSG_MAP_EX(CGradientCursor)
      MSG_WM_CREATE       (OnCreate)
      MSG_WM_DESTROY      (OnDestroy)
      MSG_WM_PAINT        (OnPaint)
      MSG_WM_TIMER        (OnTimer)
      MSG_WM_MOUSEACTIVATE(OnMouseActivate)
      MSG_WM_NCHITTEST    (OnNCHitTest)
   END_MSG_MAP()

public:
   CGradientCursor() :
      m_hRgn(NULL)
   {
      for (int i=0; i<3; i++)
         for (int j=0; j<3; j++)
            m_Color[i][j] = 0xFF>>rand(8);
   }

private:
   LRESULT OnCreate(LPCREATESTRUCT lpCreateStruct) {
      lpCreateStruct->cx = 18; lpCreateStruct->cy = 24; //  

      //   '- '
      POINT pArr[] = {{0,0}, {17,17}, {6,17}, {0,23}};//{{0,0}, {17,17}, {9,17}, {14,28}, {13,29}, {11,29}, {5,18}, {0,23}};
      m_hRgn = ::CreatePolygonRgn(pArr, chDIMOF(pArr), WINDING);
      SetWindowRgn(m_hRgn, TRUE);

      //    
      SetTimer(ID_TIMER_REDRAW, 10);

      return 0;
   }

   void OnDestroy() {
      //  
      SetWindowRgn(NULL, FALSE); //  m_hRgn.   m_hRgn       -
      m_hRgn = NULL;
   }

   void OnPaint(HDC /*hDC*/)
   {
      CPaintDC dc(m_hWnd);

      // gradient fill
      /** /
      if (0) {
         HBRUSH hBrOld = dc.SelectBrush(::CreateSolidBrush(RGB(m_Color[0][0], m_Color[0][1], m_Color[0][2])));
         dc.PatBlt(0, 0, 17, 23, PATCOPY);
         ::DeleteObject(hBrOld);
      } else/**/
      {
         TRIVERTEX vert[] = {
            {
               0,       // LONG    x;
               0,       // LONG    y;
               m_Color[0][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[0][1]<<8,  // COLOR16 Green;
               m_Color[0][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }, {
               17,      // LONG    x;
               17,      // LONG    y;   
               m_Color[1][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[1][1]<<8,  // COLOR16 Green;
               m_Color[1][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }, {
               0,       // LONG    x;
               23,      // LONG    y;   
               m_Color[2][0]<<8,  // COLOR16 Red;   0x0000..0xff00
               m_Color[2][1]<<8,  // COLOR16 Green;
               m_Color[2][2]<<8,  // COLOR16 Blue;
               0x0000   // COLOR16 Alpha;
            }
         };
         GRADIENT_TRIANGLE gTri = {0, 1, 2};
         dc.GradientFill(vert, chDIMOF(vert), &gTri, 1, GRADIENT_FILL_TRIANGLE);
      }

   }

   void OnTimer(UINT nIDEvent, TIMERPROC /*tmprc*/) {
      if (nIDEvent == ID_TIMER_REDRAW) {
         DWORD dwTC = ::GetTickCount();
         int i = (dwTC>>0)%3;//0;//
         int j = (dwTC>>2)%3;//rand(2);//
         BYTE &bC = m_Color[i][j];
         bC = ((bC & 1) ? 0x00 : 0x80) | (bC>>1);
         InvalidateRect(&CRect(0,0,17,23), FALSE);
      }
   }

   int OnMouseActivate(HWND /*hDesktopWnd*/, UINT /*nHitTest*/, UINT /*message*/) {
      return MA_NOACTIVATE;
   }

   UINT OnNCHitTest(const CPoint& /*point*/) {
      return HTTRANSPARENT;
   }
};

#endif // _WTL_VER
