////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "CustomSkill.h"
//
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__CUSTOMSKILL__
#define __FILE__CUSTOMSKILL__

#include "../Control/ButtonFM.h"
#include "../Control/EditFM.h"
#include "../Resource.h"

#ifndef _WTL_VER

class CDialogCustomSkill {
public:
   HWND m_hWnd;
private:
   CButtonExFM     m_BtnOk, m_BtnCancel;
   CButtonImageFM  m_BtnPopup;
   CEditGradientFM m_EditX, m_EditY, m_EditMine;
   HMENU           m_hMenu;

   SIZE m_sizeNewMosaic;
   int  m_iNewMines;

private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("CustomSkillLevel"); return szResourceName; }
   static INT_PTR CALLBACK DialogProcThis(HWND, UINT, WPARAM, LPARAM);
   INT_PTR DialogProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam)  {
      switch (uMsg){
      HANDLE_MSG(m_hWnd, WM_CONTEXTMENU, OnContextMenu);
      HANDLE_MSG(m_hWnd, WM_INITDIALOG , OnInitDialog);
      HANDLE_MSG(m_hWnd, WM_MEASUREITEM, OnMeasureItem);
      HANDLE_MSG(m_hWnd, WM_DRAWITEM   , OnDrawItem);
      HANDLE_MSG(m_hWnd, WM_COMMAND    , OnCommand);
      HANDLE_MSG(m_hWnd, WM_CLOSE      , OnClose);
      HANDLE_MSG(m_hWnd, WM_DESTROY    , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND , OnEraseBkgnd);
      }
      return FALSE;
   }
public:
   CDialogCustomSkill() : m_hWnd(NULL) {}
   HWND Create(HWND hWndParent);
private:
   BOOL OnInitDialog (HWND, HWND, LPARAM);
   void OnMeasureItem(HWND, MEASUREITEMSTRUCT*);
   void OnDrawItem   (HWND, const DRAWITEMSTRUCT*);
   void OnCommand    (HWND, int, HWND, UINT);
   void OnClose      (HWND);
   void OnDestroy    (HWND);
   BOOL OnEraseBkgnd (HWND, HDC);
   void OnContextMenu(HWND, HWND, UINT, UINT);
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CDialogCustomSkill : public CDialogImpl<CDialogCustomSkill>, public CWinDataExchange<CDialogCustomSkill>
{
private:
   class CSpinImpl: public CWindowImpl<CSpinImpl, CUpDownCtrl> { DECLARE_EMPTY_MSG_MAP() };
private:
   CButtonExFM     m_BtnOk, m_BtnCancel;
   CButtonImageFM  m_BtnPopup;
   CSpinImpl       m_SpinX, m_SpinY, m_SpinMine;
   CEditGradientFM m_EditX, m_EditY, m_EditMine;
   CMenu           m_hMenu;

   SIZE m_sizeNewMosaic;
   int  m_iNewMines;
public:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("CustomSkillLevel"); return szResourceName; }//enum { IDD = 0 };

	// modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL);

private:
   BEGIN_MSG_MAP_EX(CDialogCustomSkill)
      MSG_WM_INITDIALOG (OnInitDialog)
      MSG_WM_MEASUREITEM(OnMeasureItem)
      MSG_WM_DRAWITEM   (OnDrawItem)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_EDIT_X           , EN_CHANGE , OnSpinChangeX)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_EDIT_Y           , EN_CHANGE , OnSpinChangeY)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_EDIT_MINE        , EN_CHANGE , OnSpinChangeMine)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_FULLSCREEN1      , BN_CLICKED, OnClickFullScreen)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_FULLSCREEN2      , BN_CLICKED, OnClickFullScreen)
      COMMAND_HANDLER_EX(IDC_CUSTOMSKILL_BUTTON_POPUP_MENU, BN_CLICKED, OnClickPopupMenu)
      COMMAND_HANDLER_EX(IDOK                             , BN_CLICKED, OnClickOkCancel)
      COMMAND_HANDLER_EX(IDCANCEL                         , BN_CLICKED, OnClickOkCancel)
      MSG_WM_CLOSE      (OnClose)
      MSG_WM_DESTROY    (OnDestroy)
      MSG_WM_ERASEBKGND (OnEraseBkgnd)
      MSG_WM_CONTEXTMENU(OnContextMenu)
   END_MSG_MAP()
	BEGIN_DDX_MAP(CDialogAbout)
      DDX_CONTROL(IDOK                             , m_BtnOk)
      DDX_CONTROL(IDCANCEL                         , m_BtnCancel)
      DDX_CONTROL(IDC_CUSTOMSKILL_BUTTON_POPUP_MENU, m_BtnPopup)
      DDX_CONTROL(IDC_CUSTOMSKILL_SPIN_X           , m_SpinX)
      DDX_CONTROL(IDC_CUSTOMSKILL_SPIN_Y           , m_SpinY)
      DDX_CONTROL(IDC_CUSTOMSKILL_SPIN_MINE        , m_SpinMine)
      DDX_CONTROL(IDC_CUSTOMSKILL_EDIT_X           , m_EditX)
      DDX_CONTROL(IDC_CUSTOMSKILL_EDIT_Y           , m_EditY)
      DDX_CONTROL(IDC_CUSTOMSKILL_EDIT_MINE        , m_EditMine)
	END_DDX_MAP()

   BOOL OnInitDialog(HWND, LPARAM);
   void OnMeasureItem(UINT, LPMEASUREITEMSTRUCT);
   void OnDrawItem   (UINT, LPDRAWITEMSTRUCT);
   void OnSpinChangeX    (UINT, int, HWND);
   void OnSpinChangeY    (UINT, int, HWND);
   void OnSpinChangeMine (UINT, int, HWND);
   void OnClickFullScreen(UINT, int, HWND);
   void OnClickPopupMenu (UINT, int, HWND);
   void OnClickOkCancel  (UINT, int, HWND);
   void OnClose();
   void OnDestroy();
   BOOL OnEraseBkgnd(HDC hDC);
   void OnContextMenu(HWND, const CPoint&);
};

#endif // _WTL_VER

#endif // __FILE__CUSTOMSKILL__
