////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "InputText.cpp"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "InputText.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "./DialogMessageHook.h"
#include "./Statistics.h"
#include "./Skin.h"
#include "./Administration.h"

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

WNDPROC_STATIC(IDC_INPUTTEXT_STATIC_TEXT        , GetFM2WinMain().GetSkin())
WNDPROC_STATIC(IDC_INPUTTEXT_STATIC_PASSWORD    , GetFM2WinMain().GetSkin())
WNDPROC_STATIC(IDC_INPUTTEXT_STATIC_CONFIRMATION, GetFM2WinMain().GetSkin())

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////
#ifndef _WTL_VER

CDialogInputText::CDialogInputText(EOperation eOperation, LPCTSTR szName) :
   m_strName   (szName),
   m_eOperation(eOperation),
   m_hWnd      (NULL)
{
}

// modal dialogs
int CDialogInputText::DoModal(HWND hWndParent) {
   m_bModal = true;
   return ::DialogBoxParam(::GetModuleHandle(NULL), CDialogInputText::IDD(), hWndParent, CDialogInputText::DialogProcThis, (LPARAM)this);
}

// modeless dialogs
HWND CDialogInputText::Create(HWND hWndParent) {
   m_bModal = false;
	return ::CreateDialogParam(::GetModuleHandle(NULL), CDialogInputText::IDD(), hWndParent, (DLGPROC)CDialogInputText::DialogProcThis, (LPARAM)this);
}

/* static */ INT_PTR CALLBACK CDialogInputText::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   CDialogInputText *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CDialogInputText*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CDialogInputText*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CDialogInputText::OnInitDialog(HWND, HWND hWndFocus, LPARAM lParam) {
   m_BtnOk       .SubclassWindow(::GetDlgItem(m_hWnd, IDOK));
   m_EditName    .SubclassWindow(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_EDIT_TEXT        ));
   m_EditPassword.SubclassWindow(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_EDIT_PASSWORD    ));
   m_EditConfirm .SubclassWindow(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_EDIT_CONFIRMATION));

   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    ), CLang::GetString(IDS__DIALOG_ADMINISTRATION__PASSWORD    ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION), CLang::GetString(IDS__DIALOG_ADMINISTRATION__CONFIRMATION));
 //::SetWindowText(::GetDlgItem(m_hWnd, IDCANCEL                         ), CLang::GetString(IDS__CANCEL                           ));

   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_TEXT        );
   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    );
   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION);

   switch (m_eOperation) {
   case eSkinName:
      {
         ::SetWindowText(m_hWnd, CLang::GetString(IDS__DIALOG_INPUTTEXT_CAPTION));
         ::SetWindowText(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_INPUTTEXT_STATIC_TEXT));
         ::ShowWindow(m_EditPassword.m_hWnd                                  , SW_HIDE);
         ::ShowWindow(m_EditConfirm  .m_hWnd                                 , SW_HIDE);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    ), SW_HIDE);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         ::MoveWindow(m_BtnOk.m_hWnd, 150, 60, 74, 24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         ::MoveWindow(m_hWnd,
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width(),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerSelect:
      {
         ::SetWindowText(m_hWnd, CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_PASSWORD));
         ::ShowWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_TEXT        ), SW_HIDE);
         ::ShowWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         ::ShowWindow(    m_EditName.m_hWnd                                    , SW_HIDE);
         ::ShowWindow( m_EditConfirm.m_hWnd                                    , SW_HIDE);
         ::MoveWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD),  11,11, 80,13, FALSE);
         ::MoveWindow(m_EditPassword.m_hWnd                                ,  11,28,212,23, FALSE);
         ::MoveWindow(       m_BtnOk.m_hWnd                                , 150,60, 74,24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         ::MoveWindow(m_hWnd,
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerNewPassword:
      {
         ::SetWindowText(m_hWnd, CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPASSWORD));
         ::ShowWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_TEXT), SW_HIDE);
         ::ShowWindow(m_EditName    .m_hWnd, SW_HIDE);
         ::MoveWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    ), 11,11,212,13, FALSE);
         ::MoveWindow(m_EditPassword.m_hWnd,  11, 28,212,23, FALSE);
         ::MoveWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION), 11,58,212,13, FALSE);
         ::MoveWindow(m_EditConfirm .m_hWnd,  11, 80,212,23, FALSE);
         ::MoveWindow(m_BtnOk       .m_hWnd, 150,113, 74,24, FALSE);

         RECTEX rect(0,0,241,170);
         SIZE sizeScreen = ::GetScreenSize();
         ::MoveWindow(m_hWnd,
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerNew:
      ::SetWindowText(m_hWnd, CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPLAYER));
      ::SetWindowText(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_NAME));
      ::EnableWindow(m_BtnOk.m_hWnd, FALSE);
      break;
   case ePlayerRename:
      {
         ::SetWindowText(m_hWnd, CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_NEW_NAME));
         ::SetWindowText(::GetDlgItem(m_hWnd, IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEW_NAME));
         ::ShowWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    ), SW_HIDE);
         ::ShowWindow(::GetDlgItem(  m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         ::ShowWindow(m_EditPassword.m_hWnd                                    , SW_HIDE);
         ::ShowWindow(m_EditConfirm .m_hWnd                                    , SW_HIDE);
         ::SetWindowText(m_EditName .m_hWnd, m_strName);
         ::EnableWindow(m_BtnOk.m_hWnd, FALSE);
         ::MoveWindow(m_BtnOk.m_hWnd, 150, 60, 74, 24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         ::MoveWindow(m_hWnd,
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   }

   if (!m_bModal) {
      CDialogMessageHook::InstallHook(m_hWnd);
   }

   return TRUE;
}

void CDialogInputText::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_INPUTTEXT_EDIT_TEXT:
      if (codeNotify==EN_CHANGE) {
         m_strName = ::GetWindowText(m_EditName.m_hWnd);
         ::EnableWindow(m_BtnOk.m_hWnd, !m_strName.IsEmpty() &&
                                        !GetFM2WinMain().GetPlayers().PlayerExist(m_strName));
      }
      return;
   case IDOK:
      if (codeNotify==BN_CLICKED) {
         switch (m_eOperation) {
         case eSkinName:
            {
               CFileSkin FileSkin;
               FileSkin.SetSkinName(::GetWindowText(m_EditName.m_hWnd));
               if (::FileExist(FileSkin.GetFileName())) {
                  if (IDNO == ::MessageBox(m_hWnd, CLang::GetString(IDS__FILESKIN__OVERWRITE), CLang::GetString(IDS__FILESKIN__FILE_COPYING), MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2))
                     return;
               }
               FileSkin.SetSkin(GetFM2WinMain().GetSkin());
               BOOL bRes = FileSkin.SaveFile();
               if (bRes) {
                  CSkinMenu::ReloadMenuList(GetFM2WinMain().GetSkinSubMenu());
               }
            }
            break;
         case ePlayerNew:
         case ePlayerRename:
            if (m_strName == CDialogAdministration::DEFAULT_PLAYER_NAME()) {
               ::MessageBox(m_hWnd,
                  CLang::GetString(IDS__DIALOG_ADMINISTRATION__CANT_USE_VIRT_NAME),
                  CLang::GetString(IDS__DIALOG_ADMINISTRATION__INVALID_NAME),
                  MB_OK | MB_ICONERROR);
               return;
            }
            if (m_eOperation == ePlayerRename) break;
         case ePlayerNewPassword:

            m_strPassword = ::GetWindowText(m_EditPassword.m_hWnd);
            if (m_strPassword != ::GetWindowText(m_EditPassword.m_hWnd)) {
               ::MessageBox(m_hWnd,
                  CLang::GetString(IDS__DIALOG_ADMINISTRATION__PASSWORD_NOT_CONFIRMED),
                  CLang::GetString(IDS__ERROR),
                  MB_OK | MB_ICONERROR);
               return;
            }
            break;
         case ePlayerSelect:
            m_strPassword = ::GetWindowText(m_EditPassword.m_hWnd);
            break;
         }
      }
   case IDCANCEL:
      if (codeNotify==BN_CLICKED) {
         OnClose(m_hWnd, idControl);
      }
      return;
   }
}

void CDialogInputText::OnClose(HWND, int idControl) {
   if (m_bModal) {
      ::EndDialog(m_hWnd, idControl);
   } else {
      ::DestroyWindow(m_hWnd);
   }
}

void CDialogInputText::OnDestroy(HWND) {
   if (!m_bModal) {
      CDialogMessageHook::UninstallHook(m_hWnd);
   }
   m_hWnd = NULL;
}

BOOL CDialogInputText::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // ::DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hdc, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

CDialogInputText::CDialogInputText(EOperation eOperation, LPCTSTR szName) :
   m_strName(szName),
   m_eOperation   (eOperation)
{
}

// modal dialogs
int CDialogInputText::DoModal(HWND hWndParent, LPARAM dwInitParam) {
   ATLASSERT(m_hWnd == NULL);
   _Module.AddCreateWndData(&m_thunk.cd, (CDialogImpl<CDialogInputText>*)this);

//#ifdef _DEBUG
   m_bModal = true;
//#endif //_DEBUG
   return ::DialogBoxParam(_Module.GetResourceInstance(), CDialogInputText::IDD(), hWndParent, (DLGPROC)CDialogInputText::StartDialogProc, dwInitParam);
}

// modeless dialogs
HWND CDialogInputText::Create(HWND hWndParent, LPARAM dwInitParam)
{
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, (CDialogImpl<CDialogInputText>*)this);
//#ifdef _DEBUG
	m_bModal = false;
//#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CDialogInputText::IDD(), hWndParent, (DLGPROC)CDialogInputText::StartDialogProc, dwInitParam);
	ATLASSERT(m_hWnd == hWnd);
	return hWnd;
}

BOOL CDialogInputText::OnInitDialog(HWND hwndFocus, LPARAM lParam) {
   DoDataExchange(FALSE, IDOK);
   DoDataExchange(FALSE, IDC_INPUTTEXT_EDIT_TEXT);
   DoDataExchange(FALSE, IDC_INPUTTEXT_EDIT_PASSWORD);
   DoDataExchange(FALSE, IDC_INPUTTEXT_EDIT_CONFIRMATION);

   ::SetWindowText(GetDlgItem(IDC_INPUTTEXT_STATIC_PASSWORD    ), CLang::GetString(IDS__DIALOG_ADMINISTRATION__PASSWORD    ));
   ::SetWindowText(GetDlgItem(IDC_INPUTTEXT_STATIC_CONFIRMATION), CLang::GetString(IDS__DIALOG_ADMINISTRATION__CONFIRMATION));
 //::SetWindowText(GetDlgItem(IDCANCEL                         ), CLang::GetString(IDS__CANCEL                           ));

   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_TEXT        );
   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_PASSWORD    );
   SETNEWWNDPROC(m_hWnd, IDC_INPUTTEXT_STATIC_CONFIRMATION);

   switch (m_eOperation) {
   case eSkinName:
      {
         SetWindowText(CLang::GetString(IDS__DIALOG_INPUTTEXT_CAPTION));
         ::SetWindowText(GetDlgItem(IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_INPUTTEXT_STATIC_TEXT));
         m_EditPassword.ShowWindow(SW_HIDE);
         m_EditConfirm .ShowWindow(SW_HIDE);
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_PASSWORD    ), SW_HIDE);
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         m_BtnOk.MoveWindow(150, 60, 74, 24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         MoveWindow(
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width(),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerSelect:
      {
         SetWindowText(CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_PASSWORD));
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_TEXT        ), SW_HIDE);
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         m_EditName.ShowWindow(SW_HIDE);
         m_EditConfirm.ShowWindow(SW_HIDE);
         ::MoveWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_PASSWORD), 11,11, 80,13, FALSE);
         m_EditPassword.MoveWindow(11,28,212,23, FALSE);
         m_BtnOk.MoveWindow(150, 60, 74, 24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         MoveWindow(
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerNewPassword:
      {
         SetWindowText(CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPASSWORD));
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_TEXT), SW_HIDE);
         m_EditName.ShowWindow(SW_HIDE);
         ::MoveWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_PASSWORD    ), 11,11,212,13, FALSE);
         m_EditPassword.MoveWindow(11,28,212,23, FALSE);
         ::MoveWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_CONFIRMATION), 11,58,212,13, FALSE);
         m_EditConfirm.MoveWindow(11,80,212,23, FALSE);
         m_BtnOk.MoveWindow(150, 113, 74, 24, FALSE);

         RECTEX rect(0,0,241,170);
         SIZE sizeScreen = ::GetScreenSize();
         MoveWindow(
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   case ePlayerNew:
      SetWindowText(CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPLAYER));
      ::SetWindowText(GetDlgItem(IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_NAME));
      m_BtnOk.EnableWindow(FALSE);
      break;
   case ePlayerRename:
      {
         SetWindowText(CLang::GetString(IDS__DIALOG_ADMINISTRATION__ENTER_NEW_NAME));
         ::SetWindowText(GetDlgItem(IDC_INPUTTEXT_STATIC_TEXT), CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEW_NAME));
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_PASSWORD    ), SW_HIDE);
         ::ShowWindow(GetDlgItem(IDC_INPUTTEXT_STATIC_CONFIRMATION), SW_HIDE);
         m_EditPassword.ShowWindow(SW_HIDE);
         m_EditConfirm .ShowWindow(SW_HIDE);
         m_EditName.SetWindowText(m_strName);
         m_BtnOk.EnableWindow(FALSE);
         m_BtnOk.MoveWindow(150, 60, 74, 24, FALSE);

         RECTEX rect(0,0,241,120);
         SIZE sizeScreen = ::GetScreenSize();
         MoveWindow(
            sizeScreen.cx/2 - rect.width ()/2,
            sizeScreen.cy/2 - rect.height()/2,
            rect.width (),
            rect.height(),
            FALSE);
      }
      break;
   }

   if (!m_bModal) {
      CDialogMessageHook::InstallHook(m_hWnd);
   }

   return TRUE;
}

void CDialogInputText::OnChangeLogin(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   DoDataExchange(TRUE, IDC_INPUTTEXT_EDIT_TEXT);
   m_BtnOk.EnableWindow(!m_strName.IsEmpty() &&
                        !GetFM2WinMain().GetPlayers().PlayerExist(m_strName));
}

void CDialogInputText::OnClickOk(UINT /*wNotifyCode*/, int idControl, HWND /*hWndCtl*/) {
   switch (m_eOperation) {
   case eSkinName:
      {
         CFileSkin FileSkin;
         FileSkin.SetSkinName(::GetWindowText(m_EditName.m_hWnd));
         if (::FileExist(FileSkin.GetFileName())) {
            if (IDNO == MessageBox(CLang::GetString(IDS__FILESKIN__OVERWRITE), CLang::GetString(IDS__FILESKIN__FILE_COPYING), MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2))
               return;
         }
         FileSkin.SetSkin(GetFM2WinMain().GetSkin());
         BOOL bRes = FileSkin.SaveFile();
         if (bRes) {
            CSkinMenu::ReloadMenuList(GetFM2WinMain().GetSkinSubMenu());
         }
      }
      break;
   case ePlayerNew:
   case ePlayerRename:
      if (m_strName == CDialogAdministration::DEFAULT_PLAYER_NAME()) {
         MessageBox(
            CLang::GetString(IDS__DIALOG_ADMINISTRATION__CANT_USE_VIRT_NAME),
            CLang::GetString(IDS__DIALOG_ADMINISTRATION__INVALID_NAME),
            MB_OK | MB_ICONERROR);
         return;
      }
      if (m_eOperation == ePlayerRename) break;
   case ePlayerNewPassword:
      DoDataExchange(TRUE, IDC_INPUTTEXT_EDIT_PASSWORD);
      DoDataExchange(TRUE, IDC_INPUTTEXT_EDIT_CONFIRMATION);
      if (m_strPassword != m_strPasswordConfirm) {
         MessageBox(
            CLang::GetString(IDS__DIALOG_ADMINISTRATION__PASSWORD_NOT_CONFIRMED),
            CLang::GetString(IDS__ERROR),
            MB_OK | MB_ICONERROR);
         return;
      }
      break;
   case ePlayerSelect:
      DoDataExchange(TRUE, IDC_INPUTTEXT_EDIT_PASSWORD);
      break;
   }
   OnClose(idControl);
}

void CDialogInputText::OnClickCancel(UINT /*wNotifyCode*/, int idControl, HWND /*hWndCtl*/) {
   OnClose(idControl);
}

void CDialogInputText::OnClose(int idControl) {
   if (m_bModal) {
      EndDialog(idControl);
   } else {
      DestroyWindow();
   }
}

void CDialogInputText::OnDestroy() {
   if (!m_bModal) {
      CDialogMessageHook::UninstallHook(m_hWnd);
   }

}

BOOL CDialogInputText::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(WM_ERASEBKGND, (WPARAM)hdc, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

#endif // _WTL_VER
