////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "SelectMosaic.cpp"
//    "Select Mosaic"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "SelectMosaic.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "../Lang.h"
#include "./DialogMessageHook.h"

////////////////////////////////////////////////////////////////////////////////
//                             global variables
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

WNDPROC_STATIC(IDC_SELECTMOSAIC_STATIC_NUMBER, GetFM2WinMain().GetSkin())
WNDPROC_STATIC(IDC_SELECTMOSAIC_SPIN         , GetFM2WinMain().GetSkin())
WNDPROC_STATIC(IDC_SELECTMOSAIC_STATIC_NAME  , GetFM2WinMain().GetSkin())
WNDPROC_STATIC(IDC_SELECTMOSAIC_COMBOBOX     , GetFM2WinMain().GetSkin())

////////////////////////////////////////////////////////////////////////////////
//                          implementation CDialogSelectMosaic
////////////////////////////////////////////////////////////////////////////////
#ifndef _WTL_VER

HWND CDialogSelectMosaic::Create(HWND hWndParent) {
   return ::CreateDialogParam(::GetModuleHandle(NULL), CDialogSelectMosaic::IDD(), hWndParent, (DLGPROC)CDialogSelectMosaic::DialogProcThis, (LPARAM)this);
}

/*static*/ INT_PTR CALLBACK CDialogSelectMosaic::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogSelectMosaic::DialogProcThis: %s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CDialogSelectMosaic *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CDialogSelectMosaic*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CDialogSelectMosaic*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CDialogSelectMosaic::OnInitDialog(HWND, HWND hwndFocus, LPARAM lParam) {
   m_BtnOk.SubclassWindow(::GetDlgItem(m_hWnd, IDOK));
   m_Edit .SubclassWindow(::GetDlgItem(m_hWnd, IDC_SELECTMOSAIC_EDIT));

   ::SetWindowText(m_hWnd                                              , CLang::GetString(IDS__DIALOG_SELECTMOSAIC        ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_SELECTMOSAIC_STATIC_NUMBER), CLang::GetString(IDS__DIALOG_SELECTMOSAIC__NUMBER));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_SELECTMOSAIC_STATIC_NAME  ), CLang::GetString(IDS__DIALOG_SELECTMOSAIC__NAME  ));

   m_iNumberMosaic = m_iFirstKey * 10;
   m_eMosaic = NumberMosaicToNameMosaic(m_iNumberMosaic);

   for (int i=0; i<=eMosaicNil; i++)
      ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_COMBOBOX, CB_ADDSTRING, 0L, (LPARAM)(LPCTSTR)CLang::GetString(IDS__MOSAIC_NAME_00+i));

   ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_SPIN, UDM_SETRANGE, 0L, MAKELPARAM(72, 30));
   ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_SPIN, UDM_SETPOS  , 0L, m_iNumberMosaic);
   ::PostMessage(m_Edit.m_hWnd, EM_SETSEL, 1L, 2L);

   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_STATIC_NUMBER);
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_SPIN         );
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_STATIC_NAME  );
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_COMBOBOX     );

   CDialogMessageHook::InstallHook(m_hWnd);

   return TRUE;
}

void CDialogSelectMosaic::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_SELECTMOSAIC_COMBOBOX:
      if (codeNotify == CBN_SELCHANGE) {
         m_eMosaic = (EMosaic)::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_COMBOBOX, CB_GETCURSEL, 0L, 0L);
         m_iNumberMosaic = NameMosaicToNumberMosaic(m_eMosaic);
         ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_SPIN, UDM_SETPOS, 0L, m_iNumberMosaic);
         ::SendMessage(m_BtnOk.m_hWnd, BM_SETIMAGE, IMAGE_ICON, (LPARAM)CFastMines2WinMain::GetIconMosaic((EMosaic)m_eMosaic));
      }
      return;
   case IDC_SELECTMOSAIC_EDIT:
      if (codeNotify == EN_CHANGE) {
         m_iNumberMosaic = ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_SPIN, UDM_GETPOS, 0L, 0L);
         m_eMosaic = NumberMosaicToNameMosaic(m_iNumberMosaic);
         ::SendDlgItemMessage(m_hWnd, IDC_SELECTMOSAIC_COMBOBOX, CB_SETCURSEL, (WPARAM)m_eMosaic, 0L);
         ::SendMessage(m_BtnOk.m_hWnd, BM_SETIMAGE, IMAGE_ICON, (LPARAM)CFastMines2WinMain::GetIconMosaic((EMosaic)m_eMosaic));
      }
      return;
   case IDOK:
      if (codeNotify == BN_CLICKED) {
         if (m_eMosaic != eMosaicNil) {
            GetFM2WinMain().SetMosaic(m_eMosaic);
         }
      }
   case IDCANCEL:
      if (codeNotify == BN_CLICKED) {
         ::SendMessage(m_hWnd, WM_CLOSE, 0L, 0L);
      }
      return;
   }
}

void CDialogSelectMosaic::OnClose(HWND) {
   ::DestroyWindow(m_hWnd);
}

void CDialogSelectMosaic::OnDestroy(HWND) {
   CDialogMessageHook::UninstallHook(m_hWnd);
   m_hWnd = NULL;
}

BOOL CDialogSelectMosaic::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// modeless dialogs
HWND CDialogSelectMosaic::Create(HWND hWndParent, LPARAM dwInitParam)
{
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, this);
#ifdef _DEBUG
	m_bModal = false;
#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CDialogSelectMosaic::IDD(), hWndParent, (DLGPROC)CDialogSelectMosaic::StartDialogProc, dwInitParam);
	ATLASSERT(m_hWnd == hWnd);
	return hWnd;
}

BOOL CDialogSelectMosaic::OnInitDialog(HWND hwndFocus, LPARAM lParam) {
   DoDataExchange();

   SetWindowText(                                              CLang::GetString(IDS__DIALOG_SELECTMOSAIC        ));
   ::SetWindowText(GetDlgItem(IDC_SELECTMOSAIC_STATIC_NUMBER), CLang::GetString(IDS__DIALOG_SELECTMOSAIC__NUMBER));
   ::SetWindowText(GetDlgItem(IDC_SELECTMOSAIC_STATIC_NAME  ), CLang::GetString(IDS__DIALOG_SELECTMOSAIC__NAME  ));

   m_iNumberMosaic = m_iFirstKey * 10;
   m_eMosaic = NumberMosaicToNameMosaic(m_iNumberMosaic);

   for (int i=0; i<=eMosaicNil; i++)
      m_ComboBox.AddString(CLang::GetString(IDS__MOSAIC_NAME_00+i));

   SendDlgItemMessage(IDC_SELECTMOSAIC_SPIN, UDM_SETRANGE, 0L, MAKELPARAM(72, 30));
   SendDlgItemMessage(IDC_SELECTMOSAIC_SPIN, UDM_SETPOS  , 0L, m_iNumberMosaic);
   m_Edit.PostMessage(EM_SETSEL, 1L, 2L);

   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_STATIC_NUMBER);
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_SPIN         );
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_STATIC_NAME  );
   SETNEWWNDPROC(m_hWnd, IDC_SELECTMOSAIC_COMBOBOX     );

   CDialogMessageHook::InstallHook(m_hWnd);

   return TRUE;
}

void CDialogSelectMosaic::OnChangeCombo(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   m_eMosaic = (EMosaic)m_ComboBox.GetCurSel();
   m_iNumberMosaic = NameMosaicToNumberMosaic(m_eMosaic);
   SendDlgItemMessage(IDC_SELECTMOSAIC_SPIN, UDM_SETPOS, 0L, m_iNumberMosaic);
   m_BtnOk.SendMessage(BM_SETIMAGE, IMAGE_ICON, (LPARAM)CFastMines2WinMain::GetIconMosaic((EMosaic)m_eMosaic));
}

void CDialogSelectMosaic::OnChangeEdit(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   m_iNumberMosaic = SendDlgItemMessage(IDC_SELECTMOSAIC_SPIN, UDM_GETPOS, 0L, 0L);
   m_eMosaic = NumberMosaicToNameMosaic(m_iNumberMosaic);
   if (m_ComboBox.m_hWnd) m_ComboBox.SetCurSel(m_eMosaic);
   if (m_BtnOk   .m_hWnd) m_BtnOk.SendMessage(BM_SETIMAGE, IMAGE_ICON, (LPARAM)CFastMines2WinMain::GetIconMosaic((EMosaic)m_eMosaic));
}

void CDialogSelectMosaic::OnClickOkCancel(UINT /*wNotifyCode*/, int idControl, HWND /*hWndCtl*/) {
   if (idControl == IDOK) {
      if (m_eMosaic != eMosaicNil) {
         GetFM2WinMain().SetMosaic(m_eMosaic);
      }
   }
   SendMessage(WM_CLOSE, 0L, 0L);
}

void CDialogSelectMosaic::OnClose() {
   DestroyWindow();
}

void CDialogSelectMosaic::OnDestroy() {
   CDialogMessageHook::UninstallHook(m_hWnd);
}

BOOL CDialogSelectMosaic::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

#endif // _WTL_VER

/* static */ int CDialogSelectMosaic::GetGroup(EMosaic mosaic) {
   switch (mosaic) {
   case eMosaicTriangle1:
   case eMosaicTriangle2:
   case eMosaicTriangle3:
   case eMosaicTriangle4:
      return 0;
   case eMosaicSquare1:
   case eMosaicSquare2:
   case eMosaicParquet1:
   case eMosaicParquet2:
   case eMosaicTrapezoid1:
   case eMosaicTrapezoid2:
   case eMosaicTrapezoid3:
   case eMosaicRhombus1:
   case eMosaicQuadrangle1:
      return 1;
   case eMosaicPentagonT24:
   case eMosaicPentagonT5:
   case eMosaicPentagonT10:
      return 2;
   case eMosaicHexagon1:
      return 3;
   case eMosaicTrSq1:
   case eMosaicTrSq2:
   case eMosaicSqTrHex:
      return 4;
   }
   return 5;
}

/* static */ EMosaic CDialogSelectMosaic::NumberMosaicToNameMosaic(int m_iNumberMosaic) {
   switch (m_iNumberMosaic) {
   case 30: return eMosaicTriangle1;
   case 31: return eMosaicTriangle2;
   case 32: return eMosaicTriangle3;
   case 33: return eMosaicTriangle4;
   case 40: return eMosaicSquare1;
   case 41: return eMosaicSquare2;
   case 42: return eMosaicParquet1;
   case 43: return eMosaicParquet2;
   case 44: return eMosaicTrapezoid1;
   case 45: return eMosaicTrapezoid2;
   case 46: return eMosaicTrapezoid3;
   case 47: return eMosaicRhombus1;
   case 48: return eMosaicQuadrangle1;
   case 50: return eMosaicPentagonT24;
   case 51: return eMosaicPentagonT5;
   case 52: return eMosaicPentagonT10;
   case 60: return eMosaicHexagon1;
   case 70: return eMosaicTrSq1;
   case 71: return eMosaicTrSq2;
   case 72: return eMosaicSqTrHex;
   }
   return eMosaicNil;
}

/* static */ int CDialogSelectMosaic::NameMosaicToNumberMosaic(EMosaic m_eMosaic) {
   switch (m_eMosaic) {
   case eMosaicTriangle1  : return 30;
   case eMosaicTriangle2  : return 31;
   case eMosaicTriangle3  : return 32;
   case eMosaicTriangle4  : return 33;
   case eMosaicSquare1    : return 40;
   case eMosaicSquare2    : return 41;
   case eMosaicParquet1   : return 42;
   case eMosaicParquet2   : return 43;
   case eMosaicTrapezoid1 : return 44;
   case eMosaicTrapezoid2 : return 45;
   case eMosaicTrapezoid3 : return 46;
   case eMosaicRhombus1   : return 47;
   case eMosaicQuadrangle1: return 48;
   case eMosaicPentagonT24: return 50;
   case eMosaicPentagonT5 : return 51;
   case eMosaicPentagonT10: return 52;
   case eMosaicHexagon1   : return 60;
   case eMosaicTrSq1      : return 70;
   case eMosaicTrSq2      : return 71;
   case eMosaicSqTrHex    : return 72;
   }
   return 0;
}

void CDialogSelectMosaic::SetFirstKey(int iKey) {
   m_iFirstKey = iKey;
}
