////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Statistics.cpp" -   
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Statistics.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "./DialogMessageHook.h"
#include "./Administration.h"

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////

WNDPROC_ONLY_ERASEBKGND(IDC_STATISTICSorCHAMPIONS_TABCONTROL, GetFM2WinMain().GetSkin())
WNDPROC_STATIC(UPDOWN_CLASS                                 , GetFM2WinMain().GetSkin())

TC_ITEM *TC_Item(EMosaic eMosaic) {
   static TC_ITEM items[eMosaicNil] = 
     {{TCIF_IMAGE, 0,0, NULL, 0, eMosaicTriangle1  },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTriangle2  },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTriangle3  },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTriangle4  },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicSquare1    },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicSquare2    },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicParquet1   },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicParquet2   },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTrapezoid1 },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTrapezoid2 },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTrapezoid3 },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicRhombus1   },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicQuadrangle1},
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicPentagonT24},
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicPentagonT5 },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicPentagonT10},
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicHexagon1   },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTrSq1      },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicTrSq2      },
      {TCIF_IMAGE, 0,0, NULL, 0, eMosaicSqTrHex    }};
   return &items[eMosaic];
}

////////////////////////////////////////////////////////////////////////////////
//                         implementation - CDialogStatisticsOrChampions
////////////////////////////////////////////////////////////////////////////////

#ifndef _WTL_VER

HWND CDialogStatisticsOrChampions::Create(HWND hWndParent) {
   return ::CreateDialogParam(::GetModuleHandle(NULL), CDialogStatisticsOrChampions::IDD(), hWndParent, (DLGPROC)CDialogStatisticsOrChampions::DialogProcThis, (LPARAM)this);
}

/*static*/ INT_PTR CALLBACK CDialogStatisticsOrChampions::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogStatisticsOrChampions::DialogProcThis: %s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CDialogStatisticsOrChampions *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CDialogStatisticsOrChampions*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CDialogStatisticsOrChampions*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CDialogStatisticsOrChampions::OnInitDialog(HWND, HWND hWndFocus, LPARAM lParam) {
   m_Mosaic     = GetFM2WinMain().GetMosaic();
   m_SkillLevel = GetFM2WinMain().GetSkillLevel();
   if (m_SkillLevel == eSkillLevelCustom) m_SkillLevel = eSkillLevelBeginner;
   Button_SetCheck(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER + m_SkillLevel), BST_CHECKED);

   m_BtnBeginner  .SubclassWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER    ));
   m_BtnAmateur   .SubclassWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_BUTTON_AMATEUR     ));
   m_BtnProfi     .SubclassWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_BUTTON_PROFESSIONAL));
   m_BtnCrazy     .SubclassWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_BUTTON_CRAZY       ));
   m_Table        .SubclassWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABLE));
   m_hTabCtrl = ::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL);

   static COSVersion OS;
   if (!OS.IsWin9598Me()) {
      ::SetWindowStyle(m_hTabCtrl, ::GetWindowStyle(m_hTabCtrl) |  TCS_OWNERDRAWFIXED);
   } else {
      ::SetWindowStyle(m_hTabCtrl, ::GetWindowStyle(m_hTabCtrl) & ~TCS_OWNERDRAWFIXED);
   }

   if (!OS.IsWin9598Me()) {
      for (int i=0; i<eMosaicNil; i++) {
         m_TabItems[i].Attach(m_hTabCtrl, i, &CFastMines2WinMain::GetImageMosaic((EMosaic)i));//&CImage(_T("?2#Mine")));//
      }
      DWORD dwRes = TabCtrl_SetItemSize(m_hTabCtrl, 0,54);
   } else {
      m_hImageList = ImageList_Create(32,32, ILC_MASK, eMosaicNil, 1);
      int i=0;
      for (; i<eMosaicNil; i++) {
         ImageList_AddIcon(m_hImageList, CFastMines2WinMain::GetIconMosaic((EMosaic)i));
      }

      TabCtrl_SetImageList(m_hTabCtrl, m_hImageList);
      for (i=0; i<eMosaicNil; i++)
         TabCtrl_InsertItem(m_hTabCtrl, i, TC_Item((EMosaic)i));
   }
   TabCtrl_SetCurSel(m_hTabCtrl, m_Mosaic);

   OnInitTable();

   ::SetWindowText(m_BtnBeginner.m_hWnd, CLang::GetString(IDS__MENU_GAME__BEGINNER    ));
   ::SetWindowText(m_BtnAmateur .m_hWnd, CLang::GetString(IDS__MENU_GAME__AMATEUR     ));
   ::SetWindowText(m_BtnProfi   .m_hWnd, CLang::GetString(IDS__MENU_GAME__PROFESSIONAL));
   ::SetWindowText(m_BtnCrazy   .m_hWnd, CLang::GetString(IDS__MENU_GAME__CRAZY       ));

   ::SetWindowLong(m_hWnd, GWL_STYLE, WS_MAXIMIZEBOX ^ ::GetWindowLong(m_hWnd, GWL_STYLE));
   //::DrawMenuBar(m_hWnd);

   SETNEWWNDPROC(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL);
   SETNEWWNDPROC_HWND(::GetDlgItem(m_hTabCtrl, 1), UPDOWN_CLASS);

   CDialogMessageHook::InstallHook(m_hWnd);

   OnChangeTabPage();
   ModifyCaption();

   SIZE sizeDlg = GetFirstSizeDialog();
   SIZE sizeScreen = ::GetScreenSize();
   ::MoveWindow(m_hWnd,
      (sizeScreen.cx>>1) - (sizeDlg.cx>>1),
      (sizeScreen.cy>>1) - (sizeDlg.cy>>1),
      sizeDlg.cx,
      sizeDlg.cy,
      TRUE);

   return TRUE;
}

void CDialogStatisticsOrChampions::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER:
   case IDC_STATISTICSorCHAMPIONS_BUTTON_AMATEUR:
   case IDC_STATISTICSorCHAMPIONS_BUTTON_PROFESSIONAL:
   case IDC_STATISTICSorCHAMPIONS_BUTTON_CRAZY:
      if (codeNotify == BN_CLICKED) {
         int iOffset = idControl-IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER;
         if (m_SkillLevel == (eSkillLevelBeginner+iOffset)) break;
         m_SkillLevel = ESkillLevel(eSkillLevelBeginner+iOffset);
         OnChangeTabPage();
      }
      break;
   case IDOK:
   case IDCANCEL:
      if (codeNotify == BN_CLICKED) {
         ::SendMessage(m_hWnd, WM_CLOSE, 0L, 0L);
      }
      break;
   }
}

void CDialogStatisticsOrChampions::OnNotify(HWND, int idControl, LPNMHDR pNMHdr) {
   switch (pNMHdr->code) {
   case TCN_SELCHANGE:
      m_Mosaic = (EMosaic)TabCtrl_GetCurSel(m_hTabCtrl);
      OnChangeTabPage();
      ModifyCaption();
      break;
   }
}

void CDialogStatistics::OnSize(HWND, UINT state, int cx, int cy) {
   int hB = 30;    //  
   int wB = cx>>2; //  
   ::MoveWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL), 0,0,cx,cy-hB, TRUE);
   ::MoveWindow(m_BtnBeginner.m_hWnd, 0*wB,cy-hB,wB,hB, TRUE);
   ::MoveWindow(m_BtnAmateur .m_hWnd, 1*wB,cy-hB,wB,hB, TRUE);
   ::MoveWindow(m_BtnProfi   .m_hWnd, 2*wB,cy-hB,wB,hB, TRUE);
   ::MoveWindow(m_BtnCrazy   .m_hWnd, 3*wB,cy-hB,wB,hB, TRUE);

   RECT rcTab = ::GetClientRect(m_hTabCtrl);
   RECTEX rcItem; TabCtrl_GetItemRect(m_hTabCtrl, 0, &rcItem);
   RECT rcTbl = {2, rcItem.height()+4, rcTab.right-2, rcTab.bottom-3};
   ::MoveWindow(m_Table.m_hWnd, rcTbl);

   float arrW[] = {0.21f,0.14f,0.14f,0.16f,0.16f,0.19f}; //   
   m_Table.SetColWidth(0, toINT(cx*arrW[0])-1);
   m_Table.SetColWidth(1, toINT(cx*arrW[1])-1);
   m_Table.SetColWidth(2, toINT(cx*arrW[2])-1);
   m_Table.SetColWidth(3, toINT(cx*arrW[3])-1);
   m_Table.SetColWidth(4, toINT(cx*arrW[4])-1);
   m_Table.SetColWidth(5, toINT(cx*arrW[5])-1);
}

void CDialogStatisticsOrChampions::OnClose(HWND) {
   ::DestroyWindow(m_hWnd);
}

void CDialogStatisticsOrChampions::OnDestroy(HWND) {
   if (m_hImageList) {
      ::ImageList_Destroy(m_hImageList); m_hImageList = NULL;
   }
   CDialogMessageHook::UninstallHook(m_hWnd);
   m_hWnd = NULL;
}

BOOL CDialogStatisticsOrChampions::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // ::DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogStatisticsOrChampions::ModifyCaption() {
   ::SetWindowText(m_hWnd, CLang::GetString(GetIdCaptionString()) + _T(" - ") + CLang::GetString(IDS__MOSAIC_NAME_00+m_Mosaic));
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

HWND CDialogStatisticsOrChampions::Create(HWND hWndParent, LPARAM dwInitParam) {
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, this);
#ifdef _DEBUG
	m_bModal = false;
#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CDialogStatisticsOrChampions::IDD(), hWndParent, (DLGPROC)CDialogStatisticsOrChampions::StartDialogProc, dwInitParam);
	ATLASSERT(hWnd == m_hWnd);
	return m_hWnd;
}

BOOL CDialogStatisticsOrChampions::OnInitDialog(HWND hWndFocus, LPARAM lParam) {
   m_Mosaic     = GetFM2WinMain().GetMosaic();
   m_SkillLevel = GetFM2WinMain().GetSkillLevel();
   if (m_SkillLevel == eSkillLevelCustom) m_SkillLevel = eSkillLevelBeginner;
   Button_SetCheck(GetDlgItem(IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER + m_SkillLevel), BST_CHECKED);

   
   
   //SETNEWWNDPROC(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL);
   DoDataExchange();




   static COSVersion OS;
   if (!OS.IsWin9598Me()) {
      ::SetWindowStyle(m_TabCtrl.m_hWnd, ::GetWindowStyle(m_TabCtrl.m_hWnd) |  TCS_OWNERDRAWFIXED);
   } else {
      ::SetWindowStyle(m_TabCtrl.m_hWnd, ::GetWindowStyle(m_TabCtrl.m_hWnd) & ~TCS_OWNERDRAWFIXED);
   }

   if (!OS.IsWin9598Me()) {
      for (int i=0; i<eMosaicNil; i++) {
         m_TabItems[i].Attach(m_TabCtrl.m_hWnd, i, &CFastMines2WinMain::GetImageMosaic((EMosaic)i));
      }
      DWORD dwRes = TabCtrl_SetItemSize(m_TabCtrl.m_hWnd, 0,54);
   } else {
      m_ImageList.Create(32,32, ILC_MASK, eMosaicNil, 1);
      int i=0;
      for (; i<eMosaicNil; i++) {
         m_ImageList.AddIcon(CFastMines2WinMain::GetIconMosaic((EMosaic)i));
      }

      m_TabCtrl.SetImageList(m_ImageList);
      for (i=0; i<eMosaicNil; i++)
         m_TabCtrl.InsertItem(i, TC_Item((EMosaic)i));
   }
   m_TabCtrl.SetCurSel(m_Mosaic);

   OnInitTable();

   m_BtnBeginner.SetWindowText(CLang::GetString(IDS__MENU_GAME__BEGINNER    ));
   m_BtnAmateur .SetWindowText(CLang::GetString(IDS__MENU_GAME__AMATEUR     ));
   m_BtnProfi   .SetWindowText(CLang::GetString(IDS__MENU_GAME__PROFESSIONAL));
   m_BtnCrazy   .SetWindowText(CLang::GetString(IDS__MENU_GAME__CRAZY       ));

   SetWindowLong(GWL_STYLE, WS_MAXIMIZEBOX ^ GetWindowLong(GWL_STYLE));
   //DrawMenuBar();

   SETNEWWNDPROC_HWND(m_TabCtrl.GetDlgItem(1), UPDOWN_CLASS);


   CDialogMessageHook::InstallHook(m_hWnd);

   OnChangeTabPage();
   ModifyCaption();

   SIZE sizeDlg = GetFirstSizeDialog();
   SIZE sizeScreen = ::GetScreenSize();
   MoveWindow(
      (sizeScreen.cx>>1) - (sizeDlg.cx>>1),
      (sizeScreen.cy>>1) - (sizeDlg.cy>>1),
      sizeDlg.cx,
      sizeDlg.cy,
      TRUE);

   return TRUE;
}

void CDialogStatisticsOrChampions::OnClickSkillLevel(UINT /*wNotifyCode*/, int idControl, HWND /*hWndCtl*/) {
   int iOffset = idControl-IDC_STATISTICSorCHAMPIONS_BUTTON_BEGINNER;
   if (m_SkillLevel == (eSkillLevelBeginner+iOffset)) return;
   m_SkillLevel = ESkillLevel(eSkillLevelBeginner+iOffset);
   OnChangeTabPage();
}

void CDialogStatisticsOrChampions::OnClickOkCancel(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   SendMessage(WM_CLOSE, 0L, 0L);
}

LRESULT CDialogStatisticsOrChampions::OnNotify(int idControl, LPNMHDR pNMHdr) {
   switch (pNMHdr->code) {
   case TCN_SELCHANGE:
      m_Mosaic = (EMosaic)m_TabCtrl.GetCurSel();
      OnChangeTabPage();
      ModifyCaption();
      break;
   }
   return 0;
}

void CDialogStatistics::OnSize(UINT state, CSize &size) {
   int hB = 30;    //  
   int wB = size.cx>>2; //  
   m_TabCtrl    .MoveWindow(0,0,size.cx,size.cy-hB, TRUE);
   m_BtnBeginner.MoveWindow(0*wB,size.cy-hB,wB,hB, TRUE);
   m_BtnAmateur .MoveWindow(1*wB,size.cy-hB,wB,hB, TRUE);
   m_BtnProfi   .MoveWindow(2*wB,size.cy-hB,wB,hB, TRUE);
   m_BtnCrazy   .MoveWindow(3*wB,size.cy-hB,wB,hB, TRUE);

   RECT rcTab = ::GetClientRect(m_TabCtrl.m_hWnd);
   RECTEX rcItem; m_TabCtrl.GetItemRect(0, &rcItem);
   RECT rcTbl = {2, rcItem.height()+4, rcTab.right-2, rcTab.bottom-3};
   m_Table.MoveWindow(&rcTbl, TRUE);

   float arrW[] = {0.21f,0.14f,0.14f,0.16f,0.16f,0.19f}; //   
   m_Table.SetColWidth(0, toINT(size.cx*arrW[0])-1);
   m_Table.SetColWidth(1, toINT(size.cx*arrW[1])-1);
   m_Table.SetColWidth(2, toINT(size.cx*arrW[2])-1);
   m_Table.SetColWidth(3, toINT(size.cx*arrW[3])-1);
   m_Table.SetColWidth(4, toINT(size.cx*arrW[4])-1);
   m_Table.SetColWidth(5, toINT(size.cx*arrW[5])-1);
}

void CDialogStatisticsOrChampions::OnClose() {
   DestroyWindow();
}

void CDialogStatisticsOrChampions::OnDestroy() {
   m_ImageList.Destroy();
   CDialogMessageHook::UninstallHook(m_hWnd);

}

BOOL CDialogStatisticsOrChampions::CTabCtrlImpl::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}
BOOL CDialogStatisticsOrChampions::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogStatisticsOrChampions::ModifyCaption() {
   SetWindowText(CLang::GetString(GetIdCaptionString()) + _T(" - ") + CLang::GetString(IDS__MOSAIC_NAME_00+m_Mosaic));
}

#endif // _WTL_VER

void CDialogStatistics::OnInitTable() {
   m_Table.SetColNumber(6);
   m_Table.SetRowNumber(GetFM2WinMain().GetPlayers().NumberPlayers()+1);
   m_Table.SetText(0,0, CLang::GetString(IDS__PLAYERSandSTATISTIC__PLAYER_NAME  ));
   m_Table.SetText(1,0, CLang::GetString(IDS__PLAYERSandSTATISTIC__NUMBER__GAMES));
   m_Table.SetText(2,0, CLang::GetString(IDS__PLAYERSandSTATISTIC__GAMES_WON    ));
   CString strBuf;
   strBuf.Format(_T("%s (%s %d%%)"),
      (LPCTSTR)CLang::GetString(IDS__PLAYERSandSTATISTIC__OPEN),
      (LPCTSTR)CLang::GetString(IDS__MAX),
      100-toINT(CMosaic::GetPercentMine(m_SkillLevel, m_Mosaic)));
   m_Table.SetText(3,0, strBuf);
   m_Table.SetText(4,0, CLang::GetString(IDS__PLAYERSandSTATISTIC__AVERAGED_GAME_TIME    ));
   m_Table.SetText(5,0, CLang::GetString(IDS__PLAYERSandSTATISTIC__AVERAGED_NUMBER_CLICKS));

   int iCurr = GetFM2WinMain().GetPlayers().FindName(GetFM2WinMain().GetPlayerName());
   if (iCurr>=0)
      m_Table.SetCurrentCell(0, iCurr+1, true);
   for (int i=0; i<GetFM2WinMain().GetPlayers().NumberPlayers(); i++) {
      m_Table.SetFormatText(0, i+1, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
      m_Table.SetText(0,i+1, GetFM2WinMain().GetPlayers().GetPlayerName(i));
      m_Table.SetImage(0,i+1, &CImageSerialize(GetFM2WinMain().GetPlayers().GetAvatar(i)));
   }
}

void CDialogStatistics::OnChangeTabPage() {
   CString strBuf;
   strBuf.Format(_T("%s (%s %d%%)"),
      (LPCTSTR)CLang::GetString(IDS__PLAYERSandSTATISTIC__OPEN),
      (LPCTSTR)CLang::GetString(IDS__MAX),
      100-toINT(CMosaic::GetPercentMine(m_SkillLevel, m_Mosaic)));
   m_Table.SetText(3, 0, strBuf);
   for (int i=0; i<GetFM2WinMain().GetPlayers().NumberPlayers(); i++) {
      DWORD dwGameNumber = GetFM2WinMain().GetPlayers().GetInfo_GameNumber(i, m_Mosaic, m_SkillLevel);
      DWORD dwGameWin    = GetFM2WinMain().GetPlayers().GetInfo_GameWin   (i, m_Mosaic, m_SkillLevel);
      DWORD dwOpenField  = GetFM2WinMain().GetPlayers().GetInfo_OpenField (i, m_Mosaic, m_SkillLevel);
      DWORD dwPlayTime   = GetFM2WinMain().GetPlayers().GetInfo_PlayTime  (i, m_Mosaic, m_SkillLevel);
      DWORD dwClickCount = GetFM2WinMain().GetPlayers().GetInfo_ClickCount(i, m_Mosaic, m_SkillLevel);

      //m_Table.SetText(0, i+1, GetFM2WinMain().GetPlayers().GetPlayerName(i));      m_Table.SetFormatText(0, i+1, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
      strBuf.Format(_T("%d"), dwGameNumber);
      m_Table.SetText(1, i+1, strBuf);
      strBuf.Format(_T("%d / %.3f%%"), dwGameWin, ((float)dwGameWin/(float)max(1, dwGameNumber))*100.f);
      m_Table.SetText(2, i+1, strBuf);
      strBuf.Format(_T("%.3f%%"), ((float)dwOpenField /(float)max(1, dwGameNumber))*100.f/(float)(CMosaic::DefaultSize(m_SkillLevel).cx*CMosaic::DefaultSize(m_SkillLevel).cy));
      m_Table.SetText(3, i+1, strBuf);
      if (dwPlayTime)
         strBuf.Format(_T("%.3f %s"),  (float)dwPlayTime/(float)max(1, dwGameWin), (LPCTSTR)CLang::GetString(IDS__SEC));
      else
         strBuf = _T("???");
      m_Table.SetText(4, i+1, strBuf);
      if (dwClickCount)
         strBuf.Format(_T("%.3f"),  (float)dwClickCount/(float)max(1, dwGameWin));
      m_Table.SetText(5, i+1, strBuf);
   }
}
