//
// Dynamic load libraries
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#pragma once

#include <TChar.h>
#include <MAPI.H>
#include <UxTheme.h>
//#pragma comment(lib, "UxTheme.lib") // . DLL   .      .
#if (_WIN32_WINNT >= 0x0501)
   //#include <AtlTheme.h> //    ,     WTL   CTheme, ..  CTheme     XP.
#endif

class CWinXPTheme {
private:
   static HMODULE GetDLL() { static HMODULE hMod = ::LoadLibrary(_T("UxTheme.dll")); return hMod; }
public:
   typedef BOOL    (STDAPICALLTYPE* PF_IsThemeActive                )();
   typedef HTHEME  (STDAPICALLTYPE* PF_OpenThemeData                )(HWND, LPCWSTR);
   typedef HRESULT (STDAPICALLTYPE* PF_CloseThemeData               )(HTHEME);
   typedef HRESULT (STDAPICALLTYPE* PF_GetThemeBackgroundContentRect)(HTHEME, HDC, int, int, const RECT*, RECT*);
   typedef HRESULT (STDAPICALLTYPE* PF_DrawThemeBackground          )(HTHEME, HDC, int, int, const RECT*, const RECT*);
   typedef HRESULT (STDAPICALLTYPE* PF_DrawThemeParentBackground    )(HWND  , HDC, RECT*);
   typedef void    (STDAPICALLTYPE* PF_SetThemeAppProperties        )(DWORD dwFlags);
   static PF_IsThemeActive                 IsThemeActive                () { static PF_IsThemeActive                 func = (PF_IsThemeActive                )::GetProcAddress(GetDLL(), "IsThemeActive"                ); return func; }
   static PF_OpenThemeData                 OpenThemeData                () { static PF_OpenThemeData                 func = (PF_OpenThemeData                )::GetProcAddress(GetDLL(), "OpenThemeData"                ); return func; }
   static PF_CloseThemeData                CloseThemeData               () { static PF_CloseThemeData                func = (PF_CloseThemeData               )::GetProcAddress(GetDLL(), "CloseThemeData"               ); return func; }
   static PF_GetThemeBackgroundContentRect GetThemeBackgroundContentRect() { static PF_GetThemeBackgroundContentRect func = (PF_GetThemeBackgroundContentRect)::GetProcAddress(GetDLL(), "GetThemeBackgroundContentRect"); return func; }
   static PF_DrawThemeBackground           DrawThemeBackground          () { static PF_DrawThemeBackground           func = (PF_DrawThemeBackground          )::GetProcAddress(GetDLL(), "DrawThemeBackground"          ); return func; }
   static PF_DrawThemeParentBackground     DrawThemeParentBackground    () { static PF_DrawThemeParentBackground     func = (PF_DrawThemeParentBackground    )::GetProcAddress(GetDLL(), "DrawThemeParentBackground"    ); return func; }
   static PF_SetThemeAppProperties         SetThemeAppProperties        () { static PF_SetThemeAppProperties         func = (PF_SetThemeAppProperties        )::GetProcAddress(GetDLL(), "SetThemeAppProperties"        ); return func; }
};

class CGDI32Lib {
private:
   static HMODULE GetDLL() { static HMODULE hMod = ::LoadLibrary(_T("GDI32.dll")); return hMod; }
public:
   typedef BOOL (WINAPI* PF_SetWorldTransform)(IN HDC, IN CONST XFORM *);
   static PF_SetWorldTransform SetWorldTransform() { static PF_SetWorldTransform func = (PF_SetWorldTransform)::GetProcAddress(GetDLL(), "SetWorldTransform"); return func; }
};

class CShlWAPILib {
private:
   static HMODULE GetDLL() { static HMODULE hMod = ::LoadLibrary(_T("shlwapi.dll")); return hMod; }
public:
   typedef void     (STDAPICALLTYPE *PF_ColorRGBToHLS)(IN COLORREF, OUT WORD*, OUT WORD*, OUT WORD*);
   typedef COLORREF (STDAPICALLTYPE *PF_ColorHLSToRGB)(IN WORD, IN WORD, IN WORD);
   static PF_ColorRGBToHLS ColorRGBToHLS() { static PF_ColorRGBToHLS func = (PF_ColorRGBToHLS)::GetProcAddress(GetDLL(), "ColorRGBToHLS"); return func; }
   static PF_ColorHLSToRGB ColorHLSToRGB() { static PF_ColorHLSToRGB func = (PF_ColorHLSToRGB)::GetProcAddress(GetDLL(), "ColorHLSToRGB"); return func; }
};

class CKernel32Lib {
private:
   static HMODULE GetDLL() { static HMODULE hMod = ::LoadLibrary(_T("Kernel32.dll")); return hMod; }
public:
   typedef BOOL (WINAPI *PF_CreateHardLinkW)(IN LPCWSTR, IN LPCWSTR, LPSECURITY_ATTRIBUTES);
   typedef BOOL (WINAPI *PF_CreateHardLinkA)(IN LPCSTR , IN LPCSTR , LPSECURITY_ATTRIBUTES);
   static PF_CreateHardLinkW CreateHardLinkW() { static PF_CreateHardLinkW func = (PF_CreateHardLinkW)::GetProcAddress(GetDLL(), "CreateHardLinkW"); return func; }
   static PF_CreateHardLinkA CreateHardLinkA() { static PF_CreateHardLinkA func = (PF_CreateHardLinkA)::GetProcAddress(GetDLL(), "CreateHardLinkA"); return func; }
};

class CMapi32Lib {
private:
   static HMODULE GetDLL() { static HMODULE hMod = ::LoadLibrary(_T("Mapi32.dll")); return hMod; }
public:
   typedef ULONG (FAR PASCAL *PF_MAPISendMail)(LHANDLE lhSession, ULONG_PTR ulUIParam, lpMapiMessage lpMessage, FLAGS flFlags, ULONG ulReserved );
   typedef ULONG (FAR PASCAL *PF_MAPILogon   )(ULONG_PTR ulUIParam, LPSTR lpszProfileName, LPSTR lpszPassword, FLAGS flFlags, ULONG ulReserved, LPLHANDLE lplhSession);
   typedef ULONG (FAR PASCAL *PF_MAPILogoff  )(LHANDLE lhSession, ULONG_PTR ulUIParam, FLAGS flFlags, ULONG ulReserved);
   static PF_MAPISendMail MAPISendMail() { static PF_MAPISendMail func = (PF_MAPISendMail)::GetProcAddress(GetDLL(), "MAPISendMail"); return func; }
   static PF_MAPILogon    MAPILogon   () { static PF_MAPILogon    func = (PF_MAPILogon   )::GetProcAddress(GetDLL(), "MAPILogon"   ); return func; }
   static PF_MAPILogoff   MAPILogoff  () { static PF_MAPILogoff   func = (PF_MAPILogoff  )::GetProcAddress(GetDLL(), "MAPILogoff"  ); return func; }
};
