// Help functions - intercept Win32 API and others functions
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#include "stdafx.h"

#if !defined(__AFX_H__) && !defined(_WTL_USE_CSTRING)
   #include "./Lib/CStringKS.h"
#endif
#include "./Lib/Logger.h"
#include "./Lib/CommonLib.h"
#include <detours.h> // using Detours API: Binary Interception of Win32 Functions.
                       // download from http://research.microsoft.com/sn/detours/
                       //  
#ifdef _DETOURS_H_
   #pragma comment(lib, "detours")
   #ifndef DETOUR_TRAMPOLINE_SIZE // defined Detours API ver.1.5 (exclude ver.2.1)
      #pragma comment(lib, "detoured")
   #endif
#endif _DETOURS_H_

#include <ImageHlp.h>
#pragma comment(lib, "ImageHlp")
//#pragma comment(lib, "DbgHelp")

//#pragma comment(lib, "MsImg32") // GradientFill

#ifdef _DEBUG
   #define new DEBUG_NEW
   #undef THIS_FILE
   static char THIS_FILE[] = __FILE__;
#endif


/*******************************************************************************************/
//
//           -
//
/*******************************************************************************************/

typedef BOOL (WINAPI* PF_InvalidateRect)(IN HWND, IN CONST RECT*, IN BOOL);
PF_InvalidateRect& g_pfInvalidateRect() { static PF_InvalidateRect func = ::InvalidateRect; return func; }
BOOL WINAPI My_InvalidateRect(IN HWND hWnd, IN CONST RECT *lpRect, IN BOOL bErase) {
   ::SetLastError(0);
   BOOL bRes = g_pfInvalidateRect()(hWnd, lpRect, bErase);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  InvalidateRect: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef HGDIOBJ (WINAPI *PF_SelectObject)(IN HDC, IN HGDIOBJ);
PF_SelectObject& g_pfSelectObject() { static PF_SelectObject func = ::SelectObject; return func; }
HGDIOBJ WINAPI My_SelectObject(IN HDC hDC, IN HGDIOBJ hGdiObj) {
   //::SetLastError(0);
   HGDIOBJ hObj = g_pfSelectObject()(hDC, hGdiObj);
   //DWORD dwErrCode = GetLastError();
   if (!hObj || (hObj == HGDI_ERROR)) {
    //LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SelectObject: "), dwErrCode);
      LOG_PUT(g_pLog, Put, LL_ERROR) _T("  SelectObject: "));
   }
   //::SetLastError(dwErrCode);
   return hObj;
}

typedef DWORD (WINAPI *PF_GetObjectType)(IN HGDIOBJ);
PF_GetObjectType& g_pfGetObjectType() { static PF_GetObjectType func = ::GetObjectType; return func; }
DWORD WINAPI My_GetObjectType(IN HGDIOBJ hGdiObj) {
   ::SetLastError(0);
   DWORD dwType = g_pfGetObjectType()(hGdiObj);
   DWORD dwErrCode = GetLastError();
   if (dwType == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetObjectType: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return dwType;
}

typedef BOOL (WINAPI* PF_MoveToEx)(IN HDC, IN int, IN int, OUT LPPOINT);
PF_MoveToEx& g_pfMoveToEx() { static PF_MoveToEx func = ::MoveToEx; return func; }
BOOL WINAPI My_MoveToEx(IN HDC hdc, IN int X, IN int Y, OUT LPPOINT lpPoint) {
   ::SetLastError(0);
   BOOL bRes = g_pfMoveToEx()(hdc, X, Y, lpPoint);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  MoveToEx: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_LineTo)(IN HDC, IN int, IN int);
PF_LineTo& g_pfLineTo() { static PF_LineTo func = ::LineTo; return func; }
BOOL WINAPI My_LineTo(IN HDC hdc, IN int nXEnd , IN int nYEnd) {
   ::SetLastError(0);
   BOOL bRes = g_pfLineTo()(hdc, nXEnd , nYEnd);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  LineTo: "), dwErrCode); // !!!
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_BitBlt)(IN HDC, IN int, IN int, IN int, IN int, IN HDC, IN int, IN int, IN DWORD);
PF_BitBlt& g_pfBitBlt() { static PF_BitBlt func = (PF_BitBlt)::BitBlt; return func; }
BOOL WINAPI My_BitBlt(HDC hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, HDC hdcSrc, int nXSrc, int nYSrc, DWORD dwRop) {
   ::SetLastError(0);
   BOOL bRes = g_pfBitBlt()(hdcDest, nXDest, nYDest, nWidth, nHeight, hdcSrc, nXSrc, nYSrc, dwRop);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  BitBlt: "), dwErrCode); // !!!
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_PatBlt)(IN HDC, IN int, IN int, IN int, IN int, IN DWORD);
PF_PatBlt& g_pfPatBlt() { static PF_PatBlt func = ::PatBlt; return func; }
BOOL WINAPI My_PatBlt(HDC hdc, int nXLeft, int nYLeft, int nWidth, int nHeight, DWORD dwRop) {
   ::SetLastError(0);
   BOOL bRes = g_pfPatBlt()(hdc, nXLeft, nYLeft, nWidth, nHeight, dwRop);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  PatBlt: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_StretchBlt)(IN HDC, IN int, IN int, IN int, IN int, IN HDC, IN int, IN int, IN int, IN int, IN DWORD);
PF_StretchBlt& g_pfStretchBlt() { static PF_StretchBlt func = (PF_StretchBlt)::StretchBlt; return func; }
BOOL WINAPI My_StretchBlt(HDC hdcDest, int nXOriginDest, int nYOriginDest, int nWidthDest, int nHeightDest, HDC hdcSrc, int nXOriginSrc, int nYOriginSrc, int nWidthSrc, int nHeightSrc, DWORD dwRop) {
   ::SetLastError(0);
   BOOL bRes = g_pfStretchBlt()(hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, hdcSrc, nXOriginSrc, nYOriginSrc, nWidthSrc, nHeightSrc, dwRop);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  StretchBlt: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef COLORREF (WINAPI *PF_SetTextColor)(IN HDC, IN COLORREF);
PF_SetTextColor& g_pfSetTextColor() { static PF_SetTextColor func = ::SetTextColor; return func; }
COLORREF WINAPI My_SetTextColor(HDC hdc, COLORREF crColor) {
   ::SetLastError(0);
   COLORREF color = g_pfSetTextColor()(hdc, crColor);
   DWORD dwErrCode = GetLastError();
   if (color == CLR_INVALID) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetTextColor: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return color;
}

typedef COLORREF (WINAPI *PF_SetBkColor)(IN HDC, IN COLORREF);
PF_SetBkColor& g_pfSetBkColor() { static PF_SetBkColor func = ::SetBkColor; return func; }
COLORREF WINAPI My_SetBkColor(HDC hdc, COLORREF crColor) {
   ::SetLastError(0);
   COLORREF color = g_pfSetBkColor()(hdc, crColor);
   DWORD dwErrCode = GetLastError();
   if (color == CLR_INVALID) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetBkColor: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return color;
}

typedef int (WINAPI* PF_DrawTextA)(IN HDC, IN LPCSTR, IN int, IN OUT LPRECT, IN UINT);
PF_DrawTextA& g_pfDrawTextA() { static PF_DrawTextA func = ::DrawTextA; return func; }
int WINAPI My_DrawTextA(IN HDC hDC, IN LPCSTR lpString, IN int nCount, IN OUT LPRECT lpRect, IN UINT uFormat) {
   ::SetLastError(0);
   int iRes = g_pfDrawTextA()(hDC, lpString, nCount, lpRect, uFormat);
   DWORD dwErrCode = GetLastError();
   if (iRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DrawTextA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef int (WINAPI* PF_DrawTextW)(IN HDC, IN LPCWSTR, IN int, IN OUT LPRECT, IN UINT);
PF_DrawTextW& g_pfDrawTextW() { static PF_DrawTextW func = ::DrawTextW; return func; }
int WINAPI My_DrawTextW(IN HDC hDC, IN LPCWSTR lpString, IN int nCount, IN OUT LPRECT lpRect, IN UINT uFormat) {
   ::SetLastError(0);
   int iRes = g_pfDrawTextW()(hDC, lpString, nCount, lpRect, uFormat);
   DWORD dwErrCode = GetLastError();
   if (iRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DrawTextW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef HDC (WINAPI* PF_CreateCompatibleDC)(IN HDC);
PF_CreateCompatibleDC& g_pfCreateCompatibleDC() { static PF_CreateCompatibleDC func = ::CreateCompatibleDC; return func; }
HDC WINAPI My_CreateCompatibleDC(IN HDC hDC) {
   ::SetLastError(0);
   HDC hDCRes = g_pfCreateCompatibleDC()(hDC);
   DWORD dwErrCode = GetLastError();
   if (!hDCRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateCompatibleDC: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hDCRes;
}

typedef BOOL (WINAPI* PF_DeleteDC)(IN HDC);
PF_DeleteDC& g_pfDeleteDC() { static PF_DeleteDC func = ::DeleteDC; return func; }
BOOL WINAPI My_DeleteDC(IN HDC hDC) {
   ::SetLastError(0);
   BOOL bRes = g_pfDeleteDC()(hDC);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DeleteDC: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_DeleteObject)(IN HGDIOBJ);
PF_DeleteObject& g_pfDeleteObject() { static PF_DeleteObject func = ::DeleteObject; return func; }
BOOL WINAPI My_DeleteObject(IN HGDIOBJ hObject) {
   ::SetLastError(0);
   BOOL bRes = g_pfDeleteObject()(hObject);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DeleteObject: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef HBITMAP (WINAPI* PF_CreateCompatibleBitmap)(IN HDC, IN int, IN int);
PF_CreateCompatibleBitmap& g_pfCreateCompatibleBitmap() { static PF_CreateCompatibleBitmap func = ::CreateCompatibleBitmap; return func; }
HBITMAP WINAPI My_CreateCompatibleBitmap(IN HDC hdc, IN int nWidth, IN int nHeight) {
   ::SetLastError(0);
   HBITMAP hBMP = g_pfCreateCompatibleBitmap()(hdc, nWidth, nHeight);
   DWORD dwErrCode = GetLastError();
   if (!hBMP) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateCompatibleBitmap: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hBMP;
}

typedef HBITMAP (WINAPI* PF_CreateBitmap)(IN int, IN int, IN UINT, IN UINT, IN CONST VOID *);
PF_CreateBitmap& g_pfCreateBitmap() { static PF_CreateBitmap func = (PF_CreateBitmap)::CreateBitmap; return func; }
HBITMAP WINAPI My_CreateBitmap(IN int nWidth, IN int nHeight, IN UINT cPlanes, IN UINT cBitsPerPel, IN CONST VOID *lpvBits) {
   ::SetLastError(0);
   HBITMAP hBMP = g_pfCreateBitmap()(nWidth, nHeight, cPlanes, cBitsPerPel, lpvBits);
   DWORD dwErrCode = GetLastError();
   if (!hBMP) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateBitmap: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hBMP;
}

typedef HBITMAP (WINAPI* PF_LoadBitmapA)(IN HINSTANCE, IN LPCSTR);
PF_LoadBitmapA& g_pfLoadBitmapA() { static PF_LoadBitmapA func = ::LoadBitmapA; return func; }
HBITMAP WINAPI My_LoadBitmapA(IN HINSTANCE hInstance, IN LPCSTR lpBitmapName) {
   ::SetLastError(0);
   HBITMAP hBMP = g_pfLoadBitmapA()(hInstance, lpBitmapName);
   DWORD dwErrCode = GetLastError();
   if (!hBMP) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  LoadBitmapA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hBMP;
}

typedef HBITMAP (WINAPI* PF_LoadBitmapW)(IN HINSTANCE, IN LPCWSTR);
PF_LoadBitmapW& g_pfLoadBitmapW() { static PF_LoadBitmapW func = ::LoadBitmapW; return func; }
HBITMAP WINAPI My_LoadBitmapW(IN HINSTANCE hInstance, IN LPCWSTR lpBitmapName) {
   ::SetLastError(0);
   HBITMAP hBMP = g_pfLoadBitmapW()(hInstance, lpBitmapName);
   DWORD dwErrCode = GetLastError();
   if (!hBMP) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  LoadBitmapW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hBMP;
}

typedef HANDLE (WINAPI* PF_LoadImageA)(IN HINSTANCE, IN LPCSTR, IN UINT, IN int, IN int, IN UINT);
PF_LoadImageA& g_pfLoadImageA() { static PF_LoadImageA func = ::LoadImageA; return func; }
HANDLE WINAPI My_LoadImageA(HINSTANCE hInst, LPCSTR lpszName, UINT uType, int cxDesired, int cyDesired, UINT fuLoad) {
   ::SetLastError(0);
   HANDLE hImage = g_pfLoadImageA()(hInst, lpszName, uType, cxDesired, cyDesired, fuLoad);
   DWORD dwErrCode = GetLastError();
   if (!hImage) {
      //LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  LoadImageA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hImage;
}

typedef HANDLE (WINAPI* PF_LoadImageW)(IN HINSTANCE, IN LPCWSTR, IN UINT, IN int, IN int, IN UINT);
PF_LoadImageW& g_pfLoadImageW() { static PF_LoadImageW func = ::LoadImageW; return func; }
HANDLE WINAPI My_LoadImageW(HINSTANCE hInst, LPCWSTR lpszName, UINT uType, int cxDesired, int cyDesired, UINT fuLoad) {
   ::SetLastError(0);
   HANDLE hImage = g_pfLoadImageW()(hInst, lpszName, uType, cxDesired, cyDesired, fuLoad);
   DWORD dwErrCode = GetLastError();
   if (!hImage) {
      //LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  LoadImageW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hImage;
}

typedef BOOL (WINAPI* PF_Polygon)(IN HDC, IN CONST POINT *, IN int);
PF_Polygon& g_pfPolygon() { static PF_Polygon func = ::Polygon; return func; }
BOOL WINAPI My_Polygon(IN HDC hdc, IN CONST POINT *lpPoints, IN int nCount) {
   ::SetLastError(0);
   BOOL bRes = g_pfPolygon()(hdc, lpPoints, nCount);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  Polygon: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef int (WINAPI* PF_GetObjectA)(IN HGDIOBJ, IN int, OUT LPVOID);
PF_GetObjectA& g_pfGetObjectA() { static PF_GetObjectA func = ::GetObjectA; return func; }
int WINAPI My_GetObjectA(IN HGDIOBJ hgdiobj, IN int cbBuffer, IN LPVOID lpvObject) {
   ::SetLastError(0);
   int iRes = g_pfGetObjectA()(hgdiobj, cbBuffer, lpvObject);
   DWORD dwErrCode = GetLastError();
   if (iRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetObjectA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef int (WINAPI* PF_GetObjectW)(IN HGDIOBJ, IN int, OUT LPVOID);
PF_GetObjectW& g_pfGetObjectW() { static PF_GetObjectW func = ::GetObjectW; return func; }
int WINAPI My_GetObjectW(IN HGDIOBJ hgdiobj, IN int cbBuffer, IN LPVOID lpvObject) {
   ::SetLastError(0);
   int iRes = g_pfGetObjectW()(hgdiobj, cbBuffer, lpvObject);
   DWORD dwErrCode = GetLastError();
   if (iRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetObjectW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef BOOL (WINAPI* PF_DrawIconEx)(IN HDC, IN int, IN int, IN HICON, IN int, IN int, IN UINT, IN HBRUSH, IN UINT);
PF_DrawIconEx& g_pfDrawIconEx() { static PF_DrawIconEx func = ::DrawIconEx; return func; }
BOOL WINAPI My_DrawIconEx(IN HDC hdc, IN int xLeft, IN int yTop, IN HICON hIcon, IN int cxWidth, IN int cyWidth, IN UINT istepIfAniCur, IN HBRUSH hbrFlickerFreeDraw, IN UINT diFlags) {
   ::SetLastError(0);
   BOOL bRes = g_pfDrawIconEx()(hdc, xLeft, yTop, hIcon, cxWidth, cyWidth, istepIfAniCur, hbrFlickerFreeDraw, diFlags);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DrawIconEx: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_GetIconInfo)(IN HICON, OUT PICONINFO);
PF_GetIconInfo& g_pfGetIconInfo() { static PF_GetIconInfo func = ::GetIconInfo; return func; }
BOOL WINAPI My_GetIconInfo(IN HICON hIcon, OUT PICONINFO piconinfo) {
   ::SetLastError(0);
   BOOL bRes = g_pfGetIconInfo()(hIcon, piconinfo);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetIconInfo: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_DestroyIcon)(IN HICON);
PF_DestroyIcon& g_pfDestroyIcon() { static PF_DestroyIcon func = ::DestroyIcon; return func; }
BOOL WINAPI My_DestroyIcon(IN HICON hIcon) {
   ::SetLastError(0);
   BOOL bRes = g_pfDestroyIcon()(hIcon);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DestroyIcon: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_PlayEnhMetaFile)(IN HDC, IN HENHMETAFILE, IN CONST RECT *);
PF_PlayEnhMetaFile& g_pfPlayEnhMetaFile() { static PF_PlayEnhMetaFile func = ::PlayEnhMetaFile; return func; }
BOOL WINAPI My_PlayEnhMetaFile(IN HDC hdc, IN HENHMETAFILE hemf, IN CONST RECT *lpRect) {
   ::SetLastError(0);
   BOOL bRes = g_pfPlayEnhMetaFile()(hdc, hemf, lpRect);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  PlayEnhMetaFile: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef UINT (WINAPI* PF_GetEnhMetaFileHeader)(IN HENHMETAFILE, IN UINT, OUT LPENHMETAHEADER);
PF_GetEnhMetaFileHeader& g_pfGetEnhMetaFileHeader() { static PF_GetEnhMetaFileHeader func = ::GetEnhMetaFileHeader; return func; }
UINT WINAPI My_GetEnhMetaFileHeader(IN HENHMETAFILE hemf, IN UINT cbBuffer, IN LPENHMETAHEADER lpemh) {
   ::SetLastError(0);
   UINT uiRes = g_pfGetEnhMetaFileHeader()(hemf, cbBuffer, lpemh);
   DWORD dwErrCode = GetLastError();
   if (uiRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetEnhMetaFileHeader: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return uiRes;
}

typedef BOOL (WINAPI* PF_DeleteEnhMetaFile)(IN HENHMETAFILE);
PF_DeleteEnhMetaFile& g_pfDeleteEnhMetaFile() { static PF_DeleteEnhMetaFile func = ::DeleteEnhMetaFile; return func; }
BOOL WINAPI My_DeleteEnhMetaFile(IN HENHMETAFILE hemf) {
   ::SetLastError(0);
   BOOL bRes = g_pfDeleteEnhMetaFile()(hemf);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DeleteEnhMetaFile: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef int (WINAPI* PF_SetStretchBltMode)(IN HDC, IN int);
PF_SetStretchBltMode& g_pfSetStretchBltMode() { static PF_SetStretchBltMode func = ::SetStretchBltMode; return func; }
int WINAPI My_SetStretchBltMode(IN HDC hdc, IN int iStretchMode) {
   ::SetLastError(0);
   int iRes = g_pfSetStretchBltMode()(hdc, iStretchMode);
   DWORD dwErrCode = GetLastError();
   if (iRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetStretchBltMode: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef BOOL (WINAPI* PF_SetEvent)(IN HANDLE);
PF_SetEvent& g_pfSetEvent() { static PF_SetEvent func = ::SetEvent; return func; }
BOOL WINAPI My_SetEvent(IN HANDLE hEvent) {
   ::SetLastError(0);
   BOOL bRes = g_pfSetEvent()(hEvent);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetEvent: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_ResetEvent)(IN HANDLE);
PF_ResetEvent& g_pfResetEvent() { static PF_ResetEvent func = ::ResetEvent; return func; }
BOOL WINAPI My_ResetEvent(IN HANDLE hEvent) {
   ::SetLastError(0);
   BOOL bRes = g_pfResetEvent()(hEvent);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  ResetEvent: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef DWORD (WINAPI* PF_WaitForSingleObject)(IN HANDLE, IN DWORD);
PF_WaitForSingleObject& g_pfWaitForSingleObject() { static PF_WaitForSingleObject func = ::WaitForSingleObject; return func; }
DWORD WINAPI My_WaitForSingleObject(IN HANDLE hHandle, IN DWORD dwMilliseconds) {
   ::SetLastError(0);
   DWORD dwRes = g_pfWaitForSingleObject()(hHandle, dwMilliseconds);
   DWORD dwErrCode = GetLastError();
   switch (dwRes) {
   case WAIT_OBJECT_0:
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WaitForSingleObject: WAIT_OBJECT_0 "));
      break;
   case WAIT_TIMEOUT:
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WaitForSingleObject: WAIT_TIMEOUT "));
      break;
   case WAIT_ABANDONED:
      LOG_PUT(g_pLog, Put, LL_ERROR) _T("  WaitForSingleObject: WAIT_ABANDONED "));
      break;
   case WAIT_FAILED:
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  WaitForSingleObject: WAIT_FAILED: "), GetLastError());
      break;
   default:
      LOG_PUT(g_pLog, Put, LL_ERROR) _T("  WaitForSingleObject: WAIT_OBJECT_0+%i "), dwRes-WAIT_OBJECT_0);
      break;
   }
   ::SetLastError(dwErrCode);
   return dwRes;
}

typedef DWORD (WINAPI* PF_WaitForMultipleObjects)(IN DWORD, IN CONST HANDLE*, IN BOOL, IN DWORD);
PF_WaitForMultipleObjects& g_pfWaitForMultipleObjects() { static PF_WaitForMultipleObjects func = ::WaitForMultipleObjects; return func; }
DWORD WINAPI My_WaitForMultipleObjects(IN DWORD nCount, IN CONST HANDLE *lpHandles, IN BOOL bWaitAll, IN DWORD dwMilliseconds) {
   ::SetLastError(0);
   DWORD dwRes = g_pfWaitForMultipleObjects()(nCount, lpHandles, bWaitAll, dwMilliseconds);
   DWORD dwErrCode = GetLastError();
   switch (dwRes) {
   case WAIT_OBJECT_0:
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WaitForMultipleObjects: WAIT_OBJECT_0 "));
      break;
   case WAIT_TIMEOUT:
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WaitForMultipleObjects: WAIT_TIMEOUT "));
      break;
   case WAIT_ABANDONED:
      LOG_PUT(g_pLog, Put, LL_ERROR) _T("  WaitForMultipleObjects: WAIT_ABANDONED "));
      break;
   case WAIT_FAILED:
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  WaitForMultipleObjects: WAIT_FAILED: "), GetLastError());
      break;
   default:
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WaitForMultipleObjects: WAIT_OBJECT_0+%i "), dwRes-WAIT_OBJECT_0);
      break;
   }
   ::SetLastError(dwErrCode);
   return dwRes;
}

typedef HBRUSH (WINAPI *PF_CreateSolidBrush)(IN COLORREF);
PF_CreateSolidBrush& g_pfCreateSolidBrush() { static PF_CreateSolidBrush func = ::CreateSolidBrush; return func; }
HBRUSH WINAPI My_CreateSolidBrush(COLORREF crColor) {
   ::SetLastError(0);
   HBRUSH hBrush = g_pfCreateSolidBrush()(crColor);
   DWORD dwErrCode = GetLastError();
   if (!hBrush) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateSolidBrush: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hBrush;
}

typedef BOOL (WINAPI* PF_GradientFill)(IN HDC, IN PTRIVERTEX, IN ULONG, IN PVOID, IN ULONG, IN ULONG);
PF_GradientFill& g_pfGradientFill() { static PF_GradientFill func = ::GradientFill; return func; }
BOOL WINAPI My_GradientFill(IN HDC hdc, IN PTRIVERTEX pVertex, IN ULONG dwNumVertex, IN PVOID pMesh, IN ULONG dwNumMesh, IN ULONG dwMode) {
   ::SetLastError(0);
   BOOL bRes = g_pfGradientFill()(hdc, pVertex, dwNumVertex, pMesh, dwNumMesh, dwMode);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GradientFill: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_DeleteFileA)(IN LPCSTR);
PF_DeleteFileA& g_pfDeleteFileA() { static PF_DeleteFileA func = ::DeleteFileA; return func; }
BOOL WINAPI My_DeleteFileA(IN LPCSTR lpFileName) {
   ::SetLastError(0);
   BOOL bRes = g_pfDeleteFileA()(lpFileName);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) ::Format(_T("  DeleteFileA: file=[%s] "), (LPCTSTR)CString(lpFileName)), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_DeleteFileW)(IN LPCWSTR);
PF_DeleteFileW& g_pfDeleteFileW() { static PF_DeleteFileW func = ::DeleteFileW; return func; }
BOOL WINAPI My_DeleteFileW(IN LPCWSTR lpFileName) {
   ::SetLastError(0);
   BOOL bRes = g_pfDeleteFileW()(lpFileName);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) ::Format(_T("  DeleteFileW: file=[%s] "), (LPCTSTR)CString(lpFileName)), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef HANDLE (WINAPI* PF_CreateFileA)(IN LPCSTR, IN DWORD, IN DWORD, IN LPSECURITY_ATTRIBUTES, IN DWORD, IN DWORD, IN HANDLE);
PF_CreateFileA& g_pfCreateFileA() { static PF_CreateFileA func = ::CreateFileA; return func; }
HANDLE WINAPI My_CreateFileA(IN LPCSTR lpFileName, IN DWORD dwDesiredAccess, IN DWORD dwShareMode, IN LPSECURITY_ATTRIBUTES lpSecurityAttributes, IN DWORD dwCreationDisposition, IN DWORD dwFlagsAndAttributes, IN HANDLE hTemplateFile) {
   ::SetLastError(0);
   HANDLE hRes = g_pfCreateFileA()(lpFileName, dwDesiredAccess, dwShareMode, lpSecurityAttributes, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile);
   DWORD dwErrCode = GetLastError();
   if (!hRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) ::Format(_T("  CreateFileA: file=[%s] "), (LPCTSTR)CString(lpFileName)), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hRes;
}

typedef HANDLE (WINAPI* PF_CreateFileW)(IN LPCWSTR, IN DWORD, IN DWORD, IN LPSECURITY_ATTRIBUTES, IN DWORD, IN DWORD, IN HANDLE);
PF_CreateFileW& g_pfCreateFileW() { static PF_CreateFileW func = ::CreateFileW; return func; }
HANDLE WINAPI My_CreateFileW(IN LPCWSTR lpFileName, IN DWORD dwDesiredAccess, IN DWORD dwShareMode, IN LPSECURITY_ATTRIBUTES lpSecurityAttributes, IN DWORD dwCreationDisposition, IN DWORD dwFlagsAndAttributes, IN HANDLE hTemplateFile) {
   ::SetLastError(0);
   HANDLE hRes = g_pfCreateFileW()(lpFileName, dwDesiredAccess, dwShareMode, lpSecurityAttributes, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile);
   DWORD dwErrCode = GetLastError();
   if (!hRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) ::Format(_T("  CreateFileW: file=[%s] "), (LPCTSTR)CString(lpFileName)), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hRes;
}

typedef BOOL (WINAPI* PF_ReadFile)(IN HANDLE, OUT LPVOID, IN DWORD, OUT LPDWORD, IN LPOVERLAPPED);
PF_ReadFile& g_pfReadFile() { static PF_ReadFile func = ::ReadFile; return func; }
BOOL WINAPI My_ReadFile(IN HANDLE hFile, OUT LPVOID lpBuffer, IN DWORD nNumberOfBytesToRead, OUT LPDWORD lpNumberOfBytesRead, IN LPOVERLAPPED lpOverlapped) {
   ::SetLastError(0);
   BOOL bRes = g_pfReadFile()(hFile, lpBuffer, nNumberOfBytesToRead, lpNumberOfBytesRead, lpOverlapped);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  ReadFile: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_WriteFile)(IN HANDLE, IN LPCVOID, IN DWORD, OUT LPDWORD, IN LPOVERLAPPED);
PF_WriteFile& g_pfWriteFile() { static PF_WriteFile func = ::WriteFile; return func; }
BOOL WINAPI My_WriteFile(IN HANDLE hFile, IN LPCVOID lpBuffer, IN DWORD nNumberOfBytesToWrite, OUT LPDWORD lpNumberOfBytesWritten, IN LPOVERLAPPED lpOverlapped) {
   ::SetLastError(0);
   BOOL bRes = g_pfWriteFile()(hFile, lpBuffer, nNumberOfBytesToWrite, lpNumberOfBytesWritten, lpOverlapped);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  WriteFile: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_CloseHandle)(IN OUT HANDLE);
PF_CloseHandle& g_pfCloseHandle() { static PF_CloseHandle func = ::CloseHandle; return func; }
BOOL WINAPI My_CloseHandle(IN OUT HANDLE hObject) {
   ::SetLastError(0);
   BOOL bRes = g_pfCloseHandle()(hObject);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CloseHandle: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef HWND (WINAPI* PF_CreateDialogParamA)(IN HINSTANCE, IN LPCSTR, IN HWND, IN DLGPROC, IN LPARAM);
PF_CreateDialogParamA& g_pfCreateDialogParamA() { static PF_CreateDialogParamA func = ::CreateDialogParamA; return func; }
HWND WINAPI My_CreateDialogParamA(IN HINSTANCE hInstance, IN LPCSTR lpTemplateName, IN HWND hWndParent, IN DLGPROC lpDialogFunc, IN LPARAM dwInitParam) {
   ::SetLastError(0);
   HWND hWnd = g_pfCreateDialogParamA()(hInstance, lpTemplateName, hWndParent, lpDialogFunc, dwInitParam);
   DWORD dwErrCode = GetLastError();
   if (!hWnd) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateDialogParamA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hWnd;
}

typedef HWND (WINAPI* PF_CreateDialogParamW)(IN HINSTANCE, IN LPCWSTR, IN HWND, IN DLGPROC, IN LPARAM);
PF_CreateDialogParamW& g_pfCreateDialogParamW() { static PF_CreateDialogParamW func = ::CreateDialogParamW; return func; }
HWND WINAPI My_CreateDialogParamW(IN HINSTANCE hInstance, IN LPCWSTR lpTemplateName, IN HWND hWndParent, IN DLGPROC lpDialogFunc, IN LPARAM dwInitParam) {
   ::SetLastError(0);
   HWND hWnd = g_pfCreateDialogParamW()(hInstance, lpTemplateName, hWndParent, lpDialogFunc, dwInitParam);
   DWORD dwErrCode = GetLastError();
   if (!hWnd) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateDialogParamW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hWnd;
}

typedef INT_PTR (WINAPI* PF_DialogBoxParamA)(IN HINSTANCE, IN LPCSTR, IN HWND, IN DLGPROC, IN LPARAM);
PF_DialogBoxParamA& g_pfDialogBoxParamA() { static PF_DialogBoxParamA func = ::DialogBoxParamA; return func; }
INT_PTR WINAPI My_DialogBoxParamA(IN HINSTANCE hInstance, IN LPCSTR lpTemplateName, IN HWND hWndParent, IN DLGPROC lpDialogFunc, IN LPARAM dwInitParam) {
   ::SetLastError(0);
   INT_PTR iRes = g_pfDialogBoxParamA()(hInstance, lpTemplateName, hWndParent, lpDialogFunc, dwInitParam);
   DWORD dwErrCode = GetLastError();
   if (iRes == -1) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DialogBoxParamA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef INT_PTR (WINAPI* PF_DialogBoxParamW)(IN HINSTANCE, IN LPCWSTR, IN HWND, IN DLGPROC, IN LPARAM);
PF_DialogBoxParamW& g_pfDialogBoxParamW() { static PF_DialogBoxParamW func = ::DialogBoxParamW; return func; }
INT_PTR WINAPI My_DialogBoxParamW(IN HINSTANCE hInstance, IN LPCWSTR lpTemplateName, IN HWND hWndParent, IN DLGPROC lpDialogFunc, IN LPARAM dwInitParam) {
   ::SetLastError(0);
   INT_PTR iRes = g_pfDialogBoxParamW()(hInstance, lpTemplateName, hWndParent, lpDialogFunc, dwInitParam);
   DWORD dwErrCode = GetLastError();
   if (iRes == -1) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DialogBoxParamW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef HWND (WINAPI* PF_CreateWindowExA)(IN DWORD, IN LPCSTR, IN LPCSTR, IN DWORD, IN int, IN int, IN int, IN int, IN HWND, IN HMENU, IN HINSTANCE, IN LPVOID);
PF_CreateWindowExA& g_pfCreateWindowExA() { static PF_CreateWindowExA func = ::CreateWindowExA; return func; }
HWND WINAPI My_CreateWindowExA(IN DWORD dwExStyle, IN LPCSTR lpClassName, IN LPCSTR lpWindowName, IN DWORD dwStyle, IN int X, IN int Y, IN int nWidth, IN int nHeight, IN HWND hWndParent, IN HMENU hMenu, IN HINSTANCE hInstance, IN LPVOID lpParam) {
   ::SetLastError(0);
   HWND hWnd = g_pfCreateWindowExA()(dwExStyle, lpClassName, lpWindowName, dwStyle, X, Y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam);
   DWORD dwErrCode = GetLastError();
   if (!hWnd) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateWindowExA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hWnd;
}

typedef HWND (WINAPI* PF_CreateWindowExW)(IN DWORD, IN LPCWSTR, IN LPCWSTR, IN DWORD, IN int, IN int, IN int, IN int, IN HWND, IN HMENU, IN HINSTANCE, IN LPVOID);
PF_CreateWindowExW& g_pfCreateWindowExW() { static PF_CreateWindowExW func = ::CreateWindowExW; return func; }
HWND WINAPI My_CreateWindowExW(IN DWORD dwExStyle, IN LPCWSTR lpClassName, IN LPCWSTR lpWindowName, IN DWORD dwStyle, IN int X, IN int Y, IN int nWidth, IN int nHeight, IN HWND hWndParent, IN HMENU hMenu, IN HINSTANCE hInstance, IN LPVOID lpParam) {
   ::SetLastError(0);
   HWND hWnd = g_pfCreateWindowExW()(dwExStyle, lpClassName, lpWindowName, dwStyle, X, Y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam);
   DWORD dwErrCode = GetLastError();
   if (!hWnd) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  CreateWindowExW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hWnd;
}

typedef BOOL (WINAPI* PF_DestroyWindow)(IN HWND);
PF_DestroyWindow& g_pfDestroyWindow() { static PF_DestroyWindow func = ::DestroyWindow; return func; }
BOOL WINAPI My_DestroyWindow(IN HWND hWnd) {
   ::SetLastError(0);
   BOOL bRes = g_pfDestroyWindow()(hWnd);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DestroyWindow: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef UINT_PTR (WINAPI* PF_SetTimer)(IN HWND hWnd, IN UINT_PTR nIDEvent, IN UINT uElapse, IN TIMERPROC lpTimerFunc);
PF_SetTimer& g_pfSetTimer() { static PF_SetTimer func = ::SetTimer; return func; }
UINT_PTR WINAPI My_SetTimer(IN HWND hWnd, IN UINT_PTR nIDEvent, IN UINT uElapse, IN TIMERPROC lpTimerFunc) {
   ::SetLastError(0);
   UINT_PTR iRes = g_pfSetTimer()(hWnd, nIDEvent, uElapse, lpTimerFunc);
   DWORD dwErrCode = GetLastError();
   if (!iRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetTimer: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef int (WINAPI* PF_SetWindowRgn)(IN HWND, IN HRGN, IN BOOL);
PF_SetWindowRgn& g_pfSetWindowRgn() { static PF_SetWindowRgn func = ::SetWindowRgn; return func; }
int WINAPI My_SetWindowRgn(IN HWND hWnd, IN HRGN hRgn, IN BOOL bRedraw) {
   ::SetLastError(0);
   int iRes = g_pfSetWindowRgn()(hWnd, hRgn, bRedraw);
   DWORD dwErrCode = GetLastError();
   if (!iRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetWindowRgn: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef ATOM (WINAPI* PF_RegisterClassA)(IN CONST WNDCLASSA *);
PF_RegisterClassA& g_pfRegisterClassA() { static PF_RegisterClassA func = ::RegisterClassA; return func; }
ATOM WINAPI My_RegisterClassA(IN CONST WNDCLASSA *lpWndClass) {
   ::SetLastError(0);
   ATOM atom = g_pfRegisterClassA()(lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!atom) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegisterClassA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return atom;
}

typedef ATOM (WINAPI* PF_RegisterClassW)(IN CONST WNDCLASSW *);
PF_RegisterClassW& g_pfRegisterClassW() { static PF_RegisterClassW func = ::RegisterClassW; return func; }
ATOM WINAPI My_RegisterClassW(IN CONST WNDCLASSW *lpWndClass) {
   ::SetLastError(0);
   ATOM atom = g_pfRegisterClassW()(lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!atom) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegisterClassW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return atom;
}

typedef ATOM (WINAPI* PF_RegisterClassExA)(IN CONST WNDCLASSEXA *);
PF_RegisterClassExA& g_pfRegisterClassExA() { static PF_RegisterClassExA func = ::RegisterClassExA; return func; }
ATOM WINAPI My_RegisterClassExA(IN CONST WNDCLASSEXA *lpWndClass) {
   ::SetLastError(0);
   ATOM atom = g_pfRegisterClassExA()(lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!atom) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegisterClassExA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return atom;
}

typedef ATOM (WINAPI* PF_RegisterClassExW)(IN CONST WNDCLASSEXW *);
PF_RegisterClassExW& g_pfRegisterClassExW() { static PF_RegisterClassExW func = ::RegisterClassExW; return func; }
ATOM WINAPI My_RegisterClassExW(IN CONST WNDCLASSEXW *lpWndClass) {
   ::SetLastError(0);
   ATOM atom = g_pfRegisterClassExW()(lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!atom) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegisterClassExW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return atom;
}

typedef BOOL (WINAPI* PF_GetClassInfoA)(IN HINSTANCE, IN LPCSTR, OUT LPWNDCLASSA);
PF_GetClassInfoA& g_pfGetClassInfoA() { static PF_GetClassInfoA func = ::GetClassInfoA; return func; }
BOOL WINAPI My_GetClassInfoA(IN HINSTANCE hInstance, IN LPCSTR lpClassName, OUT LPWNDCLASSA lpWndClass) {
   ::SetLastError(0);
   BOOL bRes = g_pfGetClassInfoA()(hInstance, lpClassName, lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetClassInfoA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_GetClassInfoW)(IN HINSTANCE, IN LPCWSTR, OUT LPWNDCLASSW);
PF_GetClassInfoW& g_pfGetClassInfoW() { static PF_GetClassInfoW func = ::GetClassInfoW; return func; }
BOOL WINAPI My_GetClassInfoW(IN HINSTANCE hInstance, IN LPCWSTR lpClassName, OUT LPWNDCLASSW lpWndClass) {
   ::SetLastError(0);
   BOOL bRes = g_pfGetClassInfoW()(hInstance, lpClassName, lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetClassInfoW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_GetClassInfoExA)(IN HINSTANCE, IN LPCSTR, OUT LPWNDCLASSEXA);
PF_GetClassInfoExA& g_pfGetClassInfoExA() { static PF_GetClassInfoExA func = ::GetClassInfoExA; return func; }
BOOL WINAPI My_GetClassInfoExA(IN HINSTANCE hInstance, IN LPCSTR lpClassName, OUT LPWNDCLASSEXA lpWndClass) {
   ::SetLastError(0);
   BOOL bRes = g_pfGetClassInfoExA()(hInstance, lpClassName, lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetClassInfoExA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_GetClassInfoExW)(IN HINSTANCE, IN LPCWSTR, OUT LPWNDCLASSEXW);
PF_GetClassInfoExW& g_pfGetClassInfoExW() { static PF_GetClassInfoExW func = ::GetClassInfoExW; return func; }
BOOL WINAPI My_GetClassInfoExW(IN HINSTANCE hInstance, IN LPCWSTR lpClassName, OUT LPWNDCLASSEXW lpWndClass) {
   ::SetLastError(0);
   BOOL bRes = g_pfGetClassInfoExW()(hInstance, lpClassName, lpWndClass);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetClassInfoExW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_DrawMenuBar)(IN HWND hWnd);
PF_DrawMenuBar& g_pfDrawMenuBar() { static PF_DrawMenuBar func = ::DrawMenuBar; return func; }
BOOL WINAPI My_DrawMenuBar(IN HWND hWnd) {
   ::SetLastError(0);
   BOOL bRes = g_pfDrawMenuBar()(hWnd);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  DrawMenuBar: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef HDC (WINAPI* PF_GetDC)(IN HWND);
PF_GetDC& g_pfGetDC() { static PF_GetDC func = ::GetDC; return func; }
HDC WINAPI My_GetDC(IN HWND hWnd) {
   ::SetLastError(0);
   HDC hDC = g_pfGetDC()(hWnd);
   DWORD dwErrCode = GetLastError();
   if (!hDC) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetDC: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hDC;
}

typedef HDC (WINAPI* PF_GetWindowDC)(IN HWND);
PF_GetWindowDC& g_pfGetWindowDC() { static PF_GetWindowDC func = ::GetWindowDC; return func; }
HDC WINAPI My_GetWindowDC(IN HWND hWnd) {
   ::SetLastError(0);
   HDC hDC = g_pfGetWindowDC()(hWnd);
   DWORD dwErrCode = GetLastError();
   if (!hDC) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetWindowDC: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hDC;
}

typedef HDC (WINAPI* PF_GetDCEx)(IN HWND, IN HRGN, IN DWORD);
PF_GetDCEx& g_pfGetDCEx() { static PF_GetDCEx func = ::GetDCEx; return func; }
HDC WINAPI My_GetDCEx(IN HWND hWnd, IN HRGN hRgnClip, IN DWORD flags) {
   ::SetLastError(0);
   HDC hDC = g_pfGetDCEx()(hWnd, hRgnClip, flags);
   DWORD dwErrCode = GetLastError();
   if (!hDC) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  GetDCEx: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return hDC;
}

typedef BOOL (WINAPI* PF_RedrawWindow)(IN HWND, IN CONST RECT*, IN HRGN, IN UINT);
PF_RedrawWindow& g_pfRedrawWindow() { static PF_RedrawWindow func = ::RedrawWindow; return func; }
BOOL WINAPI My_RedrawWindow(IN HWND hWnd, IN CONST RECT *lprcUpdate, IN HRGN hRgnUpdate, IN UINT flags) {
   ::SetLastError(0);
   BOOL bRes = g_pfRedrawWindow()(hWnd, lprcUpdate, hRgnUpdate, flags);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RedrawWindow: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef int (WINAPI* PF_ReleaseDC)(IN HWND, IN HDC);
PF_ReleaseDC& g_pfReleaseDC() { static PF_ReleaseDC func = ::ReleaseDC; return func; }
int WINAPI My_ReleaseDC(IN HWND hWnd, IN HDC hDC) {
   ::SetLastError(0);
   int iRes = g_pfReleaseDC()(hWnd, hDC);
   DWORD dwErrCode = GetLastError();
   if (iRes != 1) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  ReleaseDC: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return iRes;
}

typedef LONG (WINAPI* PF_SetWindowLongA)(IN HWND, IN int, IN LONG);
PF_SetWindowLongA& g_pfSetWindowLongA() { static PF_SetWindowLongA func = ::SetWindowLongA; return func; }
LONG WINAPI My_SetWindowLongA(IN HWND hWnd, IN int nIndex, IN LONG dwNewLong) {
   ::SetLastError(0);
   LONG lRes = g_pfSetWindowLongA()(hWnd, nIndex, dwNewLong);
   DWORD dwErrCode = GetLastError();
   if ((lRes == 0) && (dwErrCode != NO_ERROR)) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetWindowLongA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return lRes;
}

typedef LONG (WINAPI* PF_SetWindowLongW)(IN HWND, IN int, IN LONG);
PF_SetWindowLongW& g_pfSetWindowLongW() { static PF_SetWindowLongW func = ::SetWindowLongW; return func; }
LONG WINAPI My_SetWindowLongW(IN HWND hWnd, IN int nIndex, IN LONG dwNewLong) {
   ::SetLastError(0);
   LONG lRes = g_pfSetWindowLongW()(hWnd, nIndex, dwNewLong);
   DWORD dwErrCode = GetLastError();
   if ((lRes == 0) && (dwErrCode != NO_ERROR)) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetWindowLongW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return lRes;
}

/** /
typedef LONG_PTR (WINAPI* PF_SetWindowLongPtrA)(IN HWND, IN int, IN LONG_PTR);
PF_SetWindowLongPtrA& g_pfSetWindowLongPtrA() { static PF_SetWindowLongPtrA func = ::SetWindowLongPtrA; return func; }
LONG_PTR WINAPI My_SetWindowLongPtrA(IN HWND hWnd, IN int nIndex, IN LONG_PTR dwNewLong) {
   ::SetLastError(0);
   LONG_PTR lRes = g_pfSetWindowLongPtrA()(hWnd, nIndex, dwNewLong);
   DWORD dwErrCode = GetLastError();
   if (lRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetWindowLongPtrA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return lRes;
}

typedef LONG_PTR (WINAPI* PF_SetWindowLongPtrW)(IN HWND, IN int, IN LONG_PTR);
PF_SetWindowLongPtrW& g_pfSetWindowLongPtrW() { static PF_SetWindowLongPtrW func = ::SetWindowLongPtrW; return func; }
LONG_PTR WINAPI My_SetWindowLongPtrW(IN HWND hWnd, IN int nIndex, IN LONG_PTR dwNewLong) {
   ::SetLastError(0);
   LONG_PTR lRes = g_pfSetWindowLongPtrW()(hWnd, nIndex, dwNewLong);
   DWORD dwErrCode = GetLastError();
   if (lRes == 0) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetWindowLongPtrW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return lRes;
}
/**/

typedef BOOL (WINAPI* PF_SetMenuItemInfoW)(IN HMENU, IN UINT, IN BOOL, IN LPCMENUITEMINFOW);
PF_SetMenuItemInfoW& g_pfSetMenuItemInfoW() { static PF_SetMenuItemInfoW func = ::SetMenuItemInfoW; return func; }
BOOL WINAPI My_SetMenuItemInfoW(IN HMENU hMenu, IN UINT uItem, IN BOOL fByPosition, IN LPCMENUITEMINFOW lpmii) {
   ::SetLastError(0);
   BOOL bRes = g_pfSetMenuItemInfoW()(hMenu, uItem, fByPosition, lpmii);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetMenuItemInfoW: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_SetMenuItemInfoA)(IN HMENU, IN UINT, IN BOOL, IN LPCMENUITEMINFOA);
PF_SetMenuItemInfoA& g_pfSetMenuItemInfoA() { static PF_SetMenuItemInfoA func = ::SetMenuItemInfoA; return func; }
BOOL WINAPI My_SetMenuItemInfoA(IN HMENU hMenu, IN UINT uItem, IN BOOL fByPosition, IN LPCMENUITEMINFOA lpmii) {
   ::SetLastError(0);
   BOOL bRes = g_pfSetMenuItemInfoA()(hMenu, uItem, fByPosition, lpmii);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  SetMenuItemInfoA: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef LONG (WINAPI* PF_RegOpenKeyExA)(IN HKEY, IN LPCSTR, DWORD, IN REGSAM, IN PHKEY);
PF_RegOpenKeyExA& g_pfRegOpenKeyExA() { static PF_RegOpenKeyExA func = ::RegOpenKeyExA; return func; }
LONG WINAPI My_RegOpenKeyExA(IN HKEY hKey, IN LPCSTR lpSubKey, DWORD ulOptions, IN REGSAM samDesired, IN PHKEY phkResult) {
   ::SetLastError(0);
   LONG lRes = g_pfRegOpenKeyExA()(hKey, lpSubKey, ulOptions, samDesired, phkResult);
   if (lRes != NO_ERROR) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegOpenKeyExA: "), lRes);
   }
   ::SetLastError(lRes);
   return lRes;
}

typedef LONG (WINAPI* PF_RegOpenKeyExW)(IN HKEY, IN LPCWSTR, DWORD, IN REGSAM, IN PHKEY);
PF_RegOpenKeyExW& g_pfRegOpenKeyExW() { static PF_RegOpenKeyExW func = ::RegOpenKeyExW; return func; }
LONG WINAPI My_RegOpenKeyExW(IN HKEY hKey, IN LPCWSTR lpSubKey, DWORD ulOptions, IN REGSAM samDesired, IN PHKEY phkResult) {
   ::SetLastError(0);
   LONG lRes = g_pfRegOpenKeyExW()(hKey, lpSubKey, ulOptions, samDesired, phkResult);
   if (lRes != NO_ERROR) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  RegOpenKeyExW: "), lRes);
   }
   ::SetLastError(lRes);
   return lRes;
}

typedef BOOL (WINAPI* PF_Beep)(IN DWORD, IN DWORD);
PF_Beep& g_pfBeep() { static PF_Beep func = ::Beep; return func; }
BOOL WINAPI My_Beep(IN DWORD dwFreq, IN DWORD dwDuration) {
   ::SetLastError(0);
   BOOL bRes = g_pfBeep()(dwFreq, dwDuration);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  Beep: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

typedef BOOL (WINAPI* PF_MessageBeep)(IN UINT);
PF_MessageBeep& g_pfMessageBeep() { static PF_MessageBeep func = ::MessageBeep; return func; }
BOOL WINAPI My_MessageBeep(IN UINT uType) {
   ::SetLastError(0);
   BOOL bRes = g_pfMessageBeep()(uType);
   DWORD dwErrCode = GetLastError();
   if (!bRes) {
      LOG_PUT(g_pLog, PutErrorCode, LL_ERROR) _T("  MessageBeep: "), dwErrCode);
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

//SetWindowStyle
//GetWindowStyle
//GetPixel
//SetPixel
//BeginPaint
//EndPaint
//GetSysColor
//GetSysColorBrush
//GetProcAddress
//SendMessageA
//SendMessageW
//PostMessageA
//PostMessageW

/*******************************************************************************************/
//
//          - 
//
/*******************************************************************************************/
BOOL ReplaceIATEntryInOneMod(PCSTR szCalleeModName, PVOID pfnCurrent, PVOID pfnNew, HMODULE hmodCaller)
{
   // Get the address of the module's import section
   ULONG ulSize;
   PIMAGE_IMPORT_DESCRIPTOR pImportDesc = (PIMAGE_IMPORT_DESCRIPTOR)
      ImageDirectoryEntryToData(hmodCaller, TRUE,
      IMAGE_DIRECTORY_ENTRY_IMPORT, &ulSize);

   if (pImportDesc == NULL)
      return FALSE;  // This module has no import section

   // Find the import descriptor containing references to callee's functions
   for (; pImportDesc->Name; pImportDesc++) {
      PSTR pszModName = (PSTR) ((PBYTE) hmodCaller + pImportDesc->Name);
      if (lstrcmpiA(pszModName, szCalleeModName) == 0)
         break;   // Found
   }

   if (pImportDesc->Name == 0)
      return FALSE;  // This module doesn't import any functions from this callee

   // Get caller's import address table (IAT) for the callee's functions
   PIMAGE_THUNK_DATA pThunk = (PIMAGE_THUNK_DATA)
      ((PBYTE)/*(int)*/hmodCaller + (int)pImportDesc->FirstThunk);

   // Replace current function address with new function address
   for (; pThunk->u1.Function; pThunk++) {

      // Get the address of the function address
      PROC* ppfn = (PROC*) &pThunk->u1.Function;

      // Is this the function we're looking for?
      BOOL fFound = (*ppfn == pfnCurrent);
      /*
      if (!fFound && (*ppfn > sm_pvMaxAppAddr)) {

         // If this is not the function and the address is in a shared DLL,
         // then maybe we're running under a debugger on Windows 98. In this
         // case, this address points to an instruction that may have the
         // correct address.

         PBYTE pbInFunc = (PBYTE) *ppfn;
         if (pbInFunc[0] == cPushOpCode) {
            // We see the PUSH instruction, the real function address follows
            ppfn = (PROC*) &pbInFunc[1];

            // Is this the function we're looking for?
            fFound = (*ppfn == pfnCurrent);
         }
      }
      */
      if (fFound) {
         DWORD dwOldProtect = 0;
         // The addresses match, change the import section address
         BOOL bRes = VirtualProtect(ppfn, sizeof(PVOID), PAGE_READWRITE, &dwOldProtect);
         if (!bRes)
            return FALSE;
         DWORD dwWritten = 0;
         bRes = WriteProcessMemory(GetCurrentProcess(), ppfn, &pfnNew, sizeof(PVOID), &dwWritten); //     -   dwOldProtect
         bRes &= VirtualProtect(ppfn, sizeof(PVOID), dwOldProtect, &dwOldProtect); // restore protect
         if ((!bRes) || (dwWritten != sizeof(PVOID)))
            return FALSE;
         return TRUE;  // We did it, get out
      }
   }

   // If we get to here, the function is not in the caller's import section
   return FALSE;
}

/*******************************************************************************************/
//
//          -  
//
/*******************************************************************************************/

void DebugControlling(bool bIntercept) {

#ifdef _DETOURS_H_
   //  -   Detours API
   #ifdef DETOUR_TRAMPOLINE_SIZE // ver.1.5
      #define HACK(dll, nameFunc) \
         if (bIntercept) { \
            g_pf##nameFunc() = (PF_##nameFunc)DetourFunction((PBYTE)(PF_##nameFunc)::nameFunc, (PBYTE)My_##nameFunc); \
         } else { \
            /*g_pf##nameFunc() = (PF_##nameFunc)DetourFunction((PBYTE)My_##nameFunc, (PBYTE)(PF_##nameFunc)::nameFunc); /**/ \
         }
   #else // DETOUR_TRAMPOLINE_SIZE // ver.2.1
      ::DetourTransactionBegin();
      ::DetourUpdateThread(::GetCurrentThread());

      #define HACK(dll, nameFunc) (bIntercept ? ::DetourAttach : DetourDetach)(&(PVOID&)g_pf##nameFunc(), (PBYTE)My_##nameFunc);
      #define NEED_DETOUR_TRANSACTION_COMMIT

   #endif // DETOUR_TRAMPOLINE_SIZE
#else _DETOURS_H_
   //  - 
   #define HACK(dll, nameFunc) \
      if (bIntercept) { \
         ReplaceIATEntryInOneMod(dll, g_pf##nameFunc(), My_##nameFunc, ::GetModuleHandle(NULL)); \
      } else { \
         ReplaceIATEntryInOneMod(dll, My_##nameFunc, g_pf##nameFunc(), ::GetModuleHandle(NULL)); \
      }

#endif _DETOURS_H_

   // HDC
   HACK("User32.dll"    , InvalidateRect);
   HACK("GDI32.dll"     , SelectObject);
   HACK("GDI32.dll"     , MoveToEx);
   HACK("GDI32.dll"     , LineTo);
   HACK("GDI32.dll"     , BitBlt);
   HACK("GDI32.dll"     , PatBlt);
   HACK("GDI32.dll"     , StretchBlt);
   HACK("GDI32.dll"     , SetTextColor);
   HACK("GDI32.dll"     , SetBkColor);
   HACK("User32.dll"    , DrawTextA);
   HACK("User32.dll"    , DrawTextW);
   HACK("GDI32.dll"     , CreateCompatibleDC);
   HACK("GDI32.dll"     , DeleteDC);
   HACK("GDI32.dll"     , DeleteObject);
   HACK("GDI32.dll"     , Polygon);
   HACK("GDI32.dll"     , GetObjectA);
   HACK("GDI32.dll"     , GetObjectW);
   HACK("User32.dll"    , DrawIconEx);
   HACK("User32.dll"    , GetIconInfo);
   HACK("User32.dll"    , DestroyIcon);
   HACK("GDI32.dll"     , PlayEnhMetaFile);
   HACK("GDI32.dll"     , GetEnhMetaFileHeader);
   HACK("GDI32.dll"     , DeleteEnhMetaFile);
   HACK("GDI32.dll"     , SetStretchBltMode);
   HACK("GDI32.dll"     , CreateSolidBrush);
   HACK("MSImg32.dll"   , GradientFill);
   HACK("User32.dll"    , GetDC);
   HACK("User32.dll"    , GetWindowDC);
   HACK("User32.dll"    , ReleaseDC);
   HACK("User32.dll"    , GetDCEx);
   HACK("User32.dll"    , RedrawWindow);

   // bitmap
   HACK("GDI32.dll"     , CreateCompatibleBitmap);
   HACK("GDI32.dll"     , CreateBitmap);
   HACK("User32.dll"    , LoadBitmapA);
   HACK("User32.dll"    , LoadBitmapW);
   HACK("User32.dll"    , LoadImageA);
   HACK("User32.dll"    , LoadImageW);
 //HACK("User32.dll"    , GetObjectType);

   // sync
   HACK("kernel32.dll"  , SetEvent);
   HACK("kernel32.dll"  , ResetEvent);
   HACK("kernel32.dll"  , WaitForSingleObject);
   HACK("kernel32.dll"  , WaitForMultipleObjects);

   // file
 //HACK("kernel32.dll"  , DeleteFileA);
   HACK("kernel32.dll"  , DeleteFileW);
 //HACK("kernel32.dll"  , CreateFileA);
   HACK("kernel32.dll"  , CreateFileW);
   HACK("kernel32.dll"  , ReadFile);
   HACK("kernel32.dll"  , WriteFile);
   HACK("kernel32.dll"  , CloseHandle);

   // window
   HACK("User32.dll"    , CreateDialogParamA);
   HACK("User32.dll"    , CreateDialogParamW);
   HACK("User32.dll"    , DialogBoxParamA);
   HACK("User32.dll"    , DialogBoxParamW);
   HACK("User32.dll"    , CreateWindowExA);
   HACK("User32.dll"    , CreateWindowExW);
   HACK("User32.dll"    , DestroyWindow);
   HACK("User32.dll"    , SetTimer);
   HACK("User32.dll"    , SetWindowRgn);
   HACK("User32.dll"    , RegisterClassA);
   HACK("User32.dll"    , RegisterClassW);
   HACK("User32.dll"    , RegisterClassExA);
   HACK("User32.dll"    , RegisterClassExW);
   HACK("User32.dll"    , GetClassInfoA);
   HACK("User32.dll"    , GetClassInfoW);
   HACK("User32.dll"    , GetClassInfoExA);
   HACK("User32.dll"    , GetClassInfoExW);

   // menu
   HACK("User32.dll"    , SetMenuItemInfoW);
   HACK("User32.dll"    , SetMenuItemInfoA);

   // registry
   HACK("AdvAPI32.dll"  , RegOpenKeyExA);
   HACK("AdvAPI32.dll"  , RegOpenKeyExW);

   // others
   HACK("User32.dll"    , SetWindowLongA);
   HACK("User32.dll"    , SetWindowLongW);
   HACK("kernel32.dll"  , Beep);
   HACK("User32.dll"    , MessageBeep);
 //HACK("User32.dll"    , SetWindowLongPtrA);
 //HACK("User32.dll"    , SetWindowLongPtrW);

#ifdef NEED_DETOUR_TRANSACTION_COMMIT
   ::DetourTransactionCommit();
   #undef NEED_DETOUR_TRANSACTION_COMMIT
#endif NEED_DETOUR_TRANSACTION_COMMIT

#undef HACK
}
