////////////////////////////////////////////////////////////////////////////////
// File name: CommonLib.hpp
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// WWW page: http://kserg77.narod.ru/CommonLib.html
// Date: 2007 01 24
//
// Description:   . Inline .
////////////////////////////////////////////////////////////////////////////////

inline BOOL InitializeCriticalSectionEx(CRITICAL_SECTION &cs, DWORD dwSpinCount) {
   BOOL bRes = TRUE;
#if _WIN32_WINNT < 0x0500
   ::InitializeCriticalSection(&cs); //     ...
#else
   DWORD dwErrCode = ERROR_SUCCESS;
   bRes = ::InitializeCriticalSectionAndSpinCount(&cs, dwSpinCount);
   if (!bRes) {
      dwErrCode = ::GetLastError();
      memset(&cs, 0, sizeof(CRITICAL_SECTION));
   }
   ::SetLastError(dwErrCode);
#endif
   return bRes;
}

inline long  toLong (float f) { long  l=0  ; VarI4FromR4 (f, &l); return l; }
inline int   toInt  (float f) { long  i=0  ; VarIntFromR4(f, &i); return i; }
inline float toFloat(long  l) { float f=0.f; VarR4FromI4 (l, &f); return f; }
inline float toFloat(int   i) { float f=0.f; VarR4FromInt(i, &f); return f; }

inline RECTEX FindInnerRect(const SIZE &sizeInner, const SIZE &sizeOutward) {
   //   (sizeOutward)   
   //  (sizeInner)      
   //   , ..       .
   //       .
   RECTEX Rect;
#ifndef __EXCLUDE_CRT__
   POINTFLOAT percent = {float(sizeInner.cx)/sizeOutward.cx,
                         float(sizeInner.cy)/sizeOutward.cy};
   if (percent.x > percent.y) {
      Rect.right  = sizeOutward.cx;
      Rect.bottom = long(float(sizeInner.cy)/percent.x);

      Rect.top    = long(float(sizeOutward.cy)/2-float(Rect.bottom)/2);
      Rect.bottom = long(float(sizeOutward.cy)/2+float(Rect.bottom)/2);
   } else {
      Rect.right  = long(float(sizeInner.cx)/percent.y);
      Rect.bottom = sizeOutward.cy;

      Rect.left   = long(float(sizeOutward.cx)/2-float(Rect.right)/2);
      Rect.right  = long(float(sizeOutward.cx)/2+float(Rect.right)/2);
   }
#else __EXCLUDE_CRT__
   POINTFLOAT percent = {toFloat(sizeInner.cx)/sizeOutward.cx,
                         toFloat(sizeInner.cy)/sizeOutward.cy};
   if (percent.x > percent.y) {
      Rect.right  = sizeOutward.cx;
      Rect.bottom = toLong(toFloat(sizeInner.cy)/percent.x);

      Rect.top    = toLong(toFloat(sizeOutward.cy)/2-toFloat(Rect.bottom)/2);
      Rect.bottom = toLong(toFloat(sizeOutward.cy)/2+toFloat(Rect.bottom)/2);
   } else {
      Rect.right  = toLong(toFloat(sizeInner.cx)/percent.y);
      Rect.bottom = sizeOutward.cy;

      Rect.left   = toLong(toFloat(sizeOutward.cx)/2-toFloat(Rect.right)/2);
      Rect.right  = toLong(toFloat(sizeOutward.cx)/2+toFloat(Rect.right)/2);
   }
#endif __EXCLUDE_CRT__
   return Rect;
}

inline RECTEX FindInnerRect(const RECT &rectInner, const RECT &rectOutward) {
   return RECTEX(
      FindInnerRect(
         RECTEX(rectInner  ).size(),
         RECTEX(rectOutward).size()
      )
   ).move(rectOutward.left, rectOutward.top);
}

inline BOOL PointInRect(const POINT &point, const RECT &rect) {
   return ::PtInRect(&rect, point);
}

inline BOOL IntersectRect(const RECT &rect1, const RECT &rect2) {
   return ::IntersectRect(&RECTEX(), &rect1, &rect2);
}

inline bool PointInPolygon(const POINT& point, const POINT* const polygon, int size) {
   POINTFLOAT a = {point.x+0.01f, point.y+0.01f};
   int count = 0;
   for (int i=0; i<size; i++) {
      int j = (i+1)%size;
      if (polygon[i].y == polygon[j].y) continue;
      if (polygon[i].y > a.y && polygon[j].y > a.y) continue;
      if (polygon[i].y < a.y && polygon[j].y < a.y) continue;
      if (max(polygon[i].y, polygon[j].y) == a.y) count++;
      else
         if (min(polygon[i].y, polygon[j].y) == a.y) continue;
         else {
            float t = (float)(a.y-polygon[i].y)/(polygon[j].y-polygon[i].y);
            if (t>0 && t<1 && polygon[i].x+t*(polygon[j].x-polygon[i].x) >= a.x) count++;
         }
   }
   return count & 1;
}

inline SIZEEX GetScreenSize() {
   return SIZEEX(
      ::GetSystemMetrics(SM_CXSCREEN),
      ::GetSystemMetrics(SM_CYSCREEN)
   );
}

inline RECTEX ScreenToClient(HWND hWnd, const RECT &rect) {
   return RECTEX(
      ScreenToClient(hWnd, RECTEX(rect).pointLT()),
      ScreenToClient(hWnd, RECTEX(rect).pointRB())
   );
}

inline POINTEX ScreenToClient(HWND hWnd, const POINT &point) {
   POINTEX pointResult = point;
   return ::ScreenToClient(hWnd, &pointResult) ? pointResult : POINTEX();
}

inline RECTEX ClientToScreen(HWND hWnd, const RECT &rect) {
   return RECTEX(
      ClientToScreen(hWnd, RECTEX(rect).pointLT()),
      ClientToScreen(hWnd, RECTEX(rect).pointRB())
   );
}

inline POINTEX ClientToScreen(HWND hWnd, const POINT &point) {
   POINTEX pointResult = point;
   return ::ClientToScreen(hWnd, &pointResult) ? pointResult : POINTEX();
}

inline BOOL MoveWindow(HWND hWnd, const RECT &rect, BOOL bRepaint) {
   return ::MoveWindow(
      hWnd,
      rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
      bRepaint);
}

inline BOOL SetWindowPos(HWND hWnd, HWND hWndInsertAfter, const RECT &rect, UINT uFlags) {
   return ::SetWindowPos(
      hWnd,
      hWndInsertAfter,
      rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
      uFlags);
}

inline RECTEX GetWindowRect(HWND hWnd) {
   RECTEX rect;
   return ::GetWindowRect(hWnd, &rect) ? rect : RECTEX();
}

inline SIZEEX GetWindowSize(HWND hWnd) {
   return (SIZEEX)GetWindowRect(hWnd);
}

inline RECTEX GetClientRect(HWND hWnd) {
   RECTEX rect;
   return ::GetClientRect(hWnd, &rect) ? rect : RECTEX();
}

inline SIZEEX GetClientSize(HWND hWnd) {
   return (SIZEEX)GetClientRect(hWnd);
}

inline CString GetClassName(HWND hWnd) {
   TCHAR szClassName[256] = {0};
   int iRes = ::GetClassName(hWnd, szClassName, 256);
   return iRes ? szClassName : NULL;
}

inline WNDPROC GetWindowProc(HWND hWnd) {
   return (WNDPROC)::GetWindowLongPtr(hWnd, GWLP_WNDPROC);
}

inline DWORD GetWindowStyleEx(HWND hWnd) {
   return ::GetWindowLongPtr(hWnd, GWL_EXSTYLE);
}
#ifdef GetWindowExStyle
   #undef GetWindowExStyle
   #define GetWindowExStyle GetWindowStyleEx
#endif

inline LONG_PTR GetWindowUserData(HWND hWnd) {
   return ::GetWindowLongPtr(hWnd, GWLP_USERDATA);
}

inline WNDPROC SetWindowProc(HWND hWnd, WNDPROC pWndProc) {
   return (WNDPROC)::SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)pWndProc);
}

#ifndef GetWindowStyle
inline DWORD GetWindowStyle(HWND hWnd) {
   return (DWORD)::GetWindowLongPtr(hWnd, GWL_STYLE);
}
#endif // GetWindowStyle

inline DWORD SetWindowStyle(HWND hWnd, DWORD lStyle) {
   return (DWORD)::SetWindowLongPtr(hWnd, GWL_STYLE, (LONG_PTR)lStyle);
}

inline DWORD SetWindowStyleEx(HWND hWnd, DWORD lStyleEx) {
   return (DWORD)::SetWindowLongPtr(hWnd, GWL_EXSTYLE, (LONG_PTR)lStyleEx);
}

inline LONG_PTR SetWindowUserData(HWND hWnd, LONG_PTR lUserData) {
   return ::SetWindowLongPtr(hWnd, GWLP_USERDATA, lUserData);
}

inline POINTEX GetCursorPos() {
   POINTEX pointCur;
   return ::GetCursorPos(&pointCur) ? pointCur : POINTEX();
}

inline void ShowCursor() {
   while (::ShowCursor(TRUE)<0) {}
}

inline void HideCursor() {
   while (::ShowCursor(FALSE)>=0) {}
}

inline BOOL PathExist(LPCTSTR szPath)
{
   //BOOL PathIsDirectory( LPCTSTR pszPath );
   //Header shlwapi.h
   //Import library shlwapi.lib

   DWORD dwAttr = GetFileAttributes(szPath);
   return ((dwAttr != 0xFFFFFFFF) && (dwAttr & FILE_ATTRIBUTE_DIRECTORY));
}

inline BOOL FileExist(LPCTSTR szPath)
{
   DWORD dwAttr = GetFileAttributes(szPath);
   return ((dwAttr != 0xFFFFFFFF) && !(dwAttr & FILE_ATTRIBUTE_DIRECTORY));
}

// retrieves the full path and filename for the executable file containing the specified module
inline CString GetModuleFileName(HMODULE hModule) {
   TCHAR szPath[MAX_PATH] = {0};
   ::GetModuleFileName(hModule, szPath, MAX_PATH);
   return szPath;
}

//        (    )
inline CString GetModuleDir(HMODULE hModule) {
   return GetFileDir(GetModuleFileName(hModule));
}

inline BOOL SetMenuString(HMENU hMenu, UINT uItem, BOOL bByPosition, LPCTSTR szNewText) {
   MENUITEMINFO miiGet = {sizeof(MENUITEMINFO), MIIM_TYPE};
   ::GetMenuItemInfo(hMenu, uItem, bByPosition, &miiGet);

   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE, MFT_STRING | ((miiGet.fType & MFT_RADIOCHECK) ? MFT_RADIOCHECK : 0)};
   mii.dwTypeData = (LPTSTR)szNewText;
   return ::SetMenuItemInfo(hMenu, uItem, bByPosition, &mii);
}

inline CString GetMenuString(HMENU hMenu, UINT uItem, BOOL bByPosition) {
   TCHAR szName[MAX_PATH] = {0};
   int iRes = ::GetMenuString(hMenu, uItem, szName, MAX_PATH, bByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
   return iRes ? szName : NULL;
}

inline CString GetComputerName() {
   DWORD dwSize = MAX_COMPUTERNAME_LENGTH + 1;
   TCHAR szCompName[MAX_COMPUTERNAME_LENGTH + 1] = {0}; szCompName[MAX_COMPUTERNAME_LENGTH] = 0;
   BOOL bRes = ::GetComputerName(szCompName, &dwSize);
   return bRes ? szCompName : NULL;
}

inline CString GetUserName() {
   DWORD dwSize = 256; // == UNLEN is defined in <LMCONS.H>
   TCHAR szUName[256+1] = {0}; szUName[256] = 0;
   BOOL bRes = ::GetUserName(szUName, &dwSize);
   return bRes ? szUName : NULL;
}

inline CString GetCurrentDirectory() {
   DWORD dwNeed = ::GetCurrentDirectory(0, NULL);
   CString strCurrDir;
   DWORD dwErrCode = ERROR_SUCCESS;
   TCHAR *szDir = new TCHAR[dwNeed];
   if (szDir == NULL) {
      dwErrCode = ERROR_OUTOFMEMORY;
   } else {
      szDir[0] = 0;
      dwNeed = ::GetCurrentDirectory(dwNeed, szDir);
      if (dwNeed == 0) {
         dwErrCode = ::GetLastError();
      } else {
         strCurrDir = szDir;
      }
      delete [] szDir; szDir = NULL;
   }
   ::SetLastError(dwErrCode);
   return strCurrDir;
}

inline CString GetSystemDirectory() {
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT uiRes = ::GetSystemDirectory(szDir, MAX_PATH);
   return uiRes ? szDir : NULL;
}

inline CString GetWindowsDirectory() {
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT uiRes = ::GetWindowsDirectory(szDir, MAX_PATH);
   return uiRes ? szDir : NULL;
}

inline CString GetTempPath() {
   //   ,  .  GetTempPath   ?
   // char sz_temp[MAX_PATH] = {0};
   // SHGetFolderPath( 0, CSIDL_INTERNET_CACHE, 0, 0, sz_temp );
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT dwRes = ::GetTempPath(MAX_PATH, szDir);
   return dwRes ? szDir : NULL;
}

inline CString MIMEType2FileType(LPCTSTR szMIMEType) {
   CString strRes = CRegistry::GetString(_T("HKEY_CLASSES_ROOT\\MIME\\Database\\Content Type\\") + CString(szMIMEType) + _T("\\Extension"));
   if (!strRes.IsEmpty()) {
      strRes = strRes.Mid(1);
   }
   return strRes;
}

inline CString FileType2MIMEType(LPCTSTR szFileType) {
   return CRegistry::GetString(_T("HKEY_CLASSES_ROOT\\.") + CString(szFileType) + _T("\\Content Type"));
}

inline SIZEEX SizeIcon(HICON hIcon) {
   SIZEEX resSize;
   ICONINFO iconInfo;
   if (::GetIconInfo(hIcon, &iconInfo)) {
      resSize.cx = iconInfo.xHotspot<<1;
      resSize.cy = iconInfo.yHotspot<<1;
   }
   return resSize;
}

inline SIZEEX SizeBitmap(HBITMAP hBmp) {
   SIZE size = {0,0};
   if (hBmp) {
      BITMAP bmp = {0,0,0,0,0,0,NULL};
      int iRes = ::GetObject(hBmp, sizeof(BITMAP), &bmp);
      if (iRes == sizeof(BITMAP)) {
         size.cx = bmp.bmWidth;
         size.cy = bmp.bmHeight;
      }
   }
   return size;
}

inline SIZEEX SizeBitmap(HDC hCDC) {
   HBITMAP hBmpNil = ::CreateCompatibleBitmap(hCDC, 0, 0); // create 1x1-pixel monochrome bitmap
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmpNil);
   SIZE size = {0,0};
   if ((hBmpOld != NULL) && (hBmpOld != HGDI_ERROR)) {
      size = SizeBitmap(hBmpOld);
      ::SelectObject(hCDC, hBmpOld);
   }
   ::DeleteObject(hBmpNil);
   return size;
}

inline HBITMAP CreateBitmap(int iWidth, int iHeight) {
   HDC hDC = ::GetDC(NULL);
   HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, iWidth, iHeight);
   ::ReleaseDC(NULL, hDC);
   return hBmp;
}

inline HBITMAP CreateBitmap(const SIZE &size) {
   return ::CreateBitmap(size.cx, size.cy);
}

inline HBITMAP CreateMono(const SIZE &sizeBmp) {
   return ::CreateBitmap(sizeBmp.cx, sizeBmp.cy, 1, 1, NULL);
}

inline HBITMAP CreateCompatibleBitmap(HBITMAP hBitmap, int iWidth, int iHeight) {
	BITMAP bm;
	int iRes = ::GetObject(hBitmap, sizeof(bm), &bm);
   if (iRes == sizeof(BITMAP))
      return ::CreateBitmap(iWidth, iHeight, bm.bmPlanes, bm.bmBitsPixel, bm.bmBits);
   return NULL;
}

//     0  maxDiapason,   
inline int rand(int maxDiapason) {
   if (maxDiapason < 1) return 0;
   static bool bInitRandom = (srand(::GetTickCount()), true);
   return rand()%(maxDiapason+1);
 //return int(float(maxDiapason+1)*rand()/(RAND_MAX+1));
}

inline CString Format(LPCTSTR szFormat, ...) {
   va_list arglist;
   va_start(arglist, szFormat);
   CString strResult;
   strResult.FormatV(szFormat, arglist);
   va_end(arglist);
   return strResult;
}

#ifndef _UNICODE
inline CString AnsiToOem(LPCTSTR szStr) {
   CString str(szStr);
   str.AnsiToOem();
   return str;
}

inline CString OemToAnsi(LPCTSTR szStr) {
   CString str(szStr);
   str.OemToAnsi();
   return str;
}
#endif // _UNICODE

inline CString Replace(IN const CString &str, IN LPCTSTR szOld, IN LPCTSTR szNew) {
   CString strRes = str;
   strRes.Replace(szOld, szNew);
   return strRes;
}

inline CString Replace(IN const CString &str, IN TCHAR cOld, IN TCHAR cNew) {
   CString strRes = str;
   strRes.Replace(cOld, cNew);
   return strRes;
}

inline SYSTEMTIME GetSystemTime() {
   SYSTEMTIME SysTime = {0,0,0,0,0,0,0,0};
   ::GetSystemTime(&SysTime);
   return SysTime;
}

inline SYSTEMTIME GetLocalTime() {
   SYSTEMTIME localTime = {0,0,0,0,0,0,0,0};
   ::GetLocalTime(&localTime);
   return localTime;
}

inline FILETIME GetSystemTimeAsFileTime() {
   FILETIME FileTime = {0,0};
   ::GetSystemTimeAsFileTime(&FileTime);
   return FileTime;
}

inline SYSTEMTIME FileTimeToSystemTime(const FILETIME &FileTime) {
   SYSTEMTIME SysTime = {0,0,0,0,0,0,0,0};
   BOOL bRes = ::FileTimeToSystemTime(&FileTime, &SysTime);
   return SysTime;
}

inline FILETIME SystemTimeToFileTime(const SYSTEMTIME &SysTime) {
   FILETIME FileTime = {0,0};
   BOOL bRes = ::SystemTimeToFileTime(&SysTime, &FileTime);
   return FileTime;
}

inline FILETIME FileTimeToLocalFileTime(const FILETIME &FileTime) {
   FILETIME ftRes = {0,0};
   BOOL bRes = ::FileTimeToLocalFileTime(&FileTime, &ftRes);
   return ftRes;
}

inline BOOL BitBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const POINT &pointTopLeft, DWORD dwROp) {
   return ::BitBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, pointTopLeft.x, pointTopLeft.y, dwROp);
}

inline BOOL BitBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, 0, 0, dwROp);
}

inline BOOL BitBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, const POINT &pointTopLeft, DWORD dwROp) {
   return ::BitBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, pointTopLeft.x, pointTopLeft.y, dwROp);
}

inline BOOL BitBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, 0, 0, dwROp);
}

inline BOOL StretchBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const RECT &rectSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, rectSrc.left, rectSrc.top, rectSrc.right-rectSrc.left, rectSrc.bottom-rectSrc.top, dwROp);
}

inline BOOL StretchBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, 0, 0, sizeSrc.cx, sizeSrc.cy, dwROp);
}

inline BOOL StretchBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, 0, 0, sizeSrc.cx, sizeSrc.cy, dwROp);
}

inline BOOL StretchBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, const RECT &rectSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, rectSrc.left, rectSrc.top, rectSrc.right-rectSrc.left, rectSrc.bottom-rectSrc.top, dwROp);
}

inline HINSTANCE __stdcall GetInstanceFromAddress(PVOID pEip) {
   _ASSERTE(pEip != NULL);

   MEMORY_BASIC_INFORMATION mem;
   if (VirtualQuery(pEip, &mem, sizeof(mem)))
   {
      _ASSERTE((mem.Type == MEM_IMAGE) || (mem.Type == MEM_PRIVATE));
      _ASSERTE(mem.AllocationBase != NULL);
      return (HINSTANCE)mem.AllocationBase;
   }
   return NULL;
}

inline __declspec(naked) HINSTANCE __stdcall GetCurrentInstance() {
   /** / // VC 7.0  intrinsic-   _ReturnAddress,       .
         //        .
         //     ,   VC 6.0    (  ,    SP),
         //             VC 6.0 SP5
   MEMORY_BASIC_INFORMATION mem;
   if (VirtualQuery(_ReturnAddress(), &mem, sizeof(mem))) {
      _ASSERTE((mem.Type == MEM_IMAGE) || (mem.Type == MEM_PRIVATE));
      _ASSERTE(mem.AllocationBase != NULL);
      return (HINSTANCE)mem.AllocationBase;
   }
   return NULL;
   /**/
   __asm {
#ifdef _M_IX86
      mov   eax, [esp]
      push  eax
      jmp   GetInstanceFromAddress
#else
      #error This machine type is not supported.
#endif
   }
}

inline CString GUID2String(IN REFGUID guid) {
   CString strRes;
   OLECHAR szRes[39] = {0};
   if (::StringFromGUID2(guid, szRes, sizeof(szRes)>>1)) {
      strRes = szRes;
   } else {
      strRes.Format(_T("{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}"),
         int(guid.Data1), int(guid.Data2), int(guid.Data3), int(guid.Data4[0]), int(guid.Data4[1]), int(guid.Data4[2]), int(guid.Data4[3]), int(guid.Data4[4]), int(guid.Data4[5]), int(guid.Data4[6]), int(guid.Data4[7]));
   }
   return strRes;
}

inline GUID CoCreateGuid() {
   GUID guid; memset(&guid, 0, sizeof(GUID));
   ::CoCreateGuid(&guid);
   return guid;
}

inline GUID String2GUID(IN LPCTSTR szGUID) {
   GUID guid = {0xFFFFFFFF, 0xFFFF, 0xFFFF, {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}};
   String2GUID(szGUID, guid);
   return guid;
}


inline CString Locale(WORD wIDLanguage, LCTYPE LCType) {
   TCHAR szRes[64] = {0};
   int iRes = ::GetLocaleInfo(MAKELCID(wIDLanguage, SORT_DEFAULT), LCType, szRes, (sizeof(szRes) << (sizeof(TCHAR)-1)));
   return iRes ? szRes : NULL;
}

inline void alert(LPCTSTR szText) {
   ::MessageBox(::GetActiveWindow(), szText, _T("Info"), MB_OK | MB_ICONINFORMATION);
}
inline void alert(int iVal) {
   alert(::Format(_T("%i"), iVal));
}
inline void alert(double dVal) {
   alert(::Format(_T("%f"), dVal));
}
