////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Mosaic.cpp"
//
//  
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
//
// 2007.02.27 - 2007.03.02
//        -    
//
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Mosaic.h"
#include "./WinDefAdd.h"
#ifdef __FAST_MINES_PROJECT__
   #include "./Lang.h"
#endif
#include "./Mosaic/Hexagon1.h"
#include "./Mosaic/Parquet1.h"
#include "./Mosaic/Parquet2.h"
#include "./Mosaic/PentagonT10.h"
#include "./Mosaic/PentagonT24.h"
#include "./Mosaic/PentagonT5.h"
#include "./Mosaic/Quadrangle1.h"
#include "./Mosaic/Rhombus1.h"
#include "./Mosaic/SqTrHex.h"
#include "./Mosaic/Square1.h"
#include "./Mosaic/Square2.h"
#include "./Mosaic/Trapezoid1.h"
#include "./Mosaic/Trapezoid2.h"
#include "./Mosaic/Trapezoid3.h"
#include "./Mosaic/Triangle1.h"
#include "./Mosaic/Triangle2.h"
#include "./Mosaic/Triangle3.h"
#include "./Mosaic/Triangle4.h"
#include "./Mosaic/TrSq1.h"
#include "./Mosaic/TrSq2.h"

////////////////////////////////////////////////////////////////////////////////
//                             defines
////////////////////////////////////////////////////////////////////////////////

#define ID_TIMER_CHANGE_TIME         122 //      

#define ID_TIMER_ASSISTANT_WORK_FIRST 123 //        -  . CAssistantInfo::m_iTimeoutFirst
#define ID_TIMER_ASSISTANT_WORK_NEXT  124 //      -  . CAssistantInfo::m_iTimeoutNext
#define ID_TIMER_ASSISTANT_CHECK_CLICK_DATA  125

#define ID_TIMER_PAUSE 126 //      

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////
/* static */ float CMosaic::GetPercentMine(const ESkillLevel skill, const EMosaic mosaic) { //      
   switch (mosaic) {
#define PERCENT(A, B, C, D)  switch (skill) { case eSkillLevelBeginner: return A; case eSkillLevelAmateur: return B; case eSkillLevelProfessional: return C; case eSkillLevelCrazy: return D; }
   case eMosaicTriangle1  : PERCENT(16.5f  , 19.5f  , 22.5f  , 26.5f  );
   case eMosaicTriangle2  : PERCENT(15.f   , 18.f   , 21.f   , 25.f   );
   case eMosaicTriangle3  : PERCENT(17.143f, 20.143f, 23.143f, 27.143f);
   case eMosaicTriangle4  : PERCENT(19.762f, 22.762f, 25.762f, 29.762f);
   case eMosaicSquare1    : PERCENT(15.f   , 18.f   , 21.f   , 25.f   );
   case eMosaicSquare2    : PERCENT(14.f   , 17.f   , 20.f   , 24.f   );
   case eMosaicParquet1   : PERCENT(14.f   , 17.f   , 20.f   , 24.f   );
   case eMosaicParquet2   : PERCENT(14.429f, 17.429f, 20.429f, 24.429f);
   case eMosaicTrapezoid1 : PERCENT(14.5f  , 17.5f  , 20.5f  , 24.5f  );
   case eMosaicTrapezoid2 : PERCENT(15.222f, 18.222f, 21.222f, 25.222f);
   case eMosaicTrapezoid3 : PERCENT(16.015f, 19.015f, 22.015f, 26.015f);
   case eMosaicRhombus1   : PERCENT(15.6f  , 18.6f  , 21.6f  , 25.6f  );
   case eMosaicQuadrangle1: PERCENT(15.222f, 18.222f, 21.222f, 25.222f);
   case eMosaicPentagonT24: PERCENT(14.429f, 17.429f, 20.429f, 24.429f);
   case eMosaicPentagonT5 : PERCENT(14.5f  , 17.5f  , 20.5f  , 24.5f  );
   case eMosaicPentagonT10: PERCENT(14.151f, 17.151f, 20.151f, 24.151f);
   case eMosaicHexagon1   : PERCENT(14.f   , 17.f   , 20.f   , 24.f   );
   case eMosaicTrSq1      : PERCENT(15.852f, 18.852f, 21.852f, 25.852f);
   case eMosaicTrSq2      : PERCENT(15.809f, 18.809f, 21.809f, 25.809f);
   case eMosaicSqTrHex    : PERCENT(15.5f  , 18.5f  , 21.5f  , 25.5f  );
   default /*eMosaicNil*/ : return 1;
   }
#undef PERCENT
}

/* static */ int CMosaic::DefineNumberMines(ESkillLevel skill, EMosaic mosaic, const SIZE& sizeMosaic) {
   /** /
   int iNeighborNumber = 3;
   switch (mosaic) {
#define CASE_MOSAIC(name) case mosaic##name: iNeighborNumber = C##name(COORDEX(0,0), SIZEEX(0,0), 100, CGraphicContext()).GetNeighborNumber(); break;
   CASE_MOSAIC(Triangle1  )
   CASE_MOSAIC(Triangle2  )
   CASE_MOSAIC(Triangle3  )
   CASE_MOSAIC(Triangle4  )
   CASE_MOSAIC(Square1    )
   CASE_MOSAIC(Square2    )
   CASE_MOSAIC(Parquet1   )
   CASE_MOSAIC(Parquet2   )
   CASE_MOSAIC(Trapezoid1 )
   CASE_MOSAIC(Trapezoid2 )
   CASE_MOSAIC(Trapezoid3 )
   CASE_MOSAIC(Rhombus1   )
   CASE_MOSAIC(Quadrangle1)
   CASE_MOSAIC(PentagonT24)
   CASE_MOSAIC(PentagonT5 )
   CASE_MOSAIC(PentagonT10)
   CASE_MOSAIC(Hexagon1   )
   CASE_MOSAIC(TrSq1      )
   CASE_MOSAIC(TrSq2      )
   CASE_MOSAIC(SqTrHex    )
#undef CASE_MOSAIC
   }

   float fSkillKoef = 1.; //  

   /** /
   //  1
   switch (skill) {
   case skillLevelBeginner    : fSkillKoef = 0.01875f*1.0f; break;
   case skillLevelAmateur     : fSkillKoef = 0.01875f*1.2f; break;
   case skillLevelProfessional: fSkillKoef = 0.01875f*1.4f; break;
   case skillLevelCrazy       : fSkillKoef = 0.01875f*1.6f; break;
   }
   return fSkillKoef * sizeMosaic.cx*sizeMosaic.cy * iNeighborNumber;

   /** /

   //  2
   switch (skill) {
   case skillLevelBeginner    : fSkillKoef = 1.20f; break;
   case skillLevelAmateur     : fSkillKoef = 1.44f; break;
   case skillLevelProfessional: fSkillKoef = 1.68f; break;
   case skillLevelCrazy       : fSkillKoef = 1.92f; break;
   }
   return fSkillKoef * sizeMosaic.cx*sizeMosaic.cy / iNeighborNumber;

   /**/

   //  3 ()
   if (skill == eSkillLevelCustom)
      return 0; // error
   return toINT(sizeMosaic.cx *
                sizeMosaic.cy * GetPercentMine(skill, mosaic) / 100);
}

//void LoadDefaultImageMine   (HINSTANCE hInstance, CImage &ImgMine   ) {ImgMine   .LoadResource(hInstance , _T("Mine" ), imageBitmap     ); ImgMine   .SetTransparent(true); ImgMine   .SetPlace(placeStretch);}
//void LoadDefaultImageFlag   (HINSTANCE hInstance, CImage &ImgFlag   ) {ImgFlag   .LoadResource(hInstance , _T("Flag" ), imageBitmap     ); ImgFlag   .SetTransparent(true); ImgFlag   .SetPlace(placeStretch);}
//void LoadDefaultImagePause  (HINSTANCE hInstance, CImage &ImgPause  ) {ImgPause  .LoadResource(hInstance , _T("Pause"), imageEnhMetafile); ImgPause  .SetTransparent(true); ImgPause  .SetPlace(placeCenter );}
//void LoadDefaultImageBckgrnd(HINSTANCE hInstance, CImage &ImgBckgrnd) {ImgBckgrnd.Reset();                                                 ImgBckgrnd.SetTransparent(true); ImgBckgrnd.SetPlace(placeStretch);}

LRESULT CMosaic::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
   /**
   switch (msg) {
   case WM_MOUSEMOVE:
   case WM_NCHITTEST:
   case WM_SETCURSOR:
      break;
   default:
      g_Log.PutMsg(CLogger::LL_DEBUG, _T("CMosaic"), msg);
   }/**/
   CMosaic *const This = (CMosaic*)::GetWindowUserData(hWnd);
   if (This) {
      switch(msg){
      HANDLE_MSG(hWnd, WM_PAINT        , This->OnPaint);
      HANDLE_MSG(hWnd, WM_LBUTTONUP    , This->OnLButtonUp);
      HANDLE_MSG(hWnd, WM_LBUTTONDOWN  , This->OnLButtonDown);
      HANDLE_MSG(hWnd, WM_LBUTTONDBLCLK, This->OnLButtonDown);
      HANDLE_MSG(hWnd, WM_RBUTTONDOWN  , This->OnRButtonDown);
      HANDLE_MSG(hWnd, WM_RBUTTONDBLCLK, This->OnRButtonDown);
      HANDLE_MSG(hWnd, WM_SIZE         , This->OnSize);
      HANDLE_MSG(hWnd, WM_TIMER        , This->OnTimer);
      case WM_ERASEBKGND: return -1;
      }
   }
   return ::DefWindowProc(hWnd, msg, wParam, lParam);
}

// constructor
CMosaic::CMosaic():
   m_hWnd      (NULL),
   m_iOldMines (1),
   m_CoordDown (INCORRECT_COORD),
#pragma warning(disable:4355) // 'this' : used in base member initializer list
   m_Assistant(*this),
#pragma warning(default:4355)
   m_iCurrPause(-1)
{
   static WNDCLASS wc = {
      CS_DBLCLKS,                        // UINT    style
      CMosaic::WndProc,                  // WNDPROC lpfnWndProc
      0,                                 // int     cbClsExtra
      0,                                 // int     cbWndExtra
      ::GetModuleHandle(NULL),           // HANDLE  hInstance
      NULL,                              // HICON   hIcon
      ::LoadCursor(NULL, IDC_ARROW),     // HCURSOR hCursor
      ::GetSysColorBrush(COLOR_BTNFACE), // HBRUSH  hbrBackground
      NULL,                              // LPCTSTR lpszMenuName
      CMosaic::GetWindowClass()          // LPCTSTR lpszClassName
   };
   static ATOM atom  = ::RegisterClass(&wc);
}

BOOL CMosaic::Create(HWND hWndParent) {
   m_hWnd = ::CreateWindow(CMosaic::GetWindowClass(), NULL,
      WS_CHILD | WS_VISIBLE,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      hWndParent, (HMENU)0, ::GetModuleHandle(NULL), NULL);
   if (!m_hWnd) {
      return FALSE;
   }
   ::SetWindowUserData(m_hWnd, (LONG_PTR)this);

   //
   m_GContext.m_hDCWnd   = ::GetWindowDC(m_hWnd);
   m_GContext.m_hDCDst   = ::CreateCompatibleDC(m_GContext.m_hDCWnd);//m_GContext.m_hDCWnd;//
   m_GContext.m_hDCBck   = ::CreateCompatibleDC(m_GContext.m_hDCWnd);
   m_GContext.m_hDCTmp   = ::CreateCompatibleDC(m_GContext.m_hDCWnd);
   m_GContext.m_hDCPause = ::CreateCompatibleDC(m_GContext.m_hDCWnd);
   ::SetBkMode (m_GContext.m_hDCTmp, TRANSPARENT); //     
 //::SetBkColor(m_GContext.m_hDCTmp, RGB(255, 255, 255));
   m_GContext.m_hPenShadow = NULL;
   m_GContext.m_hPenLight  = NULL;
   m_GContext.m_hBrush     = NULL;

   m_GContext.m_SizeBitmap.cx = m_GContext.m_SizeBitmap.cy = 0;

   // load default images
   //LoadDefaultImageMine   (::GetModuleHandle(NULL), m_GContext.m_ImgMine   );
   //LoadDefaultImageFlag   (::GetModuleHandle(NULL), m_GContext.m_ImgFlag   );
   //LoadDefaultImagePause  (::GetModuleHandle(NULL), m_GContext.m_ImgPause  );
   //LoadDefaultImageBckgrnd(::GetModuleHandle(NULL), m_GContext.m_ImgBckgrnd);

   SetClasureToMosaic();
   SetBorder(m_GContext.m_Border);
   SetBrush (m_GContext.m_colorBk);

   m_GameStatus = eGSEnd;
   m_bPause = false;

   MosaicCreate();

   return TRUE;
}

// destructor
CMosaic::~CMosaic(){
   MosaicDestroy(/*m_SerializeData.m_SizeMosaic*/);
   //m_Mosaic.~vector();
   ::DeleteDC(m_GContext.m_hDCDst);
   ::DeleteDC(m_GContext.m_hDCBck);
   ::DeleteDC(m_GContext.m_hDCTmp);
   ::ReleaseDC(m_hWnd, m_GContext.m_hDCWnd);
   ::DeleteObject(m_GContext.m_hBrush);
   ::DeleteObject(m_GContext.m_hPenShadow);
   ::DeleteObject(m_GContext.m_hPenLight);
   ::DeleteObject(m_GContext.m_hFont);
}

////////////////////////////////////////////////////////////////////////////////
//                       " "
////////////////////////////////////////////////////////////////////////////////
// WM_PAINT
void CMosaic::OnPaint(HWND hwnd){
   ::DefWindowProc(hwnd, WM_PAINT, 0L, 0L); //      WM_PAINT   - BeginPaint ... EndPaint

   if (m_bPause) {
      if (!m_GContext.m_isRefreshPause) { // restore bitmap
         ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
                  m_GContext.m_hDCPause, 0, 0, SRCCOPY);
         return;
      }
      ::PatBlt(m_GContext.m_hDCPause, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy, PATCOPY);
      RECT Rect = {0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy};
      ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
      m_GContext.m_ImgsPause[m_iCurrPause].Draw(m_GContext.m_hDCPause, Rect);
      ::SetCursor(::LoadCursor(NULL, IDC_ARROW));
      ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
               m_GContext.m_hDCPause, 0, 0, SRCCOPY);
    //::SelectObject(m_GContext.m_hDCWnd, m_GContext.m_hBrush); //    
      m_GContext.m_isRefreshPause = false;
      return;
   }

   if (!m_GContext.m_isRefreshMosaic) { // restore bitmap
      ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
               m_GContext.m_hDCDst, 0, 0, SRCCOPY);
      return;
   }

   ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
   // background color
   ::PatBlt(m_GContext.m_hDCDst, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy, PATCOPY);
   ::PatBlt(m_GContext.m_hDCBck, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy, PATCOPY);
   // background image
   {
      RECT Rect = {0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy};
      m_GContext.m_ImgBckgrnd.Draw(m_GContext.m_hDCBck, Rect);
   }
   ::BitBlt(m_GContext.m_hDCTmp, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy, m_GContext.m_hDCBck, 0,0, SRCCOPY);
 //::BitBlt(m_GContext.m_hDCDst, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy, m_GContext.m_hDCBck, 0,0, SRCCOPY);
   // paint cells
   for (int i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
         m_Mosaic[i][j]->Paint();
   ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
            m_GContext.m_hDCDst, 0, 0, SRCCOPY);
   ::SetCursor(::LoadCursor(NULL, IDC_ARROW));
   m_GContext.m_isRefreshMosaic = false;
}

// WM_LBUTTONDOWN & WM_LBUTTONDBLCLK
void CMosaic::OnLButtonDown(HWND, BOOL fDoubleClick, int x, int y, UINT keyFlags){
   //::DefWindowProc(m_hWnd, fDoubleClick ? WM_LBUTTONDBLCLK : WM_LBUTTONDOWN, (WPARAM)keyFlags, MAKELPARAM(x,y));
   m_CoordDown = INCORRECT_COORD;
   if (m_GameStatus == eGSEnd) return;
   if (GetPause()) return;
   ::SetCapture(m_hWnd);
   COORD coordLDown = WinToArray(x,y);
   if (coordLDown.X < 0) return;
   FORWARD_WM_MOSAIC_CLICK(::GetParent(m_hWnd), TRUE, TRUE, ::SendMessage);
   OnLButtonDown_F(coordLDown);
}

void CMosaic::OnLButtonDown_F(const COORD &coordLDown){
   m_CoordDown = coordLDown;
   if (m_GameStatus == eGSCreateGame) {
      if(m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->Cell_GetOpen() != CCell::_Mine) {
         m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->Cell_SetStatus(CCell::_Open, NULL);
         m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->Cell_SetMine();
         m_SerializeData.m_iMines++;
         m_RepositoryMines.Add(m_CoordDown);
      } else {
         m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->Reset();
         m_SerializeData.m_iMines--;
         m_RepositoryMines.Del(m_CoordDown);
      }
      m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->Paint();
      FORWARD_WM_MOSAIC_CHANGECOUNTERS(::GetParent(m_hWnd), ::PostMessage);
   } else {
      m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->LButtonDown();
   }
   ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
            m_GContext.m_hDCDst, 0, 0, SRCCOPY);
}

// WM_LBUTTONUP
void CMosaic::OnLButtonUp(HWND, int x, int y, UINT keyFlags){
   //::DefWindowProc(m_hWnd, WM_LBUTTONUP, (WPARAM)keyFlags, MAKELPARAM(x,y));
   if (m_GameStatus == eGSEnd) return;
   if (GetPause()) return;
   ::ReleaseCapture();
   if (m_CoordDown.X < 0) return;
   FORWARD_WM_MOSAIC_CLICK(::GetParent(m_hWnd), TRUE, FALSE, ::SendMessage);
   if (m_GameStatus == eGSCreateGame) return;

   m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
   ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);
   ResetAssistant();

   COORD coordLUp = WinToArray(x,y);
   OnLButtonUp_F(coordLUp, false);
}

void CMosaic::OnLButtonUp_F(const COORD &coordLUp, bool bFromAssistant) {
   if ((m_GameStatus == eGSReady) &&
       (coordLUp == m_CoordDown)) GameBegin(m_CoordDown);
   CCell::CClickReportContext ClickReportContext;
   const CLeftUpReturn result =
      m_Mosaic[m_CoordDown.X][m_CoordDown.Y]->LButtonUp(coordLUp==m_CoordDown, &ClickReportContext);
   m_iCountOpen    += result.m_iCountOpen;
   m_iCountFlag    += result.m_iCountFlag;
   m_iCountUnknown += result.m_iCountUnknown;
   if (result.m_iCountOpen || result.m_iCountFlag || result.m_iCountUnknown) { //    (   )
      m_iCountClick++;
      if (bFromAssistant) { //    ...
         m_PlayInfo |= ePIPlayerAssistant;
      } else {
         m_PlayInfo |= ePIPlayerUser;  //  
         //gAssistant.ResetSequentialMove(); //   ,   ...
      }
   }
   FORWARD_WM_MOSAIC_CHANGECOUNTERS(::GetParent(m_hWnd), ::PostMessage);
   if (result.m_bEndGame) {
      GameEnd(result.m_bVictory);
   } else {
      if (m_iCountOpen+m_SerializeData.m_iMines == m_SerializeData.m_SizeMosaic.cx*m_SerializeData.m_SizeMosaic.cy) {
         GameEnd(true);
      } else {
         VerifyFlag();
         if (m_GameStatus != eGSEnd) {
            m_Assistant.ClickEnd(ClickReportContext);
         }
      }
   }

   ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
            m_GContext.m_hDCDst, 0, 0, SRCCOPY);

   if (bFromAssistant) { //    
      //   
      ::SetTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_NEXT, m_SerializeData.m_AssistantInfo.m_Timeout.m_iNext, NULL);
   }
}

// WM_RBUTTONDOWN & WM_RBUTTONDBLCLK
void CMosaic::OnRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags){
   //::DefWindowProc(hwnd, fDoubleClick ? WM_RBUTTONDBLCLK : WM_RBUTTONDOWN, (WPARAM)keyFlags, MAKELPARAM(x,y));
   if (m_GameStatus == eGSEnd) {
      if (!fDoubleClick)
         GameNew();
      else {
         //BeepSpeaker();
      }
      return;
   }
   if (m_GameStatus == eGSReady) return;
   if (m_GameStatus == eGSCreateGame) return;
   if (GetPause()) {
      SetPause(false);
      return;
   }

   m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
   ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);
   ResetAssistant();
   
   COORD coordRDown = WinToArray(x,y);
   if (coordRDown.X < 0) return;
   OnRButtonDown_F(coordRDown, false, CCell::_Unknown /*  , ..  */);
}

void CMosaic::OnRButtonDown_F(const COORD &coordRDown, bool bFromAssistant, CCell::EClose setCloseAssistant) {
   CCell::CClickReportContext ClickReportContext;
   CCell::EClose setClose;
   if (bFromAssistant) { //    
      m_PlayInfo |= ePIPlayerAssistant; //  
      setClose = setCloseAssistant;//(CCell::EClose)HIWORD(keyFlags);
   } else {
      switch (m_Mosaic[coordRDown.X][coordRDown.Y]->Cell_GetClose()) {
      case CCell::_Clear  : setClose = CCell::_Flag; break;
      case CCell::_Flag   : setClose = m_SerializeData.m_useUnknown ? CCell::_Unknown :
                                                                       CCell::_Clear; break;
      case CCell::_Unknown: setClose = CCell::_Clear;
      }
   }
   const CRightDownReturn result = m_Mosaic[coordRDown.X][coordRDown.Y]->RButtonDown(setClose, &ClickReportContext);
   if (result.m_iCountFlag || result.m_iCountUnknown) { //    (   )
      m_iCountClick++;
      if (!bFromAssistant) { //     
         m_PlayInfo |= ePIPlayerUser; //     
         //gAssistant.ResetSequentialMove(); //   ,   ...
      }
   }
   m_iCountFlag    += result.m_iCountFlag;
   m_iCountUnknown += result.m_iCountUnknown;
   FORWARD_WM_MOSAIC_CHANGECOUNTERS(::GetParent(m_hWnd), ::PostMessage);
   VerifyFlag();
   if (m_GameStatus != eGSEnd) {
      m_Assistant.ClickEnd(ClickReportContext);
   }
   ::BitBlt(m_GContext.m_hDCWnd, 0, 0, m_GContext.m_SizeBitmap.cx, m_GContext.m_SizeBitmap.cy,
            m_GContext.m_hDCDst, 0, 0, SRCCOPY);

   if (bFromAssistant) { //    
      //   
      ::SetTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_NEXT, m_SerializeData.m_AssistantInfo.m_Timeout.m_iNext, NULL);
   }
}

// WM_SIZE
void CMosaic::OnSize(HWND hwnd, UINT state, int cx, int cy) {
   if ((m_GContext.m_SizeBitmap.cx != cx) || (m_GContext.m_SizeBitmap.cy != cy)) {
      m_GContext.m_SizeBitmap.cx = cx;
      m_GContext.m_SizeBitmap.cy = cy;
      ::DeleteObject(::SelectObject(m_GContext.m_hDCDst  , ::CreateCompatibleBitmap(m_GContext.m_hDCWnd, m_GContext.m_SizeBitmap.cx,m_GContext.m_SizeBitmap.cy)));
      ::DeleteObject(::SelectObject(m_GContext.m_hDCBck  , ::CreateCompatibleBitmap(m_GContext.m_hDCWnd, m_GContext.m_SizeBitmap.cx,m_GContext.m_SizeBitmap.cy)));
      ::DeleteObject(::SelectObject(m_GContext.m_hDCTmp  , ::CreateCompatibleBitmap(m_GContext.m_hDCWnd, m_GContext.m_SizeBitmap.cx,m_GContext.m_SizeBitmap.cy)));
      ::DeleteObject(::SelectObject(m_GContext.m_hDCPause, ::CreateCompatibleBitmap(m_GContext.m_hDCWnd, m_GContext.m_SizeBitmap.cx,m_GContext.m_SizeBitmap.cy)));
   }
   m_GContext.m_isRefreshMosaic = m_GContext.m_isRefreshPause = true;
   //FORWARD_WM_SIZE(hwnd, state, cx, cy, ::DefWindowProc);
}

////////////////////////////////////////////////////////////////////////////////
//                             other function
////////////////////////////////////////////////////////////////////////////////
void CMosaic::MosaicDestroy(/*const SIZE& SizeMosaic*/) {
   m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
   ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);
   //delete [] m_Mosaic;
   for (int i = 0; i < m_Mosaic.size()/*SizeMosaic.cx*/; i++) {
      for (int j = 0; j < m_Mosaic[i].size()/*SizeMosaic.cy*/; j++) {
         //m_Mosaic[i][j]->~TcBase();
         //g_Log.Put(CLogger::LL_DEBUG, _T("delete [%i,%i]"),i,j);
         delete m_Mosaic[i][j]; m_Mosaic[i][j] = NULL;
      }
      //m_Mosaic[i].clear(); // ,  
   }
   //m_Mosaic.clear(); // ,  
}

void CMosaic::MosaicCreate() {
   m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
   ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);

   //m_Mosaic = new TCell[m_SerializeData.m_SizeMosaic.cx*m_SerializeData.m_SizeMosaic.cy];
   m_Mosaic.resize(m_SerializeData.m_SizeMosaic.cx);
   int i=0;
   for (; i < m_SerializeData.m_SizeMosaic.cx; i++){
      m_Mosaic[i].resize(m_SerializeData.m_SizeMosaic.cy);
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++){
         COORD coord = {i,j};
         switch (m_SerializeData.m_Mosaic) {
#define CASE_MOSAIC(name) case eMosaic##name: m_Mosaic[i][j] = new C##name(coord, m_SerializeData.m_SizeMosaic, m_SerializeData.m_iArea, m_GContext); break;
         CASE_MOSAIC(Triangle1  )
         CASE_MOSAIC(Triangle2  )
         CASE_MOSAIC(Triangle3  )
         CASE_MOSAIC(Triangle4  )
         CASE_MOSAIC(Square1    )
         CASE_MOSAIC(Square2    )
         CASE_MOSAIC(Parquet1   )
         CASE_MOSAIC(Parquet2   )
         CASE_MOSAIC(Trapezoid1 )
         CASE_MOSAIC(Trapezoid2 )
         CASE_MOSAIC(Trapezoid3 )
         CASE_MOSAIC(Rhombus1   )
         CASE_MOSAIC(Quadrangle1)
         CASE_MOSAIC(PentagonT24)
         CASE_MOSAIC(PentagonT5 )
         CASE_MOSAIC(PentagonT10)
         CASE_MOSAIC(Hexagon1   )
         CASE_MOSAIC(TrSq1      )
         CASE_MOSAIC(TrSq2      )
         CASE_MOSAIC(SqTrHex    )
         default /*mosaicNil*/ : m_Mosaic[i][j] = new CCell(coord, m_SerializeData.m_SizeMosaic, m_SerializeData.m_iArea, m_GContext,0,0,0); break;
#undef CASE_MOSAIC
         }
      }
   }
   int numberNeighbor = GetNeighborNumber();
   CCell**const ppLinkNeighbor = new CCell*[numberNeighbor];
   for (i=0; i < m_SerializeData.m_SizeMosaic.cx; i++){
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++){
         for (int k=0; k<numberNeighbor; k++) {
            const COORD &coordNeighbor = m_Mosaic[i][j]->GetNeighborCoord(k);
            if (coordNeighbor != INCORRECT_COORD)
                  ppLinkNeighbor[k] = m_Mosaic[coordNeighbor.X][coordNeighbor.Y];
            else  ppLinkNeighbor[k] = NULL;
         }
         m_Mosaic[i][j]->SetNeighborLink(ppLinkNeighbor, numberNeighbor);
      }
   }
   delete [] ppLinkNeighbor;

   ::SendMessage(::GetParent(m_hWnd), WM_SIZE, 0L, 0L);
   FORWARD_WM_MOSAIC_ADJUSTAREA(::GetParent(m_hWnd), ::SendMessage);

   SetPause(false);
   m_GameStatus = eGSEnd;
   GameNew();
   ResetAssistant();
}

void CMosaic::GameNew(bool bFromAssistant) {
   if (m_GameStatus == eGSReady) return;
   if (m_bPause) {
      SetPause(false);
      return;
   }
   if (m_RepositoryMines.GetSize()) {
      if (m_GameStatus == eGSCreateGame) {
      } else {
         if (!bFromAssistant &&
             (IDNO == MessageBox(m_hWnd,
            #ifdef __FAST_MINES_PROJECT__
               CLang::GetString(IDS__RESTORE_LAST_GAME), CLang::GetString(IDS__QUESTION),
            #else
               _T("Restore last game?"), _T("Question"),
            #endif
               MB_YESNO | MB_ICONQUESTION)))
         {
            m_RepositoryMines.Reset();
         }
         SetPause(false);
      }
   }

   if (!bFromAssistant) {
      m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
      ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);
      ResetAssistant();
   }

   SetPause((m_GameStatus = eGSReady, false));
   FORWARD_WM_MOSAIC_CHANGEGAMESTATUS(::GetParent(m_hWnd), m_GameStatus, ::SendMessage);
   //EnableWindow(m_hWnd, true); // enable all cell

   m_iCountTimer = m_iCountClick = m_iCountFlag = m_iCountOpen = m_iCountUnknown = 0;
   m_GContext.m_isRefreshMosaic = m_GContext.m_isRefreshPause = true;
   FORWARD_WM_MOSAIC_CHANGECOUNTERS(::GetParent(m_hWnd), ::PostMessage);

   for (int i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
         m_Mosaic[i][j]->Reset();

   ::InvalidateRect(m_hWnd, NULL, FALSE);
   m_PlayInfo = ePIPlayerUnknown; //      

   m_Assistant.InitForNewGame();
}

void CMosaic::GameBegin(COORD firstClick) {
   int count;
   int i, j;

   SetPause((m_GameStatus = eGSPlay, false));
   FORWARD_WM_MOSAIC_CHANGEGAMESTATUS(::GetParent(m_hWnd), m_GameStatus, ::PostMessage);

   // set mines
   if (m_RepositoryMines.GetSize()) {
      m_PlayInfo |= ePIIgnorable;
      size_t size = m_RepositoryMines.GetSize();
      for (int i=0; i<size; i++) {
         COORD c = m_RepositoryMines[i];
         if (!m_Mosaic[c.X][c.Y]->Cell_SetMine()) {
#ifdef _DEBUG
            ::MessageBox(NULL, _T("   ... :("),
            #ifdef __FAST_MINES_PROJECT__
               CLang::GetString(IDS__ERROR),
            #else
               _T("Error"),
            #endif
               MB_OK | MB_ICONERROR);
            break;
#endif
         }
      }
   } else {
      if (!m_SerializeData.m_iMines)
         m_SerializeData.m_iMines = m_iOldMines;
      // set random mines
      m_Mosaic[firstClick.X][firstClick.Y]->LockNeighbor(); //        
      count = 0;
      do {
         i = rand(m_SerializeData.m_SizeMosaic.cx-1);
         j = rand(m_SerializeData.m_SizeMosaic.cy-1);
         if (m_Mosaic[i][j]->Cell_SetMine())
            count++;
      } while (count < m_SerializeData.m_iMines);
   }
   // set other CellOpen and set all Caption
   for (i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
      for (j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
         m_Mosaic[i][j]->Cell_DefineValue();
}

void CMosaic::GameCreate() {
   SetPause(false);
   GameNew();
   if (!m_RepositoryMines.GetSize()) {
      if (m_SerializeData.m_iMines)
         m_iOldMines = m_SerializeData.m_iMines;
      m_SerializeData.m_iMines = 0;
      m_GameStatus = eGSCreateGame;
      FORWARD_WM_MOSAIC_CHANGECOUNTERS  (::GetParent(m_hWnd), ::PostMessage);
      FORWARD_WM_MOSAIC_CHANGEGAMESTATUS(::GetParent(m_hWnd), m_GameStatus, ::PostMessage);
   }
}

void CMosaic::GameEnd(bool Victory) {
   if (m_GameStatus == eGSEnd) return;

   int i, j;

   SetPause((m_GameStatus = eGSEnd, false));
   //  e
   {
      ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
      if (Victory) {
         for (i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
            for (j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++) {
               if (m_Mosaic[i][j]->Cell_GetStatus() == CCell::_Close) {
                  if(m_Mosaic[i][j]->Cell_GetOpen() == CCell::_Mine)
                     m_Mosaic[i][j]->Cell_SetClose(CCell::_Flag , NULL);
                  else {
                     m_Mosaic[i][j]->Cell_SetStatus(CCell::_Open, NULL);
                     m_Mosaic[i][j]->Cell_SetDown(true);
                  }
                  m_Mosaic[i][j]->Paint();
               }
            }
         m_iCountFlag = m_SerializeData.m_iMines;
      } else
         for (i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
            for (j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
               if (m_Mosaic[i][j]->Cell_GetStatus() == CCell::_Close) {
                  if ((m_Mosaic[i][j]->Cell_GetClose() == CCell::_Flag) &&
                      (m_Mosaic[i][j]->Cell_GetOpen()  == CCell::_Mine))
                       m_Mosaic[i][j]->Cell_SetStatus(CCell::_Close, NULL);
                  else m_Mosaic[i][j]->Cell_SetStatus(CCell::_Open , NULL);
                  m_Mosaic[i][j]->Paint();
               }
      ::SetCursor(::LoadCursor(NULL, IDC_ARROW));
   }
   //BeepSpeaker();

   FORWARD_WM_MOSAIC_CHANGECOUNTERS  (::GetParent(m_hWnd), ::PostMessage);
   FORWARD_WM_MOSAIC_CHANGEGAMESTATUS(::GetParent(m_hWnd), m_GameStatus, ::SendMessage);
}

inline void CMosaic::VerifyFlag() {
   if (m_GameStatus == eGSEnd) return;
   if (m_SerializeData.m_iMines == m_iCountFlag) {
      for (int i=0; i < m_SerializeData.m_SizeMosaic.cx; i++)
         for (int j=0; j < m_SerializeData.m_SizeMosaic.cy; j++)
            if ((m_Mosaic[i][j]->Cell_GetClose() == CCell::_Flag) &&
                (m_Mosaic[i][j]->Cell_GetOpen () != CCell::_Mine))
               return; //    -  
      GameEnd(true);
   } else
      if (m_SerializeData.m_iMines == m_iCountFlag+m_iCountUnknown) {
         for (int i=0; i < m_SerializeData.m_SizeMosaic.cx; i++)
            for (int j=0; j < m_SerializeData.m_SizeMosaic.cy; j++)
               if (((m_Mosaic[i][j]->Cell_GetClose() == CCell::_Unknown) ||
                    (m_Mosaic[i][j]->Cell_GetClose() == CCell::_Flag)) &&
                   ( m_Mosaic[i][j]->Cell_GetOpen () != CCell::_Mine))
                  return; //     '?'-  
         GameEnd(true);
      }
   return;
}

void CMosaic::SetPause(bool newPause) {
   /**/
   if (newPause &&(m_GameStatus == eGSPlay)) {
      //   ,    
      //    
      if ((m_PlayInfo & ePIPlayerAssistant) &&
         !(m_PlayInfo & ePIPlayerUser))
      {
         return;
      }
   }
   /**/
   FORWARD_WM_MOSAIC_PAUSE(::GetParent(m_hWnd), ::PostMessage);
   if (m_GameStatus != eGSPlay) {
      ::KillTimer(m_hWnd, ID_TIMER_CHANGE_TIME);
      return;
   }
   m_bPause = newPause;
   if (m_bPause) {
      ::KillTimer(m_hWnd, ID_TIMER_CHANGE_TIME);
   } else {
      ::SetTimer(m_hWnd, ID_TIMER_CHANGE_TIME, 1000, NULL);
   }
   ::InvalidateRect(m_hWnd, NULL, FALSE);
   //EnableWindow(m_hWnd, !m_bPause);
}

void CMosaic::SetMosaic(EMosaic mosaic) {
   m_RepositoryMines.Reset();
   if (m_SerializeData.m_Mosaic == mosaic) {
      GameNew();
      return;
   }
   ESkillLevel skill = GetSkillLevel(); // skill level   !!!
   MosaicDestroy(/*m_SerializeData.m_SizeMosaic*/);
   m_SerializeData.m_Mosaic = mosaic;
   SetClasureToMosaic();
   MosaicCreate();
   if (skill == eSkillLevelCustom) { // skill level   !!!
      m_SerializeData.m_iMines = min(m_SerializeData.m_iMines, GetMaxMines());
   } else {
      m_SerializeData.m_iMines = DefineNumberMines(skill, m_SerializeData.m_Mosaic, m_SerializeData.m_SizeMosaic);
   }
}

void CMosaic::SetGame(EMosaic mosaic, const SIZE& newSizeMosaic, int numberMines, const CStorageMines &RepositoryMines) {
   SetMosaic(mosaic);
   SetSkillLevelCustom(newSizeMosaic, numberMines);
   m_RepositoryMines = RepositoryMines;
}

void CMosaic::SetSkillLevelCustom(const SIZE& newSizeMosaic, int numberMines) {
   if ((m_SerializeData.m_SizeMosaic == newSizeMosaic) &&
       (m_SerializeData.m_iMines     == numberMines)) return;
   MosaicDestroy(/*m_SerializeData.m_SizeMosaic*/);
   m_SerializeData.m_SizeMosaic = newSizeMosaic;
   m_SerializeData.m_iMines     = numberMines;
   MosaicCreate();
   m_RepositoryMines.Reset();
}

void CMosaic::SetSkillLevel(ESkillLevel skill) {
   if (skill == eSkillLevelCustom) {
      return;
   }
   if (GetSkillLevel() == skill) {
      GameNew();
      return;
   }
   m_RepositoryMines.Reset();
   MosaicDestroy(/*m_SerializeData.m_SizeMosaic*/);
   m_SerializeData.m_SizeMosaic = CMosaic::DefaultSize(skill);
   m_SerializeData.m_iMines = DefineNumberMines(skill, m_SerializeData.m_Mosaic, m_SerializeData.m_SizeMosaic);
   MosaicCreate();
}

EMosaic CMosaic::GetMosaic() const {
   return m_SerializeData.m_Mosaic;
}

ESkillLevel CMosaic::GetSkillLevel() const {
   for (BYTE i = eSkillLevelBeginner; i < eSkillLevelCustom; i++)
      if ((m_SerializeData.m_SizeMosaic == CMosaic::DefaultSize((ESkillLevel)i)) &&
          (m_SerializeData.m_iMines     == DefineNumberMines((ESkillLevel)i, m_SerializeData.m_Mosaic, CMosaic::DefaultSize((ESkillLevel)i))))
      {
         return (ESkillLevel)i;
      }
   return eSkillLevelCustom;
}

void CMosaic::OnTimer(HWND hWnd, UINT idEvent) {
   switch (idEvent) {
   case ID_TIMER_CHANGE_TIME:
      m_iCountTimer++;
      FORWARD_WM_MOSAIC_CHANGECOUNTERS(::GetParent(m_hWnd), ::SendMessage);
      break;
   case ID_TIMER_ASSISTANT_WORK_FIRST:
      ::KillTimer(hWnd, ID_TIMER_ASSISTANT_WORK_FIRST);
   case ID_TIMER_ASSISTANT_WORK_NEXT:
      ::KillTimer(hWnd, ID_TIMER_ASSISTANT_WORK_NEXT);
      m_Assistant.Sync().StartWorkStage();
      ::SetTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA, 10, NULL);
      break;
   case ID_TIMER_ASSISTANT_CHECK_CLICK_DATA:
      if (m_Assistant.Sync().HaveClickData()) {
         m_Assistant.OnClick();
      }
      break;
   case ID_TIMER_PAUSE:
      m_iCurrPause++;
      if (m_iCurrPause >= m_GContext.m_ImgsPause.size()) m_iCurrPause = 0;
      m_GContext.m_isRefreshPause = true;
      ::InvalidateRect(m_hWnd, NULL, TRUE);
      break;
   }
}

COORD CMosaic::WinToArray(int x, int y) const { //      m_Mosaic'a
   COORD result = INCORRECT_COORD;
   for (int i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++) {
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++) {
         if (PointInRect(POINTEX(x,y), m_Mosaic[i][j]->GetOuterRect())) //    ...
         {
            if (m_Mosaic[i][j]->PointInRegion(POINTEX(x,y))) {
               result = m_Mosaic[i][j]->GetCoord();
               return result;
            }
         }
      }
   }
   return result;
}

void CMosaic::SetArea(int newArea) {
   if (m_SerializeData.m_iArea == newArea) return;
   m_SerializeData.m_iArea = newArea;

   for (int i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
      for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
         m_Mosaic[i][j]->SetPoint(m_SerializeData.m_iArea);
   SetDCFont();
   ::SendMessage(::GetParent(m_hWnd), WM_SIZE, 0L, 0L);
   ::InvalidateRect(m_hWnd, NULL, FALSE);
}

void CMosaic::SetClasureToMosaic() {
   switch (m_SerializeData.m_Mosaic) {
#define CASE_MOSAIC(name) case eMosaic##name: GetSizeWindow = C##name::GetSizeInPixel; break;
   CASE_MOSAIC(Triangle1  )
   CASE_MOSAIC(Triangle2  )
   CASE_MOSAIC(Triangle3  )
   CASE_MOSAIC(Triangle4  )
   CASE_MOSAIC(Square1    )
   CASE_MOSAIC(Square2    )
   CASE_MOSAIC(Parquet1   )
   CASE_MOSAIC(Parquet2   )
   CASE_MOSAIC(Trapezoid1 )
   CASE_MOSAIC(Trapezoid2 )
   CASE_MOSAIC(Trapezoid3 )
   CASE_MOSAIC(Rhombus1   )
   CASE_MOSAIC(Quadrangle1)
   CASE_MOSAIC(PentagonT24)
   CASE_MOSAIC(PentagonT5 )
   CASE_MOSAIC(PentagonT10)
   CASE_MOSAIC(Hexagon1   )
   CASE_MOSAIC(TrSq1      )
   CASE_MOSAIC(TrSq2      )
   CASE_MOSAIC(SqTrHex    )
   default /*mosaicNil*/: GetSizeWindow = CCell::GetSizeInPixel; break;
#undef CASE_MOSAIC
   }

   switch (m_SerializeData.m_Mosaic) {
#define CASE_MOSAIC(name) case eMosaic##name: GetSizeInscribedSquare = C##name::SizeInscribedSquare; break;
   CASE_MOSAIC(Triangle1  )
   CASE_MOSAIC(Triangle2  )
   CASE_MOSAIC(Triangle3  )
   CASE_MOSAIC(Triangle4  )
   CASE_MOSAIC(Square1    )
   CASE_MOSAIC(Square2    )
   CASE_MOSAIC(Parquet1   )
   CASE_MOSAIC(Parquet2   )
   CASE_MOSAIC(Trapezoid1 )
   CASE_MOSAIC(Trapezoid2 )
   CASE_MOSAIC(Trapezoid3 )
   CASE_MOSAIC(Rhombus1   )
   CASE_MOSAIC(Quadrangle1)
   CASE_MOSAIC(PentagonT24)
   CASE_MOSAIC(PentagonT5 )
   CASE_MOSAIC(PentagonT10)
   CASE_MOSAIC(Hexagon1   )
   CASE_MOSAIC(TrSq1      )
   CASE_MOSAIC(TrSq2      )
   CASE_MOSAIC(SqTrHex    )
   default /*mosaicNil*/: GetSizeInscribedSquare = CCell::SizeInscribedSquare; break;
#undef CASE_MOSAIC
   }
   SetDCFont();
}

inline void CMosaic::SetBrush(COLORREF colorSolidBrush) {
   if (m_GContext.m_hBrush) ::DeleteObject(m_GContext.m_hBrush);
   m_GContext.m_hBrush = ::CreateSolidBrush(colorSolidBrush);
   m_GContext.m_colorBk = colorSolidBrush;
   ::DeleteObject(::SelectObject(m_GContext.m_hDCDst  , m_GContext.m_hBrush)); // ???
   ::DeleteObject(::SelectObject(m_GContext.m_hDCBck  , m_GContext.m_hBrush)); // ???
   ::DeleteObject(::SelectObject(m_GContext.m_hDCPause, m_GContext.m_hBrush)); // ???
}

inline void CMosaic::SetDCFont() {
   m_GContext.m_Font.lfHeight = GetSizeInscribedSquare(m_SerializeData.m_iArea, m_GContext.m_Border.m_iWidth);
   m_GContext.m_hFont = ::CreateFontIndirect(&m_GContext.m_Font);
   //::DeleteObject(::SelectObject(m_GContext.m_hDCWnd, m_GContext.m_hFont));
   ::DeleteObject(::SelectObject(m_GContext.m_hDCTmp, m_GContext.m_hFont));
}

inline void CMosaic::SetFont(const LOGFONT& newFont) {
   m_GContext.m_Font = newFont;
   SetDCFont();
}

inline void CMosaic::SetCellColorText(const CCell::CColorText& newColorText) {
   m_GContext.m_ColorText = newColorText;
}

inline void CMosaic::SetBorder(const CCell::CBorder& newBorder) {
   int oldWidth = m_GContext.m_Border.m_iWidth;
   if (&(m_GContext.m_Border) != &newBorder) {
      m_GContext.m_Border = newBorder; // @TODO - ...
   }
   if (m_GContext.m_hPenShadow) ::DeleteObject(m_GContext.m_hPenShadow);
   m_GContext.m_hPenShadow = ::CreatePen(PS_SOLID, 2*newBorder.m_iWidth, newBorder.m_colorShadow); // Shadow pen
   if (m_GContext.m_hPenLight) ::DeleteObject(m_GContext.m_hPenLight);
   m_GContext.m_hPenLight  = ::CreatePen(PS_SOLID, 2*newBorder.m_iWidth, newBorder.m_colorLight ); // Light pen
   if(oldWidth != newBorder.m_iWidth) {
      if (!m_Mosaic.empty())
         for (int i = 0; i < m_SerializeData.m_SizeMosaic.cx; i++)
            for (int j = 0; j < m_SerializeData.m_SizeMosaic.cy; j++)
               m_Mosaic[i][j]->SetPoint(m_SerializeData.m_iArea); //     CCell::square (..        w)
      SetDCFont();
   }
}

void CMosaic::ApplySkin(const CSkinMosaic &newSkin) {

#define APPLY_IMAGE(m_Img, skinImg) \
   if (!skinImg.m_strPath.IsEmpty()) { \
      m_Img = skinImg; \
   } else { \
      /*LoadDefaultImage##name(::GetModuleHandle(NULL), m_GContext.m_Img##name);/**/ \
      m_Img.Reset(); \
      m_Img.SetPlace      (skinImg.m_Place); \
      m_Img.SetTransparent(skinImg.m_bTransparent); \
   }

   m_GContext.m_ImgsPause.clear();
   m_iCurrPause = 0;
   m_GContext.m_ImgsPause.resize(newSkin.m_ImgsPause.GetSize());
   for (int i=0; i<newSkin.m_ImgsPause.GetSize(); i++) {
      CImageSerialize imgTmpPause = newSkin.m_ImgsPause.GetImg(i);
//      if (CImage(imgTmpPause).Valid())
      {
         //m_GContext.m_ImgsPause.push_back(CImageSerialize());
         APPLY_IMAGE(m_GContext.m_ImgsPause[i], imgTmpPause);
      }
   }
   APPLY_IMAGE(m_GContext.m_ImgMine   , newSkin.m_ImgMine);
   APPLY_IMAGE(m_GContext.m_ImgFlag   , newSkin.m_ImgFlag);
   APPLY_IMAGE(m_GContext.m_ImgBckgrnd, newSkin.m_ImgBckgrnd);

#undef APPLY_IMAGE

   m_GContext.m_isRefreshMosaic = m_GContext.m_isRefreshPause = true;
   SetFont         (newSkin.m_Font);
   SetCellColorText(newSkin.m_ColorText);
   SetBorder       (newSkin.m_Border);
   SetBrush        (newSkin.GetBkColor());

   ::KillTimer(m_hWnd, ID_TIMER_PAUSE);
   if (newSkin.m_ImgsPause.GetSize() > 1) {
      ::SetTimer(m_hWnd, ID_TIMER_PAUSE, newSkin.m_iTimerPause*1000, NULL);
   }
}

BOOL CMosaic::SerializeRead(HANDLE hFile) {
   // read .dat file
   DWORD dwNOBR = 0;
   CSerialize SerializeData;
   BOOL bRes =
      ::ReadFile(hFile, &SerializeData, sizeof(CSerialize), &dwNOBR, NULL) &&
      (sizeof(CSerialize) == dwNOBR);
   if (bRes) {
      m_SerializeData = SerializeData;
      SetClasureToMosaic();
      MosaicCreate();
   }
   return bRes;
}

BOOL CMosaic::SerializeWrite(HANDLE hFile) const {
   // write .dat file
   DWORD dwNOBW = 0;
   return
      ::WriteFile(hFile, &m_SerializeData, sizeof(m_SerializeData), &dwNOBW, NULL) &&
      (sizeof(m_SerializeData) == dwNOBW);
}

void CMosaic::AssistantSuggest() {
   m_Assistant.Sync().SetSuggest();
   m_Assistant.Sync().StartWorkStage();
   ::SetTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA, 10, NULL);
}

void CMosaic::ResetAssistant() {
   if (m_SerializeData.m_AssistantInfo.m_bUse) {
      ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_NEXT);
      ::SetTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_FIRST, m_SerializeData.m_AssistantInfo.m_Timeout.m_iFirst*1000, NULL); // second to milisec
   } else {
      ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_FIRST);
      ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_WORK_NEXT);
      m_Assistant.Sync().AbortAndWaitCurrentWorkStage();
      ::KillTimer(m_hWnd, ID_TIMER_ASSISTANT_CHECK_CLICK_DATA);
   }
}

void CMosaic::SetAssistantInfo(const CAssistantInfo &newAssistantInfo) {
   bool bReset = (newAssistantInfo.m_bUse             != m_SerializeData.m_AssistantInfo.m_bUse) ||
                 (newAssistantInfo.m_bAutoStart && (m_GameStatus != eGSPlay)) ||
                 (newAssistantInfo.m_Timeout.m_iFirst != m_SerializeData.m_AssistantInfo.m_Timeout.m_iFirst) ||
                 (newAssistantInfo.m_Timeout.m_iNext  != m_SerializeData.m_AssistantInfo.m_Timeout.m_iNext );
   m_SerializeData.m_AssistantInfo = newAssistantInfo;
   if (bReset) {
      ResetAssistant();
   }
}

UINT CMosaic::GetMaxMines(const SIZE &size) const {
   int iMustFreeCell = GetNeighborNumber()+1;
   int iMaxMines = size.cx*size.cy-iMustFreeCell;
   return iMaxMines;
}

CMosaic::operator CStorageMines() const {
   CStorageMines RepositoryMines;
   for (int i=0; i<m_SerializeData.m_SizeMosaic.cx; i++)
      for (int j=0; j<m_SerializeData.m_SizeMosaic.cy; j++) {
         if (m_Mosaic[i][j]->Cell_GetOpen() == CCell::_Mine) {
            COORD cellMines = {i,j};
            RepositoryMines.Add(cellMines);
         }
      }
   return RepositoryMines;
}
