////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Base.h"
//
// O   CCell
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__BASE__
#define __FILE__BASE__

#include <set>
#include <math.h>
#include "../Image.h"

#ifndef __EXCLUDE_CRT__
   #define toINT(v)     int(v)
   #define toLONG(v)   long(v)
   #define toFLOAT(v) float(v)
#else __EXCLUDE_CRT__
   #define toINT(v)     toInt(v)
   #define toLONG(v)   toLong(v)
   #define toFLOAT(v) toFloat(v)

   #pragma intrinsic(sqrt, sin, cos, tan)

#endif __EXCLUDE_CRT__   

struct CLeftUpReturn {
   int m_iCountFlag, m_iCountOpen, m_iCountUnknown;
   bool m_bEndGame, m_bVictory;
};

struct CRightDownReturn {
   int m_iCountFlag, m_iCountUnknown;
};

#define INCORRECT_COORD COORDEX(-1,-1)

class CCell {
public:
   //////////////////////////
   //         TYPES
   //////////////////////////
   typedef std::set<const CCell*> SET_cpBase;

   enum EStatus{_Open, _Close};
   enum EOpen  {_Nil, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _Mine};
   enum EClose {_Unknown, _Clear, _Flag};

   struct CColorText {
      COLORREF m_colorOpen [_Mine+1];
      COLORREF m_colorClose[_Flag+1];
   };

   struct CBorder{
      COLORREF m_colorShadow,
               m_colorLight;
      int m_iWidth;                    //BBGGRR
      CBorder(COLORREF colorShadow = 0x00000000,
              COLORREF colorLight  = 0x00FFFFFF,
              int iWidth = 1):
         m_colorShadow(colorShadow),
         m_colorLight (colorLight),
         m_iWidth     (iWidth)
      {}
   };

   struct CClickReportContext {
      SET_cpBase m_SetOpenNil; //   (  )              
      SET_cpBase m_SetOpen;    //   ()              
      SET_cpBase m_SetFlag;    //      /   
   };

   struct CGraphicContext {
      HDC        m_hDCDst,
                 m_hDCBck,
                 m_hDCTmp;
      HPEN       m_hPenShadow,
                 m_hPenLight;
      CImage     m_ImgMine,
                 m_ImgFlag;
      CColorText m_ColorText;
      CBorder    m_Border;
      CGraphicContext(const CColorText &ColorText) :
         m_hDCDst     (NULL),
         m_hDCBck     (NULL),
         m_hDCTmp     (NULL),
         m_hPenShadow (NULL),
         m_hPenLight  (NULL),
         m_ColorText  (ColorText)
      {}
   };

private:
   mutable bool m_bPresumeFlag;
protected:
   static float& m_a () { static float  a; return  a; } //    (      )
   static float& m_sq() { static float sq; return sq; } //  ,   
public:
   static SIZE GetSizeInPixel(const SIZE &sizeField, int iArea);
   static int SizeInscribedSquare(int iArea, int iBorderWidth); //         
   static LPCTSTR GetCaptionOpen(EOpen eOpen) {
      static TCHAR szRes[_Mine+1][3] =
      {  TEXT(" \0"), // _Nil
         TEXT("1\0"), // _1
         TEXT("2\0"), // _2
         TEXT("3\0"), // _3
         TEXT("4\0"), // _4
         TEXT("5\0"), // _5
         TEXT("6\0"), // _6
         TEXT("7\0"), // _7
         TEXT("8\0"), // _8
         TEXT("9\0"), // _9
         TEXT("10" ), // _10
         TEXT("11" ), // _11
         TEXT("12" ), // _12
         TEXT("13" ), // _13
         TEXT("14" ), // _14
         TEXT("15" ), // _15
         TEXT("16" ), // _16
         TEXT("17" ), // _17
         TEXT("18" ), // _18
         TEXT("19" ), // _19
         TEXT("20" ), // _20
         TEXT("21" ), // _21
         TEXT("M\0")  // _Mine
      };
      return (LPCTSTR)szRes[eOpen];
   }
   static LPCTSTR GetCaptionClose(EClose eClose) {
      static TCHAR szRes[_Flag+1][2] =
      {  TEXT("?"), // _Unknown
         TEXT(" "), // _Clear
         TEXT("F")  // _Flag
      };
      return szRes[eClose];
   }

protected:
   COORD   m_Coord;
   RECT    m_Square;    //     -     /
   RECTEX  m_rcOuter;
   const   CGraphicContext &m_GContext;
   bool    m_bDown; // ?    m_Cell.m_Open! -    ,    .    - 

   COORD *const m_pNeighbor;  //   
   POINT *const m_pRegion;    //       
   const int    m_iDirection; //  - " " 
private:
   struct{
      EStatus m_Status;
      EOpen   m_Open;
      EClose  m_Close;
   } m_Cell;
   bool m_bLockMine; //        ?
   const int m_iNeighborNumber; //  
   const int m_iVertexNumber;   //    
private:
   CCell **m_ppLinkNeighbor;
protected:
   void VerifyNeighbor(const SIZE& sizeField);
private:
   void Lock();
public:
   void    Cell_SetDown(bool bDown);
   void    Cell_SetStatus(EStatus Status, CClickReportContext *pClickRepContext);
   EStatus Cell_GetStatus() const;
   void    Cell_DefineValue();
   bool    Cell_SetMine();
   EOpen   Cell_GetOpen() const;
   void    Cell_SetClose(EClose Close, CClickReportContext *pClickRepContext);
   EClose  Cell_GetClose() const;

  ~CCell();
   CCell(const COORD &Coord,
         const SIZE &sizeField,
         int iArea,
         const CGraphicContext &gContext,
         int iNeighborNumber,
         int iVertexNumber,
         int iDirection);
   void  LockNeighbor();
   void  SetNeighborLink(CCell *const *const ppLinkNeighbor, int numberNeighbor);
   int   GetNeighborNumber() const; //   
   const COORD &GetNeighborCoord(int index) const;
   COORD GetCoord() const;       // X  Y 
   POINT GetCenterPixel() const; //    ( )
   virtual bool PointInRegion(const POINT &point) const; //      
   const RECTEX& GetOuterRect() const { return m_rcOuter; } //       
   virtual void SetPoint(int iArea); //       
   void Reset();
   virtual void Paint() const;
   void             LButtonDown();
   CLeftUpReturn    LButtonUp  (bool isMy   , CClickReportContext *pClickRepContext);
   CRightDownReturn RButtonDown(EClose Close, CClickReportContext *pClickRepContext);

   void SetPresumeFlag(bool bValue) const {       m_bPresumeFlag = bValue;}
   bool GetPresumeFlag()            const {return m_bPresumeFlag;         }

protected:
   static double SQRT2  () { static double r = sqrt(2.);                      return r; }
   static double SQRT3  () { static double r = sqrt(3.);                      return r; }
   static double SQRT27 () { static double r = sqrt(27.);                     return r; }
   static double SQRT48 () { static double r = sqrt(48.);                     return r; }
   static double SQRT147() { static double r = sqrt(147.);                    return r; }
   static double PI     () { static double r = 3.14159265358979323;           return r; }
   static double SIN75  () { static double r = sin(PI()/180*75);              return r; }
   static double SIN15  () { static double r = sin(PI()/180*15);              return r; }
   static double TAN15  () { static double r = tan(PI()/180*15);              return r; }
   static double TAN45_2() { static double r = tan(PI()/180*45/2);            return r; }
   static double SIN135a() { static double r = sin(PI()/180*135-atan(8.f/3)); return r; }
};

#endif // __FILE__BASE__
