////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Hexagon1.cpp"
//
//   CHexagon1 -  6-  ()
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Hexagon1.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CHexagon1::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(2*iArea/SQRT27()); //   
   SIZE result = {toLONG((sizeField.cx      + 0.5f)*m_a()*SQRT3()),
                  toLONG((sizeField.cy*1.5f + 0.5f)*m_a())};
   return result;
}

int CHexagon1::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = sqrt(2*iArea/SQRT27()); //   
   m_sq() = 2*(m_a()*SQRT3()-iBorderWidth*2)/(1+SQRT3());
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CHexagon1::CHexagon1(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           6, 6,
           Coord.Y&1 // 0..1
          )
{
   SetPoint(iArea);
   //   
   m_pNeighbor[0].X = m_iDirection ? m_Coord.X   : m_Coord.X-1;
   m_pNeighbor[0].Y = m_Coord.Y-1;
   m_pNeighbor[1].X = m_iDirection ? m_Coord.X+1 : m_Coord.X;
   m_pNeighbor[1].Y = m_Coord.Y-1;
   m_pNeighbor[2].X = m_Coord.X-1;
   m_pNeighbor[2].Y = m_Coord.Y;
   m_pNeighbor[3].X = m_Coord.X+1;
   m_pNeighbor[3].Y = m_Coord.Y;
   m_pNeighbor[4].X = m_iDirection ? m_Coord.X   : m_Coord.X-1;
   m_pNeighbor[4].Y = m_Coord.Y+1;
   m_pNeighbor[5].X = m_iDirection ? m_Coord.X+1 : m_Coord.X;
   m_pNeighbor[5].Y = m_Coord.Y+1;
   VerifyNeighbor(sizeField);
}

void CHexagon1::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(2*iArea/SQRT27()); //   
      m_b() = m_a()*SQRT3();          //  
      m_h() = m_a()*2;                //  
      m_sq() = 2*(m_a()*SQRT3()-m_GContext.m_Border.m_iWidth*2)/(1+SQRT3()); //  ,   
   }

   //    
   float oX = (m_Coord.X+1)*m_b();      // offset X
   float oY = (m_Coord.Y+!m_iDirection)*m_a()*1.5f; // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX          ); m_pRegion[0].y = toLONG(oY - m_a()     );
      m_pRegion[1].x = toLONG(oX          ); m_pRegion[1].y = toLONG(oY             );
      m_pRegion[2].x = toLONG(oX - m_b()/2); m_pRegion[2].y = toLONG(oY + m_a()/2   );
      m_pRegion[3].x = toLONG(oX - m_b()  ); m_pRegion[3].y = toLONG(oY             );
      m_pRegion[4].x = toLONG(oX - m_b()  ); m_pRegion[4].y = toLONG(oY - m_a()     );
      m_pRegion[5].x = toLONG(oX - m_b()/2); m_pRegion[5].y = toLONG(oY - m_a()*1.5f);
      break;
   case 1:
      m_pRegion[0].x = toLONG(oX + m_b()/2); m_pRegion[0].y = toLONG(oY + m_a()/2   );
      m_pRegion[1].x = toLONG(oX + m_b()/2); m_pRegion[1].y = toLONG(oY + m_a()*1.5f);
      m_pRegion[2].x = toLONG(oX          ); m_pRegion[2].y = toLONG(oY + m_a()*2   );
      m_pRegion[3].x = toLONG(oX - m_b()/2); m_pRegion[3].y = toLONG(oY + m_a()*1.5f);
      m_pRegion[4].x = toLONG(oX - m_b()/2); m_pRegion[4].y = toLONG(oY + m_a()/2   );
      m_pRegion[5].x = toLONG(oX          ); m_pRegion[5].y = toLONG(oY             );
      break;
   }

   //       -     /
   {
      POINTFLOAT center; //   
      switch (m_iDirection) {
      case 0: center.x = oX - m_b()/2; center.y = oY - m_a()/2; break;
      case 1: center.x = oX;           center.y = oY + m_a();   break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CHexagon1::Paint() const {
   /**
   int minX = m_pRegion[4].x;
   int maxX = m_pRegion[0].x;
   int minY = m_pRegion[5].y;
   int maxY = m_pRegion[2].y;
   BitBlt(m_GContext.m_hDCTmp, minX, minY, maxX - minX, maxY - minY,
          hDCBck, minX, minY, SRCCOPY);
   /**/
   CCell::Paint();

   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[4].x, m_pRegion[4].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[5].x, m_pRegion[5].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 6, GetOuterRect().pointLT());
}
