////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "PentagonT10.cpp"
//
//   CPentagonT10 - 5- ,  10
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "PentagonT10.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CPentagonT10::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(iArea/7.f);
   SIZE result = {toLONG(2*m_a() +
                         5*m_a()*((sizeField.cx+1)/2) +
                         m_a()*((sizeField.cx+0)/2)),
                  toLONG(2*m_a() +
                         3*m_a()*((sizeField.cy+2)/3) +
                         3*m_a()*((sizeField.cy+1)/3) +
                         m_a()*((sizeField.cy+0)/3))};
   return result;
}

int CPentagonT10::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_sq() = 2*(sqrt(iArea/7.f)-iBorderWidth);
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CPentagonT10::CPentagonT10(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext, 7, 5,
           ((Coord.Y%6)<<1) + (Coord.X&1) // 0..11
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case  0:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y  ;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X  ;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+2;
      break;
   case  1:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-2;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X  ;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;  
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+1;
      break;
   case  2:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X+1;   m_pNeighbor[1].Y = m_Coord.Y  ;
      m_pNeighbor[2].X = m_Coord.X-1;   m_pNeighbor[2].Y = m_Coord.Y+1;
      m_pNeighbor[3].X = m_Coord.X  ;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X  ;   m_pNeighbor[5].Y = m_Coord.Y+2;  
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case  3:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X  ;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case  4:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X+1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X  ;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X+2;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case  5:
      m_pNeighbor[0].X = m_Coord.X+1;   m_pNeighbor[0].Y = m_Coord.Y-2;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X  ;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case  6:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-2;
      m_pNeighbor[1].X = m_Coord.X-2;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X  ;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+1;
      break;
   case  7:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y  ;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X  ;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+2;
      break;
   case  8:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X  ;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case  9:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y  ;
      m_pNeighbor[2].X = m_Coord.X-2;   m_pNeighbor[2].Y = m_Coord.Y+1;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X  ;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X  ;   m_pNeighbor[5].Y = m_Coord.Y+2;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case 10:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X+1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y+1;
      m_pNeighbor[4].X = m_Coord.X  ;   m_pNeighbor[4].Y = m_Coord.Y+1;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   case 11:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-2;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+2;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X  ;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6] = INCORRECT_COORD;
      break;
   }
   VerifyNeighbor(sizeField);
}

void CPentagonT10::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(iArea/7.f);
      m_sq() = 2*(m_a()-m_GContext.m_Border.m_iWidth);
   }

   //    
   float oX; // offset X
   float oY;  // offset Y
   switch (m_iDirection) {
   case 0: case 6: case  8: case 9: case 10:          oX = m_a()*2+m_a()*6*((m_Coord.X+0)/2); break;
   case 1: case 2: case  3: case 4: case 5: case 7:   oX = m_a()*5+m_a()*6*((m_Coord.X+0)/2); break;
   case 11:                                           oX = m_a()*2+m_a()*6*((m_Coord.X+1)/2); break;
   }
   switch (m_iDirection) {
   case 0:                                            oY = m_a()*5 +m_a()*14*(m_Coord.Y/6);   break;
   case 1:                                            oY =          m_a()*14*(m_Coord.Y/6);   break;
   case 2: case 3: case  4: case 5:                   oY = m_a()*6 +m_a()*14*(m_Coord.Y/6);   break;
   case 6:                                            oY = m_a()*7 +m_a()*14*(m_Coord.Y/6);   break;
   case 7:                                            oY = m_a()*12+m_a()*14*(m_Coord.Y/6);   break;
   case 8: case 9: case 10: case 11:                  oY = m_a()*13+m_a()*14*(m_Coord.Y/6);   break;
   }
   switch (m_iDirection) {
   case 0: case 3: case 7: case 8:
      m_pRegion[0].x = toLONG(oX          ); m_pRegion[0].y = toLONG(oY          );
      m_pRegion[1].x = toLONG(oX - m_a()*2); m_pRegion[1].y = toLONG(oY - m_a()*2);
      m_pRegion[2].x = toLONG(oX - m_a()  ); m_pRegion[2].y = toLONG(oY - m_a()*3);
      m_pRegion[3].x = toLONG(oX + m_a()  ); m_pRegion[3].y = toLONG(oY - m_a()*3);
      m_pRegion[4].x = toLONG(oX + m_a()*2); m_pRegion[4].y = toLONG(oY - m_a()*2);
      break;
   case 1: case 4: case 6: case 10:
      m_pRegion[0].x = toLONG(oX          ); m_pRegion[0].y = toLONG(oY          );
      m_pRegion[1].x = toLONG(oX + m_a()*2); m_pRegion[1].y = toLONG(oY + m_a()*2);
      m_pRegion[2].x = toLONG(oX + m_a()  ); m_pRegion[2].y = toLONG(oY + m_a()*3);
      m_pRegion[3].x = toLONG(oX - m_a()  ); m_pRegion[3].y = toLONG(oY + m_a()*3);
      m_pRegion[4].x = toLONG(oX - m_a()*2); m_pRegion[4].y = toLONG(oY + m_a()*2);
      break;
   case 2: case 11:
      m_pRegion[0].x = toLONG(oX - m_a()*2); m_pRegion[0].y = toLONG(oY + m_a()*2);
      m_pRegion[1].x = toLONG(oX - m_a()*3); m_pRegion[1].y = toLONG(oY + m_a()  );
      m_pRegion[2].x = toLONG(oX - m_a()*3); m_pRegion[2].y = toLONG(oY - m_a()  );
      m_pRegion[3].x = toLONG(oX - m_a()*2); m_pRegion[3].y = toLONG(oY - m_a()*2);
      m_pRegion[4].x = toLONG(oX          ); m_pRegion[4].y = toLONG(oY          );
      break;
   case 5: case 9:
      m_pRegion[0].x = toLONG(oX + m_a()*2); m_pRegion[0].y = toLONG(oY - m_a()*2);
      m_pRegion[1].x = toLONG(oX + m_a()*3); m_pRegion[1].y = toLONG(oY - m_a()  );
      m_pRegion[2].x = toLONG(oX + m_a()*3); m_pRegion[2].y = toLONG(oY + m_a()  );
      m_pRegion[3].x = toLONG(oX + m_a()*2); m_pRegion[3].y = toLONG(oY + m_a()*2);
      m_pRegion[4].x = toLONG(oX          ); m_pRegion[4].y = toLONG(oY          );
      break;
   }

   //       -     /
   {
      POINTFLOAT center;
      switch (m_iDirection) {
      case 0: case 3: case 7: case 8:  center.x = oX;           center.y = oY - m_a()*2; break;
      case 1: case 4: case 6: case 10: center.x = oX;           center.y = oY + m_a()*2; break;
      case 2: case 11:                 center.x = oX - m_a()*2; center.y = oY;           break;
      case 5: case 9:                  center.x = oX + m_a()*2; center.y = oY;           break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CPentagonT10::Paint() const {
   int minX = min(m_pRegion[1].x, m_pRegion[4].x);
   int maxX = max(m_pRegion[1].x, m_pRegion[4].x);
   int minY = min(m_pRegion[0].y, m_pRegion[3].y);
   int maxY = max(m_pRegion[0].y, m_pRegion[3].y);
   BitBlt(m_GContext.m_hDCTmp, minX, minY, maxX - minX, maxY - minY,
          m_GContext.m_hDCBck, minX, minY, SRCCOPY);

   CCell::Paint();

   switch (m_iDirection) {
   case 0: case 3: case 7: case 8:
   case 2: case 11:                 SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 1: case 4: case 6: case 10:
   case 5: case 9:                  SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
   switch (m_iDirection) {
   case 0: case 3: case 7: case 8:
   case 2: case 11:                 SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   case 1: case 4: case 6: case 10:
   case 5: case 9:                  SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[4].x, m_pRegion[4].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 5, GetOuterRect().pointLT());
}
