////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Triangle3.cpp"
//
//   CTriangle3 -  45-90-45
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Triangle3.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CTriangle3::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = 2*sqrt((float)iArea);
   SIZE result = {toLONG(m_a()*((sizeField.cx+1)>>1)),
                  toLONG(m_a()*((sizeField.cy+1)>>1))};
   return result;
}

int CTriangle3::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = 2*sqrt((float)iArea);
   m_sq() = (m_a()-iBorderWidth*2/TAN45_2())/3;
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CTriangle3::CTriangle3(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           14, 3,
           ((Coord.Y&1)<<1)+(Coord.X&1) // 0..3
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case  0:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X+3;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X-2;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X+2;   m_pNeighbor[10].Y = m_Coord.Y  ;
      m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X+1;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X+2;   m_pNeighbor[13].Y = m_Coord.Y+1;
      break;
   case  1:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
      m_pNeighbor[10].X = m_Coord.X-1;   m_pNeighbor[10].Y = m_Coord.Y+2;
      m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+2;
      m_pNeighbor[12].X = m_Coord.X+1;   m_pNeighbor[12].Y = m_Coord.Y+2;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y+3;
      break;
   case  2:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
      m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X-2;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-2;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-1;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X  ;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X  ;   m_pNeighbor[13].Y = m_Coord.Y+2;
      break;
   case  3:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X-2;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-3;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8].X = m_Coord.X-2;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
      m_pNeighbor[ 9].X = m_Coord.X-1;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
      m_pNeighbor[10].X = m_Coord.X  ;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X+1;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y+2;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y+2;
      break;
   }
   VerifyNeighbor(sizeField);
}

void CTriangle3::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt((float)iArea)*2;
      m_b() = m_a()/2;
      m_sq() = (m_a()-m_GContext.m_Border.m_iWidth*2/TAN45_2())/3;
   }

   //    
   float oX = m_a()*(m_Coord.X>>1); // offset X
   float oY = m_a()*(m_Coord.Y>>1); // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX + m_a()); m_pRegion[0].y = toLONG(oY        );
      m_pRegion[1].x = toLONG(oX        ); m_pRegion[1].y = toLONG(oY        );
      m_pRegion[2].x = toLONG(oX + m_b()); m_pRegion[2].y = toLONG(oY + m_b());
      break;
   case 1:
      m_pRegion[0].x = toLONG(oX + m_a()); m_pRegion[0].y = toLONG(oY        );
      m_pRegion[1].x = toLONG(oX + m_b()); m_pRegion[1].y = toLONG(oY + m_b());
      m_pRegion[2].x = toLONG(oX + m_a()); m_pRegion[2].y = toLONG(oY + m_a());
      break;
   case 2:
      m_pRegion[0].x = toLONG(oX        ); m_pRegion[0].y = toLONG(oY + m_a());
      m_pRegion[1].x = toLONG(oX + m_b()); m_pRegion[1].y = toLONG(oY + m_b());
      m_pRegion[2].x = toLONG(oX        ); m_pRegion[2].y = toLONG(oY        );
      break;
   case 3:
      m_pRegion[0].x = toLONG(oX        ); m_pRegion[0].y = toLONG(oY + m_a());
      m_pRegion[1].x = toLONG(oX + m_a()); m_pRegion[1].y = toLONG(oY + m_a());
      m_pRegion[2].x = toLONG(oX + m_b()); m_pRegion[2].y = toLONG(oY + m_b());
      break;
   }

   //       -     /
   {
      POINTFLOAT center; //   
      switch (m_iDirection) {
      case 0:  center.x = m_pRegion[2].x;
               center.y = m_pRegion[0].y + m_a()/6; break;
      case 1:  center.x = m_pRegion[0].x - m_a()/6;
               center.y = m_pRegion[1].y;           break;
      case 2:  center.x = m_pRegion[2].x + m_a()/6;
               center.y = m_pRegion[1].y;           break;
      case 3:  center.x = m_pRegion[2].x;
               center.y = m_pRegion[1].y - m_a()/6; break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CTriangle3::Paint() const {
   CCell::Paint();

   switch (m_iDirection) {
   case 0: case 1:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 2: case 3:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   switch (m_iDirection) {
   case 0: case 1:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   case 2: case 3:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 3, GetOuterRect().pointLT());
}
