////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Triangle4.cpp"
//
//   CTriangle4 -  30-30-120
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Triangle4.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CTriangle4::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(iArea*SQRT48());
   m_b() = m_a()/2;
   m_R() = m_a()/SQRT3();
   m_r() = m_R()/2;

   SIZE result = {toLONG( m_b()+m_b() *((sizeField.cx+2)/3)+
                                m_b() *((sizeField.cx+0)/3)),
                  toLONG((m_R()+m_r())*((sizeField.cy+1)/2))};
   return result;
}

int CTriangle4::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = sqrt(iArea*SQRT48());
   m_sq() = (m_a()-iBorderWidth*2/TAN15())/(SQRT3()+3);
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CTriangle4::CTriangle4(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           21, 3,
           (Coord.Y&3)*3+(Coord.X%3) // 0..11
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case  0:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-3;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X  ;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+1;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X-3;   m_pNeighbor[15].Y = m_Coord.Y+2;
      m_pNeighbor[16].X = m_Coord.X-2;   m_pNeighbor[16].Y = m_Coord.Y+2;
      m_pNeighbor[17].X = m_Coord.X-1;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X  ;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X-2;   m_pNeighbor[19].Y = m_Coord.Y+3;
      m_pNeighbor[20].X = m_Coord.X  ;   m_pNeighbor[20].Y = m_Coord.Y+3;
      break;
   case  1:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-2;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-1;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X  ;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X-1;   m_pNeighbor[15].Y = m_Coord.Y+2;
      m_pNeighbor[16].X = m_Coord.X  ;   m_pNeighbor[16].Y = m_Coord.Y+2;
      m_pNeighbor[17].X = m_Coord.X+1;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X+2;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X  ;   m_pNeighbor[19].Y = m_Coord.Y+3;
      m_pNeighbor[20].X = m_Coord.X+2;   m_pNeighbor[20].Y = m_Coord.Y+3;
      break;
   case  2:
      m_pNeighbor[ 0].X = m_Coord.X-3;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X-3;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+3;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-3;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-2;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X-1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+1;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X+2;   m_pNeighbor[15].Y = m_Coord.Y  ;
      m_pNeighbor[16].X = m_Coord.X+3;   m_pNeighbor[16].Y = m_Coord.Y  ;
      m_pNeighbor[17].X = m_Coord.X-3;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X-1;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X  ;   m_pNeighbor[19].Y = m_Coord.Y+1;
      m_pNeighbor[20].X = m_Coord.X+2;   m_pNeighbor[20].Y = m_Coord.Y+1;
      break;
   case  3:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X+3;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X-3;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+2;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+3;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-3;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X  ;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+1;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X+2;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X+3;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X-2;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X  ;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X+1;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X+3;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   case  4:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-3;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-2;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y-2;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+2;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X-1;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X  ;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X+1;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X+2;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X  ;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X+2;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   case  5:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
      m_pNeighbor[ 1].X = m_Coord.X+2;   m_pNeighbor[ 1].Y = m_Coord.Y-3;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X+2;   m_pNeighbor[ 4].Y = m_Coord.Y-2;
      m_pNeighbor[ 5].X = m_Coord.X+3;   m_pNeighbor[ 5].Y = m_Coord.Y-2;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+3;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X-2;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X-1;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X  ;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X+1;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X-1;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X+1;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   case  6:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X+3;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X-3;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+3;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-3;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-2;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X-1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+1;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X+2;   m_pNeighbor[15].Y = m_Coord.Y  ;
      m_pNeighbor[16].X = m_Coord.X+3;   m_pNeighbor[16].Y = m_Coord.Y  ;
      m_pNeighbor[17].X = m_Coord.X-2;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X  ;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X+1;   m_pNeighbor[19].Y = m_Coord.Y+1;
      m_pNeighbor[20].X = m_Coord.X+3;   m_pNeighbor[20].Y = m_Coord.Y+1;
      break;
   case  7:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X+2;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-2;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-1;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X  ;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X-2;   m_pNeighbor[15].Y = m_Coord.Y+2;
      m_pNeighbor[16].X = m_Coord.X-1;   m_pNeighbor[16].Y = m_Coord.Y+2;
      m_pNeighbor[17].X = m_Coord.X  ;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X+1;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X-2;   m_pNeighbor[19].Y = m_Coord.Y+3;
      m_pNeighbor[20].X = m_Coord.X  ;   m_pNeighbor[20].Y = m_Coord.Y+3;
      break;
   case  8:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-1;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X+1;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X+2;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+3;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X  ;   m_pNeighbor[15].Y = m_Coord.Y+2;
      m_pNeighbor[16].X = m_Coord.X+1;   m_pNeighbor[16].Y = m_Coord.Y+2;
      m_pNeighbor[17].X = m_Coord.X+2;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X+3;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X  ;   m_pNeighbor[19].Y = m_Coord.Y+3;
      m_pNeighbor[20].X = m_Coord.X+2;   m_pNeighbor[20].Y = m_Coord.Y+3;
      break;
   case  9:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-3;
      m_pNeighbor[ 2].X = m_Coord.X-3;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X-2;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y-2;
      m_pNeighbor[ 5].X = m_Coord.X  ;   m_pNeighbor[ 5].Y = m_Coord.Y-2;
      m_pNeighbor[ 6].X = m_Coord.X-3;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X-2;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X  ;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+1;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X-1;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X  ;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X+1;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X+2;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X-1;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X+1;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   case 10:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
      m_pNeighbor[ 1].X = m_Coord.X+2;   m_pNeighbor[ 1].Y = m_Coord.Y-3;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-2;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-2;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-2;
      m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y-1;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y-1;
      m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y-1;
      m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y-1;
      m_pNeighbor[10].X = m_Coord.X+2;   m_pNeighbor[10].Y = m_Coord.Y-1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y  ;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y  ;
      m_pNeighbor[13].X = m_Coord.X+1;   m_pNeighbor[13].Y = m_Coord.Y  ;
      m_pNeighbor[14].X = m_Coord.X+2;   m_pNeighbor[14].Y = m_Coord.Y  ;
      m_pNeighbor[15].X = m_Coord.X-2;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X-1;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X  ;   m_pNeighbor[17].Y = m_Coord.Y+1;
      m_pNeighbor[18].X = m_Coord.X+1;   m_pNeighbor[18].Y = m_Coord.Y+1;
      m_pNeighbor[19].X = m_Coord.X-2;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X  ;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   case 11:
      m_pNeighbor[ 0].X = m_Coord.X-3;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X-3;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8].X = m_Coord.X+2;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
      m_pNeighbor[ 9].X = m_Coord.X+3;   m_pNeighbor[ 9].Y = m_Coord.Y  ;
      m_pNeighbor[10].X = m_Coord.X-3;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-2;   m_pNeighbor[11].Y = m_Coord.Y+1;
      m_pNeighbor[12].X = m_Coord.X-1;   m_pNeighbor[12].Y = m_Coord.Y+1;
      m_pNeighbor[13].X = m_Coord.X  ;   m_pNeighbor[13].Y = m_Coord.Y+1;
      m_pNeighbor[14].X = m_Coord.X+1;   m_pNeighbor[14].Y = m_Coord.Y+1;
      m_pNeighbor[15].X = m_Coord.X+2;   m_pNeighbor[15].Y = m_Coord.Y+1;
      m_pNeighbor[16].X = m_Coord.X+3;   m_pNeighbor[16].Y = m_Coord.Y+1;
      m_pNeighbor[17].X = m_Coord.X-3;   m_pNeighbor[17].Y = m_Coord.Y+2;
      m_pNeighbor[18].X = m_Coord.X-1;   m_pNeighbor[18].Y = m_Coord.Y+2;
      m_pNeighbor[19].X = m_Coord.X  ;   m_pNeighbor[19].Y = m_Coord.Y+2;
      m_pNeighbor[20].X = m_Coord.X+2;   m_pNeighbor[20].Y = m_Coord.Y+2;
      break;
   }
   VerifyNeighbor(sizeField);
}

void CTriangle4::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(iArea*SQRT48());
      m_b() = m_a()/2;
      m_R() = m_a()/SQRT3();
      m_r() = m_R()/2;
      m_sq() = ((m_a()-m_GContext.m_Border.m_iWidth*2/TAN15())/(SQRT3()+3));
   }

   //    
   float oX =  (m_Coord.X/3)*m_a() + m_b();      // offset X
   float oY = ((m_Coord.Y/4)*2+1)*(m_R()+m_r()); // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX - m_b());   m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX        );   m_pRegion[1].y = toLONG(oY - m_r()      );
      m_pRegion[2].x = toLONG(oX        );   m_pRegion[2].y = toLONG(oY - m_R()-m_r());
      break;
   case 1:
      m_pRegion[0].x = toLONG(oX + m_b());   m_pRegion[0].y = toLONG(oY - m_R()      );
      m_pRegion[1].x = toLONG(oX        );   m_pRegion[1].y = toLONG(oY - m_R()-m_r());
      m_pRegion[2].x = toLONG(oX + m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 2:
      m_pRegion[0].x = toLONG(oX        );   m_pRegion[0].y = toLONG(oY - m_R()-m_r());
      m_pRegion[1].x = toLONG(oX + m_b());   m_pRegion[1].y = toLONG(oY - m_R()      );
      m_pRegion[2].x = toLONG(oX + m_a());   m_pRegion[2].y = toLONG(oY - m_R()-m_r());
      break;
   case 3:
      m_pRegion[0].x = toLONG(oX + m_b());   m_pRegion[0].y = toLONG(oY              );   
      m_pRegion[1].x = toLONG(oX        );   m_pRegion[1].y = toLONG(oY - m_r()      );
      m_pRegion[2].x = toLONG(oX - m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 4:
      m_pRegion[0].x = toLONG(oX        );   m_pRegion[0].y = toLONG(oY - m_r()      );
      m_pRegion[1].x = toLONG(oX + m_b());   m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX        );   m_pRegion[2].y = toLONG(oY - m_R()-m_r());
      break;
   case 5:
      m_pRegion[0].x = toLONG(oX + m_a());   m_pRegion[0].y = toLONG(oY - m_R()-m_r());
      m_pRegion[1].x = toLONG(oX + m_b());   m_pRegion[1].y = toLONG(oY - m_R()      );
      m_pRegion[2].x = toLONG(oX + m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 6:
      m_pRegion[0].x = toLONG(oX - m_b());   m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX        );   m_pRegion[1].y = toLONG(oY + m_r()      );
      m_pRegion[2].x = toLONG(oX + m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 7:
      m_pRegion[0].x = toLONG(oX + m_b());   m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX        );   m_pRegion[1].y = toLONG(oY + m_r()      );
      m_pRegion[2].x = toLONG(oX        );   m_pRegion[2].y = toLONG(oY + m_R()+m_r());
      break;
   case 8:
      m_pRegion[0].x = toLONG(oX + m_b());   m_pRegion[0].y = toLONG(oY + m_R()      );
      m_pRegion[1].x = toLONG(oX + m_a());   m_pRegion[1].y = toLONG(oY + m_R()+m_r());
      m_pRegion[2].x = toLONG(oX + m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 9:
      m_pRegion[0].x = toLONG(oX        );   m_pRegion[0].y = toLONG(oY + m_r()      );
      m_pRegion[1].x = toLONG(oX - m_b());   m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX        );   m_pRegion[2].y = toLONG(oY + m_R()+m_r());
      break;
   case 10:
      m_pRegion[0].x = toLONG(oX        );   m_pRegion[0].y = toLONG(oY + m_R()+m_r());
      m_pRegion[1].x = toLONG(oX + m_b());   m_pRegion[1].y = toLONG(oY + m_R()      );
      m_pRegion[2].x = toLONG(oX + m_b());   m_pRegion[2].y = toLONG(oY              );
      break;
   case 11:
      m_pRegion[0].x = toLONG(oX + m_a());   m_pRegion[0].y = toLONG(oY + m_R()+m_r());
      m_pRegion[1].x = toLONG(oX + m_b());   m_pRegion[1].y = toLONG(oY + m_R()      );
      m_pRegion[2].x = toLONG(oX        );   m_pRegion[2].y = toLONG(oY + m_R()+m_r());
      break;
   }

   //       -     /
   {
      POINTFLOAT center; //   
      float sq2   = m_sq()/2;
      float sq2w  = m_sq()/2+m_GContext.m_Border.m_iWidth;
      float sq2w3 = m_sq()/2+m_GContext.m_Border.m_iWidth/SQRT3();
      switch (m_iDirection) {
      case 0: case 10:
         center.x = m_pRegion[1].x - sq2w;
         center.y = m_pRegion[1].y - sq2w3;
         break;
      case 1: case 9:
         center.x = m_pRegion[0].x - sq2w;
         center.y = m_pRegion[0].y + sq2w3;
         break;
      case 2: case 6:
         center.x = m_pRegion[1].x;
         center.y = m_pRegion[2].y + sq2w;
         break;
      case 3: case 11:
         center.x = m_pRegion[1].x;
         center.y = m_pRegion[0].y - sq2w;
         break;
      case 4: case 8:
         center.x = m_pRegion[0].x + sq2w;
         center.y = m_pRegion[0].y - sq2w3;
         break;
      case 5: case 7:
         center.x = m_pRegion[1].x + sq2w;
         center.y = m_pRegion[1].y + sq2w3;
         break;
      }
      m_Square.left   = toLONG(center.x - sq2);
      m_Square.top    = toLONG(center.y - sq2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }

   CCell::SetPoint(iArea);
}

void CTriangle4::Paint() const {
   CCell::Paint();

   switch (m_iDirection) {
   case 0: case 2: case 4: case 6: case 8: case 10:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   case 1: case 3: case 5: case 7: case 9: case 11:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   }
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   switch (m_iDirection) {
   case 0: case 2: case 4: case 6: case 8: case 10:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 1: case 3: case 5: case 7: case 9: case 11:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 3, GetOuterRect().pointLT());
}
