////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "MenuItemEx.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "MenuItemEx.h"

//#pragma comment(lib, "MsImg32") // GradientFill

#define ID_TIMER_REDRAW 123

BOOL CMenuItemEx::DrawBkgrnd(const DRAWITEMSTRUCT &dis, HDC hCDC, const RECTEX &rc, bool isSelect, bool isGrayed, bool isSeparator) const {
   // background
   HBRUSH  hBrushNew = ::CreateSolidBrush(0xCEB7A2);
   HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
   if (!!m_uWidthCheck) {
      ::PatBlt(hCDC, rc.left, rc.top, rc.left+m_uWidthCheck, rc.height(), PATCOPY);
   }

   if (isSelect && !isSeparator) {
      COLORREF clrTop    = ::GetSysColor(COLOR_HIGHLIGHT);
      COLORREF clrBottom = GetMenuColor();
      RECTEX rcFill = rc;
      rcFill.left += m_uWidthCheck;
      TRIVERTEX vert[] = {
         {
            rcFill.left,          // LONG    x;
            rcFill.top,           // LONG    y;
            GetRValue(clrTop)<<8, // COLOR16 Red;   0x0000..0xff00
            GetGValue(clrTop)<<8, // COLOR16 Green;
            GetBValue(clrTop)<<8, // COLOR16 Blue;
            0x0000                // COLOR16 Alpha;
         }, {
            rcFill.right,            // LONG    x;
            rcFill.bottom,           // LONG    y;   
            GetRValue(clrBottom)<<8, // COLOR16 Red;   0x0000..0xff00
            GetGValue(clrBottom)<<8, // COLOR16 Green;
            GetBValue(clrBottom)<<8, // COLOR16 Blue;
            0x0000                   // COLOR16 Alpha;
         }
      };
      GRADIENT_RECT gRc[] = {{0, 1}};
      ::GradientFill(hCDC, vert, chDIMOF(vert), &gRc, 1, GRADIENT_FILL_RECT_V);
   } else {
      ::SelectObject(hCDC, hBrushOld);
      ::DeleteObject(hBrushNew);

      hBrushNew = ::CreateSolidBrush(GetMenuColor());
      ::SelectObject(hCDC, hBrushNew);

      ::PatBlt(hCDC, rc.left+m_uWidthCheck, rc.top, rc.width()-m_uWidthCheck, rc.height(), PATCOPY);
   }

   // frame
   if ((isSelect || (dis.itemState & ODS_HOTLIGHT)) && !isSeparator) {
      HPEN    hPenNew = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_HIGHLIGHT));
      HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);

      ::MoveToEx(hCDC, rc.left+m_uWidthCheck, rc.top, NULL);
      ::LineTo  (hCDC, rc.right-1, rc.top);
      ::LineTo  (hCDC, rc.right-1, rc.bottom-1);
      ::LineTo  (hCDC, rc.left+m_uWidthCheck, rc.bottom-1);
      ::LineTo  (hCDC, rc.left+m_uWidthCheck, rc.top);

      ::SelectObject(hCDC, hPenOld);
      ::DeleteObject(hPenNew);
   }

   ::SelectObject(hCDC, hBrushOld);
   ::DeleteObject(hBrushNew);

   return TRUE;
}

/* static */ VOID CALLBACK CMenuItemIridescence::TimerProc(
   HWND hWnd,     // handle of window for timer messages
   UINT uMsg,     // WM_TIMER message
   UINT idEvent,  // timer identifier
   DWORD dwTime   // current system time
) {
   CMenuItemIridescence *This = (CMenuItemIridescence*)::GetWindowUserData(hWnd);
   if (This && CMenuItem::CheckSign(This)) {
      ::KillTimer(NULL, idEvent);
      This->OnTimer(hWnd, idEvent, dwTime);
   }
}
void CMenuItemIridescence::OnTimer(HWND hWnd, UINT idEvent, DWORD dwTime) {
   if (idEvent == ID_TIMER_REDRAW) {
      DWORD dwTC = dwTime;//::GetTickCount();
      int i = (dwTC>>0)%4;//0;//
      int j = (dwTC>>2)%3;//rand(2);//
      BYTE &bC = m_Color[i][j];
      bC = ((bC & 1) ? 0x00 : 0x80) | (bC>>1);
      ::InvalidateRect(hWnd, &m_rcItemGradient, FALSE);
      ::KillTimer(hWnd, idEvent);
   }
}

BOOL CMenuItemIridescence::DrawBkgrnd(const DRAWITEMSTRUCT &dis, HDC hCDC, const RECTEX &rc, bool isSelect, bool isGrayed, bool isSeparator) const {
   // background
   HBRUSH  hBrushNew = ::CreateSolidBrush(0xCEB7A2);
   HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
   if (!!m_uWidthCheck) {
      ::PatBlt(hCDC, rc.left, rc.top, rc.left+m_uWidthCheck, rc.height(), PATCOPY);
   }

   if (isSelect && !isSeparator) {
      //COLORREF clr = ::GetSysColor(COLOR_HIGHLIGHT);
      m_rcItemGradient = rc;
      m_rcItemGradient.move(dis.rcItem.left, dis.rcItem.top);
      RECTEX rcFill = rc;
      rcFill.left += m_uWidthCheck;
      TRIVERTEX vert[] = {
         {
            rcFill.left,       // LONG    x;
            rcFill.top,        // LONG    y;
            m_Color[0][0]<<8,  // COLOR16 Red;   0x0000..0xff00
            m_Color[0][1]<<8,  // COLOR16 Green;
            m_Color[0][2]<<8,  // COLOR16 Blue;
            0x0000             // COLOR16 Alpha;
         }, {
            rcFill.left,       // LONG    x;
            rcFill.bottom,     // LONG    y;   
            m_Color[1][0]<<8,  // COLOR16 Red;   0x0000..0xff00
            m_Color[1][1]<<8,  // COLOR16 Green;
            m_Color[1][2]<<8,  // COLOR16 Blue;
            0x0000             // COLOR16 Alpha;
         }, {
            rcFill.right,      // LONG    x;
            rcFill.bottom,     // LONG    y;   
            m_Color[2][0]<<8,  // COLOR16 Red;   0x0000..0xff00
            m_Color[2][1]<<8,  // COLOR16 Green;
            m_Color[2][2]<<8,  // COLOR16 Blue;
            0x0000             // COLOR16 Alpha;
         }, {
            rcFill.right,      // LONG    x;
            rcFill.top,        // LONG    y;   
            m_Color[3][0]<<8,  // COLOR16 Red;   0x0000..0xff00
            m_Color[3][1]<<8,  // COLOR16 Green;
            m_Color[3][2]<<8,  // COLOR16 Blue;
            0x0000             // COLOR16 Alpha;
         }
      };
      GRADIENT_TRIANGLE gTri1[] = {{1, 0, 3}, {3, 2, 1}};
      GRADIENT_TRIANGLE gTri2[] = {{0, 1, 2}, {0, 3, 2}};
      ::GradientFill(hCDC, vert, chDIMOF(vert), (m_uItem&1) ? &gTri1 : &gTri2, 2, GRADIENT_FILL_TRIANGLE);
   } else {
      ::SelectObject(hCDC, hBrushOld);
      ::DeleteObject(hBrushNew);

      hBrushNew = ::CreateSolidBrush(GetMenuColor());
      ::SelectObject(hCDC, hBrushNew);

      ::PatBlt(hCDC, rc.left+m_uWidthCheck, rc.top, rc.width()-m_uWidthCheck, rc.height(), PATCOPY);
   }

   // frame
   if ((isSelect || (dis.itemState & ODS_HOTLIGHT)) && !isSeparator) {
      HPEN    hPenNew = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_HIGHLIGHT));
      HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);

      ::MoveToEx(hCDC, rc.left+m_uWidthCheck, rc.top, NULL);
      ::LineTo  (hCDC, rc.right-1, rc.top);
      ::LineTo  (hCDC, rc.right-1, rc.bottom-1);
      ::LineTo  (hCDC, rc.left+m_uWidthCheck, rc.bottom-1);
      ::LineTo  (hCDC, rc.left+m_uWidthCheck, rc.top);

      ::SelectObject(hCDC, hPenOld);
      ::DeleteObject(hPenNew);
   }

   ::SelectObject(hCDC, hBrushOld);
   ::DeleteObject(hBrushNew);

   return TRUE;
}

void CMenuItemIridescence::PostDraw(const DRAWITEMSTRUCT &dis, HDC hCDC, const RECTEX &rc, bool isSelect, bool isGrayed, bool isSeparator) const {
   if (isSelect && !isSeparator) {
      HWND hWnd = ::WindowFromDC(dis.hDC);
      ::SetWindowUserData(hWnd, (LONG_PTR)this);
      ::SetTimer(hWnd, ID_TIMER_REDRAW, 0, CMenuItemIridescence::TimerProc);
   }
}
