////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "TabItem.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TabItem.h"
#include <TmSchema.h>

CTabItem::CTabItem() : m_dwSignature(CTabItem::GetSignature()) {}

CTabItem::CTabItem(HWND hWndTab, int iItem, const CImage *pImg) :
   m_dwSignature(CTabItem::GetSignature()),
   m_hWndTab(NULL),
   m_iItem(0)
{
   Attach(hWndTab, iItem, pImg);
}

/* static */ BOOL CTabItem::CheckSign(CTabItem *This) {
   if (This==NULL) return FALSE;
#ifdef _DEBUG
   BOOL bRes = (This->m_dwSignature == CTabItem::GetSignature());
   if (!bRes) alert(_T("Invalid CTabItem signature!"));
   return bRes;
#else
   return (This->m_dwSignature == CTabItem::GetSignature());
#endif
}

/* static */ CTabItem* CTabItem::GetInstance(HWND hWndTab, int iItem) {
   CTabItem *This = NULL;
   TCITEM item = {TCIF_PARAM, };
   BOOL bRes = TabCtrl_GetItem(hWndTab, iItem, &item);
   if (bRes) {
      This = (CTabItem*)item.lParam;
      bRes = CTabItem::CheckSign(This);
      if (!bRes) This = NULL;
   }
   return This;
}

BOOL CTabItem::Attach(HWND hWndTab, int iItem, const CImage *pImg) {
   m_hWndTab = hWndTab;
   m_iItem = iItem;
   m_Img.Reset();
   if (pImg) {
      m_Img = *pImg;
      m_Img.SetPlace(CImg::ePlaceCenter | CImg::ePlaceMiddle | CImg::ePlaceAsIs/*ePlaceUniformly*/);
   }

   TCITEM item;
   item.mask = TCIF_PARAM;
   item.lParam = (LPARAM)this;
   return TabCtrl_InsertItem(m_hWndTab, m_iItem, &item);
}
BOOL CTabItem::Detach() {
   BOOL bRes = FALSE;
   if (::IsWindow(m_hWndTab)) {
      TCITEM item;
      item.mask = TCIF_PARAM;
      item.lParam = 0;
      bRes = TabCtrl_InsertItem(m_hWndTab, m_iItem, &item);
   }
   return bRes;
}

// WM_MEASUREITEM
/* static */ void CTabItem::OnMeasureItem(HWND hWndTab, MEASUREITEMSTRUCT *lpMeasureItem) {
   if (!lpMeasureItem) return;
   CTabItem *This = CTabItem::GetInstance(hWndTab, lpMeasureItem->itemID);
   if (CTabItem::CheckSign(This))
      This->OnMeasureItem(*lpMeasureItem);
}
void CTabItem::OnMeasureItem(MEASUREITEMSTRUCT &mi) {
   if (mi.CtlType != ODT_TAB) return; // error
#ifdef _DEBUG
         alert(_T("CDialogStatisticsOrChampions::OnMeasureItem"));
#endif
}

// WM_DRAWITEM
/* static */ void CTabItem::OnDrawItem(HWND hWndTab, const DRAWITEMSTRUCT *lpDrawItem) {
   if (!lpDrawItem) return;
   CTabItem *This = CTabItem::GetInstance(hWndTab, lpDrawItem->itemID);
   if (CTabItem::CheckSign(This))
      This->OnDrawItem(*lpDrawItem);
}
void CTabItem::OnDrawItem(const DRAWITEMSTRUCT &di) {
   if (di.CtlType == ODT_TAB) {
      //m_Img.Draw(di.hDC, di.rcItem);

      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  Tab::OnDrawItem: di.itemID=%i  di.itemAction=[%s] di.itemState=[%s] "), di.itemID, (LPCTSTR)CLogger::OwnerDrawItemAction(di.itemAction), (LPCTSTR)CLogger::OwnerDrawItemState(di.itemState));
      /**/
      RECTEX rcItem(di.rcItem);
      TCITEM item; memset(&item, 0, sizeof(item)); item.mask = TCIF_STATE;
      TabCtrl_GetItem(m_hWndTab, di.itemID, &item);
      /** /
      CString strTabItemState;
      if (item.dwState & TCIS_BUTTONPRESSED) strTabItemState = _T("TCIS_BUTTONPRESSED");
      if (item.dwState & TCIS_HIGHLIGHTED  ) strTabItemState = _T("TCIS_HIGHLIGHTED"  );
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  Tab::OnDrawItem: TCITEM.dwState=[%s] dwStateMask=%i "), (LPCTSTR)strTabItemState, item.dwStateMask);
      /**/
      bool bPushed   = !!(item.dwState & TCIS_BUTTONPRESSED);
      bool bHot      = !!(item.dwState & TCIS_HIGHLIGHTED);
      bool bSelected = !!(di.itemState & ODS_SELECTED);// (di.itemID == TabCtrl_GetCurSel(m_hWndTab)); //
      bool bDisabled = !::IsWindowEnabled(m_hWndTab);
      bool bFocused  = !!(di.itemAction & ODA_FOCUS);//0;//bSelected && (m_hWndTab == ::GetFocus());//(di.itemID == TabCtrl_GetCurFocus(m_hWndTab)); // 
      RECTEX rcItem2; TabCtrl_GetItemRect(m_hWndTab, di.itemID, &rcItem2);
      /** /
      if (di.itemID == 2) {
         LOG_PUT(g_pLog, Put, LL_DEBUG) _T(" Tab::OnDrawItem: rcItem2=[%i, %i, w%i, h%i] "), rcItem2.left, rcItem2.top, rcItem2.width(), rcItem2.height());
         LOG_PUT(g_pLog, Put, LL_DEBUG) _T(" Tab::OnDrawItem: rcItem =[%i, %i, w%i, h%i] "), rcItem .left, rcItem .top, rcItem .width(), rcItem .height());
      }
      /**/
      //if (bSelected)
      {
         rcItem = rcItem2;
      }

      RECTEX rect(rcItem.size());

      HDC hCDC = ::CreateCompatibleDC(di.hDC);
      HBITMAP hBmpNew  = ::CreateCompatibleBitmap(di.hDC, rect.width(), rect.height());
      HGDIOBJ hBmpOld  = ::SelectObject(hCDC, hBmpNew);

      //if (di.itemID == 0)
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T(" Tab::OnDrawItem: ID=%i, bPushed=%i, bHot=%i, bSelected=%i, bDisabled=%i, bFocused=%i "), di.itemID, bPushed, bHot, bSelected, bDisabled, bFocused);

      CWinXPTheme::PF_OpenThemeData pfOpenThemeData = CWinXPTheme::OpenThemeData();
      HTHEME hTheme = pfOpenThemeData ? pfOpenThemeData(m_hWndTab, L"Tab") : NULL;
      int iStateId = 0;

      // 1. Draw border
      if (hTheme) {
         if (bDisabled)
            iStateId = TIS_DISABLED;
         else if (bFocused)
            iStateId = TIS_FOCUSED;
         else if (bSelected)
            iStateId = TIS_SELECTED;
         else if (bHot)
            iStateId = TIS_HOT;
         else
            iStateId = TIS_NORMAL;
         CWinXPTheme::PF_DrawThemeBackground pfDrawThemeBackground = CWinXPTheme::DrawThemeBackground();
         if(pfDrawThemeBackground) {
            //DrawBkgrnd(hCDC, rect);
            pfDrawThemeBackground(hTheme, hCDC, TABP_TABITEM, iStateId, &rect, 0);
         }
      }

      // 2. Draw background
      if (hTheme) {
         CWinXPTheme::PF_GetThemeBackgroundContentRect pfGetThemeBackgroundContentRect = CWinXPTheme::GetThemeBackgroundContentRect();
         if(pfGetThemeBackgroundContentRect) {
            RECTEX rcBk;
            HRESULT hRes = pfGetThemeBackgroundContentRect(hTheme, hCDC, TABP_TABITEM, iStateId, &rect, &rcBk);
            if (SUCCEEDED(hRes)) {
               if (rcBk == rect) { // ???
                  rcBk.left  += 2;
                  rcBk.top   += 3 - !bSelected;
                  rcBk.right -= 2;
                  rcBk.bottom-= 0;
               }
               rect = rcBk;
               if (!DrawBkgrnd(di, hCDC, rect, hTheme)) {
                  CWinXPTheme::PF_DrawThemeParentBackground pfDrawThemeParentBackground = CWinXPTheme::DrawThemeParentBackground();
                  if (pfDrawThemeParentBackground)
                     pfDrawThemeParentBackground(m_hWndTab, hCDC, &rect);
               }
            }
         }
      } else {
         if (!DrawBkgrnd(di, hCDC, rect, hTheme)) {
            ::SelectObject(hCDC, ::GetSysColorBrush(COLOR_BTNFACE));
            ::PatBlt(hCDC, rect.left, rect.top, rect.width(), rect.height(), PATCOPY);
         }
      }

      // 3. Draw image
      if (!bSelected) {
         rect.left  += 1;
         rect.top   += 1;
         rect.right -= 1;
         rect.bottom-= 1;
      }
      if (!Draw(di, hCDC, rect))
         m_Img.Draw(hCDC, rect);

      ::BitBlt(di.hDC, rcItem, hCDC, SRCCOPY);

      ::SelectObject(hCDC, hBmpOld);
      ::DeleteObject(hBmpNew);
      ::DeleteDC(hCDC); hCDC = NULL;

      CWinXPTheme::PF_CloseThemeData pfCloseThemeData = CWinXPTheme::CloseThemeData();
      if (pfCloseThemeData && hTheme) {
         pfCloseThemeData(hTheme); hTheme = NULL;
      }
   }
}
